@@ -1,5 +1,16 @@
+import typing
+
+if typing.TYPE_CHECKING:
+    __version__ = "?"
+else:
+    try:
+        from .version import *
+    except ImportError:
+        __version__ = "?"
+
 from .async_s3_bucket import *
 from .sal_enums import *
+from .type_hints import *
 from .base import *
 from .dds_utils import *
 from .idl_metadata import *
@@ -14,16 +25,10 @@ from .remote import *
 from .base_script import *
 from .csc_utils import *
 from .testutils import *
+from .config_schema import *
 from .testcsc import *
 from .testscript import *
 from .base_csc_test_case import *
 from .base_config_test_case import *
 from .csc_commander import *
-
-# Backwards compatibility
-from . import testutils as test_utils
-
-try:
-    from .version import *
-except ImportError:
-    __version__ = "?"
+from .testcsccommander import *
