@@ -1,6 +1,8 @@
+from __future__ import annotations
+
 # This file is part of ts_salobj.
 #
-# Developed for the LSST Telescope and Site Systems.
+# Developed for the Rubin Observatory Telescope and Site System.
 # This product includes software developed by the LSST Project
 # (https://www.lsst.org).
 # See the COPYRIGHT file at the top-level directory of this distribution
@@ -25,10 +27,13 @@ import argparse
 import asyncio
 import enum
 import sys
+import typing
 import warnings
 
+from lsst.ts import utils
 from . import base
 from . import dds_utils
+from . import type_hints
 from .sal_enums import State
 from .controller import Controller
 from .csc_utils import make_state_transition_dict
@@ -86,8 +91,16 @@ class BaseCsc(Controller):
         is required if ``--state`` is specified.
         Ignored unless ``enable_cmdline_state`` is True
         and the CSC is a subclass of `ConfigurableCsc`.
+    simulation_help : `str`
+        A *class* attribute.
+        Help for the --simulate command, or None for the default help
+        (which is fine for the usual case of 0/1).
+        Ignored if simulation is not supported.
     valid_simulation_modes : `list` [`int`] or `None`
-        A *class* attribute. If not `None` has the following effect:
+        A *class* attribute. A value of `None` is deprecated
+        (though it is the default, for backwards compatibility;
+        once `None` is prohibited there will be no default).
+        If not `None` has the following effect:
 
         * ``simulation_mode`` will be checked in the constructor
         * `implement_simulation_mode` will be a no-op.
@@ -95,6 +108,10 @@ class BaseCsc(Controller):
           The default value will be 0 if that is a valid simulation mode
           (and it certainly should be).
           Otherwise the default value will be the first entry.
+    version : `str`
+        A *class* attribute. Used to set the ``cscVersion`` attribute of the
+        ``softwareVersions`` event. You should almost always set this to
+        your package's ``__version__`` attribute.
     heartbeat_interval : `float`
         Interval between heartbeat events, in seconds;
 
@@ -133,19 +150,20 @@ class BaseCsc(Controller):
     """
 
     # See Attributes in the doc string.
-    default_initial_state = State.STANDBY
+    default_initial_state: State = State.STANDBY
     enable_cmdline_state = False
-    require_settings = False
-    valid_simulation_modes = None
+    require_settings: bool = False
+    valid_simulation_modes: typing.Optional[typing.Sequence[int]] = None
+    simulation_help: typing.Optional[str] = None
 
     def __init__(
         self,
-        name,
-        index=None,
-        initial_state=State.STANDBY,
-        settings_to_apply="",
-        simulation_mode=0,
-    ):
+        name: str,
+        index: typing.Optional[int] = None,
+        initial_state: State = State.STANDBY,
+        settings_to_apply: str = "",
+        simulation_mode: int = 0,
+    ) -> None:
         # Check for consistent require_settings and enable_cmdline_state.
         # Do that here, instead of in make_from_cmd_line, to  make sure
         # the developer sees the problem.
@@ -160,6 +178,7 @@ class BaseCsc(Controller):
         initial_state = State(initial_state)
         if initial_state == State.FAULT:
             raise ValueError("initial_state cannot be FAULT")
+
         if self.valid_simulation_modes is None:
             warnings.warn(
                 "valid_simulation_modes=None is deprecated", DeprecationWarning
@@ -171,19 +190,39 @@ class BaseCsc(Controller):
                     f"not in valid_simulation_modes={self.valid_simulation_modes}"
                 )
 
-        # Postpone assigning command callbacks until `start` is done (but call
-        # assert_do_methods_present to fail early if there is a problem).
-        super().__init__(name=name, index=index, do_callbacks=True)
-        self._requested_simulation_mode = int(simulation_mode)
         self._settings_to_apply = settings_to_apply
         self._summary_state = State(self.default_initial_state)
         self._initial_state = initial_state
         self._faulting = False
-        self._heartbeat_task = base.make_done_future()
+        self._heartbeat_task = utils.make_done_future()
         # Interval between heartbeat events (sec)
         self.heartbeat_interval = HEARTBEAT_INTERVAL
 
-    async def start(self):
+        if not hasattr(self, "version"):
+            warnings.warn(
+                "Please set class attribute `version`. It is needed to set "
+                "the `cscVersion` field of the `softwareVersions` event.",
+                DeprecationWarning,
+            )
+
+        # Postpone assigning command callbacks until `start` is done (but call
+        # assert_do_methods_present to fail early if there is a problem).
+        super().__init__(name=name, index=index, do_callbacks=True)
+
+        # Set evt_simulationMode, now that it is available.
+        self.evt_simulationMode.set(mode=int(simulation_mode))  # type: ignore
+
+        def format_version(version: typing.Optional[str]) -> str:
+            return "?" if version is None else version
+
+        self.evt_softwareVersions.set(  # type: ignore
+            salVersion=format_version(self.salinfo.metadata.sal_version),
+            xmlVersion=format_version(self.salinfo.metadata.xml_version),
+            openSpliceVersion=dds_utils.get_dds_version(),
+            cscVersion=getattr(self, "version", "?"),
+        )
+
+    async def start(self) -> None:
         """Finish constructing the CSC.
 
         * Call `set_simulation_mode`. If this fails, set ``self.start_task``
@@ -194,7 +233,7 @@ class BaseCsc(Controller):
         await super().start()
         self._heartbeat_task.cancel()  # Paranoia
         self._heartbeat_task = asyncio.create_task(self._heartbeat_loop())
-        await self.set_simulation_mode(self._requested_simulation_mode)
+        await self.set_simulation_mode(self.simulation_mode)
 
         # Handle initial state, then transition to the desired state.
         # If this fails then log the exception and continue,
@@ -222,37 +261,40 @@ class BaseCsc(Controller):
                         data.settingsToApply = self._settings_to_apply
                     self.log.info(f"Executing {command} command during startup")
                     await method(data)
+                    # Wait briefly, to be sure the new summaryState event
+                    # will have a different timestamp than the previous one.
+                    await asyncio.sleep(0.001)
             except Exception:
                 self.log.exception(
                     f"Failed in start on state transition command {command}; continuing."
                 )
 
-        def format_version(version):
-            return "?" if version is None else version
+        self.evt_softwareVersions.put()  # type: ignore
 
-        self.evt_softwareVersions.set_put(
-            salVersion=format_version(self.salinfo.metadata.sal_version),
-            xmlVersion=format_version(self.salinfo.metadata.xml_version),
-            openSpliceVersion=dds_utils.get_dds_version(),
-            cscVersion=getattr(self, "version", None),
-        )
-
-    async def close_tasks(self):
+    async def close_tasks(self) -> None:
         """Shut down pending tasks. Called by `close`."""
         self._heartbeat_task.cancel()
 
     @classmethod
-    def make_from_cmd_line(cls, index, **kwargs):
+    def make_from_cmd_line(
+        cls, index: typing.Union[int, enum.IntEnum, bool, None], **kwargs: typing.Any
+    ) -> BaseCsc:
         """Construct a CSC from command line arguments.
 
         Parameters
         ----------
-        index : `int`, `enum.IntEnum`, `True`, `False` or `None`
-            If the CSC is indexed: specify `True` make ``index`` a required
-            command-line argument that accepts any index,
-            or an enum.IntEnum class to make ``index`` a required
-            command-line argument that only accepts the enum values.
-            If the CSC is not indexed specify `None` or 0.
+        index : `int`, `enum.IntEnum`, `True`, or `None`
+            If the CSC is indexed, do one of the following:
+
+            * Specify `True` to make ``index`` a required
+              command-line argument that accepts any nonzero index.
+            * Specify an `enum.IntEnum` *class* to make ``index`` a required
+              command-line argument that only accepts the enum values.
+            * Specify a non-zero integer to use that index.
+              This is rare; if the CSC is indexed then the user
+              should usually be allowed to specify the index.
+
+            If the CSC is not indexed, specify `None` or 0.
         **kwargs : `dict`, optional
             Additional keyword arguments for your CSC's constructor.
 
@@ -272,9 +314,9 @@ class BaseCsc(Controller):
         elif isinstance(index, type) and issubclass(index, enum.IntEnum):
             # The isinstance check just above prevents errors
             # when index is an int or bool.
-            choices = [int(item.value) for item in index]
+            choices = [int(item.value) for item in index]  # type: ignore
             names_str = ", ".join(
-                f"{item.value}: {item.name.lower()}" for item in index
+                f"{item.value}: {item.name.lower()}" for item in index  # type: ignore
             )
             help_text = f"SAL index, one of: {names_str}"
             parser.add_argument("index", type=int, help=help_text, choices=choices)
@@ -295,28 +337,40 @@ class BaseCsc(Controller):
             and len(cls.valid_simulation_modes) > 1
         )
         if add_simulate_arg:
+            assert cls.valid_simulation_modes is not None  # make mypy happy
             if 0 in cls.valid_simulation_modes:
-                default_simulation_mode = 0
+                default_simulation_mode: int = 0
             else:
                 default_simulation_mode = cls.valid_simulation_modes[0]
             if default_simulation_mode == 0 and len(cls.valid_simulation_modes) == 2:
-                # There are only two simulation modes, one of which is 0:
-                # make --simulate a flag that takes no value.
+                # There are only two simulation modes, one of which is 0.
+                # Make --simulate a flag that takes no value and stores
+                # the other value, if specified.
+                simulation_help = (
+                    "Run in simulation mode?"
+                    if cls.simulation_help is None
+                    else cls.simulation_help
+                )
                 nonzero_value = (set(cls.valid_simulation_modes) - set([0])).pop()
                 parser.add_argument(
                     "--simulate",
-                    help="Run in simulation mode?",
+                    help=simulation_help,
                     default=0,
                     action="store_const",
                     const=nonzero_value,
                 )
             else:
-                # There are more than 2 simulation modes or none of them is 0:
-                # make --simulate an argument that requires a value.
+                # There are more than 2 simulation modes or none of them is 0.
+                # Make --simulate an argument that requires a value.
+                simulation_help = (
+                    "Simulation mode"
+                    if cls.simulation_help is None
+                    else cls.simulation_help
+                )
                 parser.add_argument(
                     "--simulate",
                     type=int,
-                    help="Simulation mode",
+                    help=simulation_help,
                     default=default_simulation_mode,
                     choices=cls.valid_simulation_modes,
                 )
@@ -362,17 +416,25 @@ class BaseCsc(Controller):
         return csc
 
     @classmethod
-    async def amain(cls, index, **kwargs):
+    async def amain(
+        cls, index: typing.Union[int, enum.IntEnum, bool, None], **kwargs: typing.Any
+    ) -> None:
         """Make a CSC from command-line arguments and run it.
 
         Parameters
         ----------
-        index : `int`, `enum.IntEnum`, `True`, `False` or `None`
-            If the CSC is indexed: specify `True` make ``index`` a required
-            command-line argument that accepts any index,
-            or an enum.IntEnum class to make ``index`` a required
-            command-line argument that only accepts the enum values.
-            If the CSC is not indexed specify `None` or 0.
+        index : `int`, `enum.IntEnum`, `True`, or `None`
+            If the CSC is indexed, do one of the following:
+
+            * Specify `True` to make ``index`` a required
+              command-line argument that accepts any nonzero index.
+            * Specify an `enum.IntEnum` *class* to make ``index`` a required
+              command-line argument that only accepts the enum values.
+            * Specify a non-zero integer to use that index.
+              This is rare; if the CSC is indexed then the user
+              should usually be allowed to specify the index.
+
+            If the CSC is not indexed, specify `None` or 0.
         **kwargs : `dict`, optional
             Additional keyword arguments for your CSC's constructor.
         """
@@ -380,7 +442,7 @@ class BaseCsc(Controller):
         await csc.done_task
 
     @classmethod
-    def add_arguments(cls, parser):
+    def add_arguments(cls, parser: argparse.ArgumentParser) -> None:
         """Add arguments to the parser created by `make_from_cmd_line`.
 
         Parameters
@@ -396,12 +458,14 @@ class BaseCsc(Controller):
         pass
 
     @classmethod
-    def add_kwargs_from_args(cls, args, kwargs):
+    def add_kwargs_from_args(
+        cls, args: argparse.Namespace, kwargs: typing.Dict[str, typing.Any]
+    ) -> None:
         """Add constructor keyword arguments based on parsed arguments.
 
         Parameters
         ----------
-        args : `argparse.namespace`
+        args : `argparse.Namespace`
             Parsed command.
         kwargs : `dict`
             Keyword argument dict for the constructor.
@@ -415,7 +479,7 @@ class BaseCsc(Controller):
         """
         pass
 
-    async def do_disable(self, data):
+    async def do_disable(self, data: type_hints.BaseDdsDataType) -> None:
         """Transition from `State.ENABLED` to `State.DISABLED`.
 
         Parameters
@@ -425,7 +489,7 @@ class BaseCsc(Controller):
         """
         await self._do_change_state(data, "disable", [State.ENABLED], State.DISABLED)
 
-    async def do_enable(self, data):
+    async def do_enable(self, data: type_hints.BaseDdsDataType) -> None:
         """Transition from `State.DISABLED` to `State.ENABLED`.
 
         Parameters
@@ -435,7 +499,7 @@ class BaseCsc(Controller):
         """
         await self._do_change_state(data, "enable", [State.DISABLED], State.ENABLED)
 
-    async def do_exitControl(self, data):
+    async def do_exitControl(self, data: type_hints.BaseDdsDataType) -> None:
         """Transition from `State.STANDBY` to `State.OFFLINE` and quit.
 
         Parameters
@@ -447,7 +511,7 @@ class BaseCsc(Controller):
 
         asyncio.create_task(self.close())
 
-    async def do_standby(self, data):
+    async def do_standby(self, data: type_hints.BaseDdsDataType) -> None:
         """Transition from `State.DISABLED` or `State.FAULT` to
         `State.STANDBY`.
 
@@ -460,7 +524,7 @@ class BaseCsc(Controller):
             data, "standby", [State.DISABLED, State.FAULT], State.STANDBY
         )
 
-    async def do_start(self, data):
+    async def do_start(self, data: type_hints.BaseDdsDataType) -> None:
         """Transition from `State.STANDBY` to `State.DISABLED`.
 
         Parameters
@@ -471,7 +535,7 @@ class BaseCsc(Controller):
         await self._do_change_state(data, "start", [State.STANDBY], State.DISABLED)
 
     @property
-    def simulation_mode(self):
+    def simulation_mode(self) -> int:
         """Get the current simulation mode.
 
         0 means normal operation (no simulation).
@@ -482,9 +546,9 @@ class BaseCsc(Controller):
             If the new simulation mode is not a supported value.
 
         """
-        return self.evt_simulationMode.data.mode
+        return self.evt_simulationMode.data.mode  # type: ignore
 
-    async def set_simulation_mode(self, simulation_mode):
+    async def set_simulation_mode(self, simulation_mode: int) -> None:
         """Set the simulation mode.
 
         Await implement_simulation_mode, update the simulation mode
@@ -497,9 +561,9 @@ class BaseCsc(Controller):
         """
         await self.implement_simulation_mode(simulation_mode)
 
-        self.evt_simulationMode.set_put(mode=simulation_mode, force_output=True)
+        self.evt_simulationMode.set_put(mode=simulation_mode, force_output=True)  # type: ignore
 
-    async def implement_simulation_mode(self, simulation_mode):
+    async def implement_simulation_mode(self, simulation_mode: int) -> None:
         """Implement going into or out of simulation mode.
 
         Deprecated. See :ref:`simulation mode<lsst.ts.salobj-simulation_mode>`
@@ -524,7 +588,7 @@ class BaseCsc(Controller):
                 f"This CSC does not support simulation; simulation_mode={simulation_mode} but must be 0"
             )
 
-    async def begin_disable(self, data):
+    async def begin_disable(self, data: type_hints.BaseDdsDataType) -> None:
         """Begin do_disable; called before state changes.
 
         Parameters
@@ -534,7 +598,7 @@ class BaseCsc(Controller):
         """
         pass
 
-    async def begin_enable(self, data):
+    async def begin_enable(self, data: type_hints.BaseDdsDataType) -> None:
         """Begin do_enable; called before state changes.
 
         Parameters
@@ -544,7 +608,7 @@ class BaseCsc(Controller):
         """
         pass
 
-    async def begin_exitControl(self, data):
+    async def begin_exitControl(self, data: type_hints.BaseDdsDataType) -> None:
         """Begin do_exitControl; called before state changes.
 
         Parameters
@@ -554,7 +618,7 @@ class BaseCsc(Controller):
         """
         pass
 
-    async def begin_standby(self, data):
+    async def begin_standby(self, data: type_hints.BaseDdsDataType) -> None:
         """Begin do_standby; called before the state changes.
 
         Parameters
@@ -564,7 +628,7 @@ class BaseCsc(Controller):
         """
         pass
 
-    async def begin_start(self, data):
+    async def begin_start(self, data: type_hints.BaseDdsDataType) -> None:
         """Begin do_start; called before state changes.
 
         Parameters
@@ -574,7 +638,7 @@ class BaseCsc(Controller):
         """
         pass
 
-    async def end_disable(self, data):
+    async def end_disable(self, data: type_hints.BaseDdsDataType) -> None:
         """End do_disable; called after state changes
         but before command acknowledged.
 
@@ -585,7 +649,7 @@ class BaseCsc(Controller):
         """
         pass
 
-    async def end_enable(self, data):
+    async def end_enable(self, data: type_hints.BaseDdsDataType) -> None:
         """End do_enable; called after state changes
         but before command acknowledged.
 
@@ -596,7 +660,7 @@ class BaseCsc(Controller):
         """
         pass
 
-    async def end_exitControl(self, data):
+    async def end_exitControl(self, data: type_hints.BaseDdsDataType) -> None:
         """End do_exitControl; called after state changes
         but before command acknowledged.
 
@@ -607,7 +671,7 @@ class BaseCsc(Controller):
         """
         pass
 
-    async def end_standby(self, data):
+    async def end_standby(self, data: type_hints.BaseDdsDataType) -> None:
         """End do_standby; called after state changes
         but before command acknowledged.
 
@@ -618,7 +682,7 @@ class BaseCsc(Controller):
         """
         pass
 
-    async def end_start(self, data):
+    async def end_start(self, data: type_hints.BaseDdsDataType) -> None:
         """End do_start; called after state changes
         but before command acknowledged.
 
@@ -629,7 +693,9 @@ class BaseCsc(Controller):
         """
         pass
 
-    def fault(self, code, report, traceback=""):
+    def fault(
+        self, code: typing.Optional[int], report: str, traceback: str = ""
+    ) -> None:
         """Enter the fault state and output the ``errorCode`` event.
 
         Parameters
@@ -650,17 +716,18 @@ class BaseCsc(Controller):
         try:
             self._faulting = True
             self._summary_state = State.FAULT
-            try:
-                self.evt_errorCode.set_put(
-                    errorCode=code,
-                    errorReport=report,
-                    traceback=traceback,
-                    force_output=True,
-                )
-            except Exception:
-                self.log.exception(
-                    f"Failed to output errorCode: code={code!r}; report={report!r}"
-                )
+            if code is not None:
+                try:
+                    self.evt_errorCode.set_put(  # type: ignore
+                        errorCode=code,
+                        errorReport=report,
+                        traceback=traceback,
+                        force_output=True,
+                    )
+                except Exception:
+                    self.log.exception(
+                        f"Failed to output errorCode: code={code!r}; report={report!r}"
+                    )
             try:
                 self.report_summary_state()
             except Exception:
@@ -668,12 +735,14 @@ class BaseCsc(Controller):
                     "report_summary_state failed while going to FAULT; "
                     "some code may not have run."
                 )
-                self.evt_summaryState.set_put(summaryState=self._summary_state)
+                self.evt_summaryState.set_put(  # type: ignore
+                    summaryState=self._summary_state,
+                )  # type: ignore
             asyncio.create_task(self.handle_summary_state())
         finally:
             self._faulting = False
 
-    def assert_enabled(self, action=""):
+    def assert_enabled(self, action: str = "") -> None:
         """Assert that an action that requires ENABLED state can be run.
 
         Parameters
@@ -685,14 +754,20 @@ class BaseCsc(Controller):
         """
         if self.summary_state != State.ENABLED:
             what = f"{action} not allowed" if action else "Not allowed"
+            errorCode = self.evt_errorCode.data.errorCode  # type: ignore
+            errorReport = self.evt_errorCode.data.errorReport  # type: ignore
             if self.summary_state == State.FAULT:
-                why = f"in Fault state; errorCode.errorReport={self.evt_errorCode.data.errorReport}"
+                why = (
+                    "in Fault state: "
+                    f"errorCode={errorCode}, "  # type:ignore
+                    f"errorReport={errorReport!r}"  # type: ignore
+                )
             else:
                 why = f"in state={self.summary_state!r}"
             raise base.ExpectedError(f"{what} {why}")
 
     @property
-    def disabled_or_enabled(self):
+    def disabled_or_enabled(self) -> bool:
         """Return True if the summary state is `State.DISABLED` or
         `State.ENABLED`.
 
@@ -703,12 +778,11 @@ class BaseCsc(Controller):
         return self.summary_state in (State.DISABLED, State.ENABLED)
 
     @property
-    def summary_state(self):
-        """Get the summary state as a `State` enum.
-        """
+    def summary_state(self) -> State:
+        """Get the summary state as a `State` enum."""
         return self._summary_state
 
-    async def handle_summary_state(self):
+    async def handle_summary_state(self) -> None:
         """Called when the summary state has changed.
 
         Override to perform tasks such as starting and stopping telemetry
@@ -723,16 +797,22 @@ class BaseCsc(Controller):
         """
         pass
 
-    def report_summary_state(self):
+    def report_summary_state(self) -> None:
         """Report a new value for summary_state, including current state.
 
         Subclasses may wish to override for code that depends on
         the current state (rather than the state transition command
         that got it into that state).
         """
-        self.evt_summaryState.set_put(summaryState=self.summary_state)
+        self.evt_summaryState.set_put(summaryState=self.summary_state)  # type: ignore
 
-    async def _do_change_state(self, data, cmd_name, allowed_curr_states, new_state):
+    async def _do_change_state(
+        self,
+        data: typing.Any,
+        cmd_name: str,
+        allowed_curr_states: typing.Sequence[State],
+        new_state: State,
+    ) -> None:
         """Change to the desired state.
 
         Parameters
@@ -778,13 +858,12 @@ class BaseCsc(Controller):
         await self.handle_summary_state()
         self.report_summary_state()
 
-    async def _heartbeat_loop(self):
-        """Output heartbeat at regular intervals.
-        """
+    async def _heartbeat_loop(self) -> None:
+        """Output heartbeat at regular intervals."""
         while True:
             try:
                 await asyncio.sleep(self.heartbeat_interval)
-                self.evt_heartbeat.put()
+                self.evt_heartbeat.put()  # type: ignore
             except asyncio.CancelledError:
                 break
             except Exception as e:
