@@ -1,6 +1,6 @@
 # This file is part of ts_salobj.
 #
-# Developed for the LSST Telescope and Site Systems.
+# Developed for the Rubin Observatory Telescope and Site System.
 # This product includes software developed by the LSST Project
 # (https://www.lsst.org).
 # See the COPYRIGHT file at the top-level directory of this distribution
@@ -22,14 +22,18 @@
 __all__ = ["ConfigurableCsc"]
 
 import abc
+import argparse
 import os
 import pathlib
 import subprocess
 import types
+import typing
+import warnings
 
 import yaml
 
 from . import base
+from . import type_hints
 from .base_csc import BaseCsc, State
 from .validator import DefaultingValidator
 
@@ -46,31 +50,37 @@ class ConfigurableCsc(BaseCsc, abc.ABC):
         Name of SAL component.
     index : `int` or `None`
         SAL component index, or 0 or None if the component is not indexed.
-    schema_path : `str` or `pathlib.Path`
-        Path to a schema file used to validate configuration files
+    schema_path : `str`, `pathlib.Path` or `None`, optional
+        Path to a schema file used to validate configuration files.
+        This is deprecated; new code should specify ``config_schema`` instead.
         The recommended path is ``<package_root>/"schema"/f"{name}.yaml"``
         for example:
 
             schema_path = pathlib.Path(__file__).resolve().parents[4] \
                 / "schema" / f"{name}.yaml"
-    config_dir : `str` (optional)
+    config_schema : `dict` or None, optional
+        Configuration schema, as a dict in jsonschema format.
+        Exactly one of ``schema_path`` or ``config_schema`` must not be None.
+    config_dir : `str`, optional
         Directory of configuration files, or None for the standard
-        configuration directory (obtained from `get_default_config_dir`).
+        configuration directory (obtained from `_get_default_config_dir`).
         This is provided for unit testing.
-    initial_state : `State` or `int` (optional)
+    initial_state : `State` or `int`, optional
         The initial state of the CSC. This is provided for unit testing,
         as real CSCs should start up in `State.STANDBY`, the default.
-    simulation_mode : `int` (optional)
+    settings_to_apply : `str`, optional
+        Settings to apply if ``initial_state`` is `State.DISABLED`
+        or `State.ENABLED`.
+    simulation_mode : `int`, optional
         Simulation mode. The default is 0: do not simulate.
-    initial_simulation_mode : `int` (optional)
-        A deprecated synonym for ``simulation_mode``.
 
     Raises
     ------
     ValueError
-        If ``config_dir`` is not a directory,
-        ``initial_state`` is invalid, or
-        ``simulation_mode`` and ``initial_simulation_mode`` are both nonzero.
+        If ``config_dir`` is not a directory or ``initial_state`` is invalid.
+    ValueError
+        If ``schema_path`` and ``config_schema`` are both None,
+        or if neither is None.
     salobj.ExpectedError
         If ``simulation_mode`` is invalid.
         Note: you will only see this error if you await `start_task`.
@@ -122,20 +132,31 @@ class ConfigurableCsc(BaseCsc, abc.ABC):
 
     def __init__(
         self,
-        name,
-        index,
-        schema_path,
-        config_dir=None,
-        initial_state=State.STANDBY,
-        simulation_mode=0,
-        initial_simulation_mode=0,
-    ):
-
+        name: str,
+        index: typing.Optional[int],
+        schema_path: typing.Union[str, pathlib.Path, None] = None,
+        config_schema: typing.Optional[typing.Dict[str, typing.Any]] = None,
+        config_dir: typing.Union[str, pathlib.Path, None] = None,
+        initial_state: State = State.STANDBY,
+        settings_to_apply: str = "",
+        simulation_mode: int = 0,
+    ) -> None:
         try:
-            with open(schema_path, "r") as f:
-                schema = yaml.safe_load(f.read())
-            self.config_validator = DefaultingValidator(schema=schema)
-            title = schema["title"]
+            if (schema_path is None) == (config_schema is None):
+                raise ValueError(
+                    "Exactly one of schema_path or config_schema must not be None"
+                )
+            if schema_path is not None:
+                warnings.warn(
+                    "The schema_path argument is deprecated. "
+                    "Please specify config_schema instead.",
+                    DeprecationWarning,
+                )
+                with open(schema_path, "r") as f:
+                    config_schema = yaml.safe_load(f.read())
+            assert config_schema is not None  # Make mypy happy
+            self.config_validator = DefaultingValidator(schema=config_schema)
+            title = config_schema["title"]
             name_version = title.rsplit(" ", 1)
             if len(name_version) != 2 or not name_version[1].startswith("v"):
                 raise ValueError(f"Schema title {title!r} must end with ' v<version>'")
@@ -146,33 +167,33 @@ class ConfigurableCsc(BaseCsc, abc.ABC):
         except Exception as e:
             raise ValueError(f"Schema {schema_path} invalid") from e
 
-        super().__init__(
-            name=name,
-            index=index,
-            initial_state=initial_state,
-            simulation_mode=simulation_mode,
-            initial_simulation_mode=initial_simulation_mode,
-        )
-
         if config_dir is None:
-            config_dir = self._get_default_config_dir()
+            config_dir = self._get_default_config_dir(name)
         else:
             config_dir = pathlib.Path(config_dir)
 
         if not config_dir.is_dir():
             raise ValueError(
-                f"config_dir={config_dir} does not exists or is not a directory"
+                f"config_dir={config_dir} does not exist or is not a directory"
             )
 
         self.config_dir = config_dir
 
+        super().__init__(
+            name=name,
+            index=index,
+            initial_state=initial_state,
+            settings_to_apply=settings_to_apply,
+            simulation_mode=simulation_mode,
+        )
+
     @property
-    def config_dir(self):
+    def config_dir(self) -> pathlib.Path:
         """Get or set the configuration directory.
 
         Parameters
         ----------
-        config_dir : `str`, `bytes`, or `pathlib.Path`
+        config_dir : `str`, `pathlib.Path`
             New configuration directory.
 
         Returns
@@ -188,7 +209,7 @@ class ConfigurableCsc(BaseCsc, abc.ABC):
         return self._config_dir
 
     @config_dir.setter
-    def config_dir(self, config_dir):
+    def config_dir(self, config_dir: typing.Union[str, pathlib.Path]) -> None:
         config_dir = pathlib.Path(config_dir).resolve()
         if not config_dir.is_dir():
             raise ValueError(
@@ -196,7 +217,7 @@ class ConfigurableCsc(BaseCsc, abc.ABC):
             )
         self._config_dir = config_dir
 
-    def read_config_dir(self):
+    def read_config_dir(self) -> None:
         """Set ``self.config_label_dict`` and output ``evt_settingVersions``.
 
         Set ``self.config_label_dict`` from ``self.config_dir/_labels.yaml``.
@@ -224,13 +245,13 @@ class ConfigurableCsc(BaseCsc, abc.ABC):
 
         settings_version = self._get_settings_version()
 
-        self.evt_settingVersions.set_put(
+        self.evt_settingVersions.set_put(  # type: ignore
             recommendedSettingsLabels=",".join(labels),
             recommendedSettingsVersion=settings_version,
             settingsUrl=f"{self.config_dir.as_uri()}",
         )
 
-    async def start(self):
+    async def start(self) -> None:
         """Finish constructing the CSC.
 
         * If ``initial_summary_state`` is `State.DISABLED` or `State.ENABLED`
@@ -246,7 +267,7 @@ class ConfigurableCsc(BaseCsc, abc.ABC):
             await self.configure(config=default_config)
         await super().start()
 
-    def _get_settings_version(self):
+    def _get_settings_version(self) -> str:
         """Get data for evt_settingVersions.recommendedSettingsVersion.
 
         If config_dir is a git repository (as it should be)
@@ -263,7 +284,7 @@ class ConfigurableCsc(BaseCsc, abc.ABC):
             return ""
         return git_info.decode(errors="replace").strip()
 
-    def _make_config_label_dict(self):
+    def _make_config_label_dict(self) -> typing.Dict[str, str]:
         """Parse ``self.config_dir/_labels.yaml`` and return the dict.
 
         where the path is relative to ``self.config_dir`` directory.
@@ -321,7 +342,7 @@ class ConfigurableCsc(BaseCsc, abc.ABC):
             )
         return output_dict
 
-    def report_summary_state(self):
+    def report_summary_state(self) -> None:
         super().report_summary_state()
         if self.summary_state == State.STANDBY:
             try:
@@ -329,7 +350,7 @@ class ConfigurableCsc(BaseCsc, abc.ABC):
             except Exception as e:
                 self.log.exception(e)
 
-    async def begin_start(self, data):
+    async def begin_start(self, data: type_hints.BaseDdsDataType) -> None:
         """Begin do_start; configure the CSC before changing state.
 
         Parameters
@@ -350,7 +371,7 @@ class ConfigurableCsc(BaseCsc, abc.ABC):
         self.read_config_dir()
 
         # Read the configuration
-        config_name = data.settingsToApply
+        config_name = data.settingsToApply  # type: ignore
         config_file_path = ""
         if config_name:
             name_version = config_name.split(":")
@@ -362,7 +383,7 @@ class ConfigurableCsc(BaseCsc, abc.ABC):
                         f"config_file_path={config_file_path}; "
                         f"githash={githash}; config_dir={self.config_dir}"
                     )
-                    config_yaml = subprocess.check_output(
+                    config_yaml: typing.Union[str, bytes] = subprocess.check_output(
                         args=["git", "show", f"{githash}:./{config_file_name}"],
                         stderr=subprocess.STDOUT,
                         cwd=self.config_dir,
@@ -372,7 +393,9 @@ class ConfigurableCsc(BaseCsc, abc.ABC):
                         f"Could not read config {config_name}: {e.output}"
                     )
             elif len(name_version) == 1:
-                githash = self.evt_settingVersions.data.recommendedSettingsVersion
+                githash = (
+                    self.evt_settingVersions.data.recommendedSettingsVersion  # type: ignore
+                )
                 config_file_name = self.config_label_dict.get(config_name, config_name)
                 config_file_path = self.config_dir / config_file_name
                 try:
@@ -388,10 +411,15 @@ class ConfigurableCsc(BaseCsc, abc.ABC):
                 )
 
             user_config_dict = yaml.safe_load(config_yaml)
+            # Delete metadata, if present
+            if user_config_dict:
+                user_config_dict.pop("metadata", None)
         else:
             user_config_dict = {}
             config_file_name = ""
-            githash = self.evt_settingVersions.data.recommendedSettingsVersion
+            githash = (
+                self.evt_settingVersions.data.recommendedSettingsVersion  # type: ignore
+            )
         try:
             full_config_dict = self.config_validator.validate(user_config_dict)
         except Exception as e:
@@ -400,15 +428,15 @@ class ConfigurableCsc(BaseCsc, abc.ABC):
             )
         config = types.SimpleNamespace(**full_config_dict)
         await self.configure(config)
-        self.evt_settingsApplied.set_put(
+        self.evt_settingsApplied.set_put(  # type: ignore
             settingsVersion=f"{config_file_name}:{githash}"
         )
-        self.evt_appliedSettingsMatchStart.set_put(
+        self.evt_appliedSettingsMatchStart.set_put(  # type: ignore
             appliedSettingsMatchStartIsTrue=True, force_output=True
         )
 
     @abc.abstractmethod
-    async def configure(self, config):
+    async def configure(self, config: typing.Any) -> None:
         """Configure the CSC.
 
         Parameters
@@ -426,18 +454,22 @@ class ConfigurableCsc(BaseCsc, abc.ABC):
 
     @staticmethod
     @abc.abstractmethod
-    def get_config_pkg():
-        """Get the name of the configuration package, e.g. "ts_config_ocs".
-        """
+    def get_config_pkg() -> str:
+        """Get the name of the configuration package, e.g. "ts_config_ocs"."""
         raise NotImplementedError()
 
-    def _get_default_config_dir(self):
-        """Compute the default directory for configuration files.
+    def _get_default_config_dir(self, name: str) -> pathlib.Path:
+        """Compute the default package directory for configuration files.
+
+        Parameters
+        ----------
+        name : `str`
+            SAL component name.
 
         Returns
         -------
         config_dir : `pathlib.Path`
-            Default configuration directory.
+            Default package directory for configuration files.
 
         Raises
         ------
@@ -462,7 +494,7 @@ class ConfigurableCsc(BaseCsc, abc.ABC):
         config_pkg = self.get_config_pkg()
         config_env_var_name = f"{config_pkg.upper()}_DIR"
         try:
-            config_pkg_dir = os.environ[config_env_var_name]
+            config_pkg_dir: type_hints.PathType = os.environ[config_env_var_name]
         except KeyError:
             raise RuntimeError(
                 f"Environment variable {config_env_var_name} not defined"
@@ -474,7 +506,7 @@ class ConfigurableCsc(BaseCsc, abc.ABC):
                 "does not exists or is not a directory"
             )
 
-        config_dir = config_pkg_dir / self.salinfo.name / self.schema_version
+        config_dir = config_pkg_dir / name / self.schema_version
         if not config_dir.is_dir():
             raise RuntimeError(
                 f"{config_dir} = ${config_env_var_name}/SAL_component_name/schema_version "
@@ -483,12 +515,27 @@ class ConfigurableCsc(BaseCsc, abc.ABC):
         return config_dir
 
     @classmethod
-    def add_arguments(cls, parser):
+    def add_arguments(cls, parser: argparse.ArgumentParser) -> None:
         parser.add_argument(
             "--configdir",
-            help="Directory containing configuration files for the start command.",
+            help="directory containing configuration files for the start command.",
         )
+        if cls.enable_cmdline_state:
+            settings_help = "settings to apply if --state is disabled or enabled"
+            if cls.require_settings:
+                settings_help += "; required if --state is disabled or enabled"
+                settings_default = None
+            else:
+                settings_default = ""
+            parser.add_argument(
+                "--settings",
+                help=settings_help,
+                default=settings_default,
+                dest="settings_to_apply",
+            )
 
     @classmethod
-    def add_kwargs_from_args(cls, args, kwargs):
+    def add_kwargs_from_args(
+        cls, args: argparse.Namespace, kwargs: typing.Dict[str, typing.Any]
+    ) -> None:
         kwargs["config_dir"] = args.configdir
