@@ -1,6 +1,8 @@
+from __future__ import annotations
+
 # This file is part of ts_salobj.
 #
-# Developed for the LSST Telescope and Site Systems.
+# Developed for the Rubin Observatory Telescope and Site System.
 # This product includes software developed by the LSST Project
 # (https://www.lsst.org).
 # See the COPYRIGHT file at the top-level directory of this distribution
@@ -22,25 +24,53 @@
 __all__ = ["Controller", "OPTIONAL_COMMAND_NAMES"]
 
 import asyncio
+import types
+import typing
 
 from . import base
+from . import type_hints
 from .domain import Domain
 from .sal_info import SalInfo
-from .topics import ControllerEvent, ControllerTelemetry, ControllerCommand
 from .sal_log_handler import SalLogHandler
+from .topics import ControllerEvent, ControllerTelemetry, ControllerCommand
 
-# TODO DM-25126: Remove setAuthList once ts_salobj supports auth lists.
-# TODO DM-17157: Remove the non-setAuthList entries if and when
-# SALSubsystems.xml explicitly lists all generic topics used.
-OPTIONAL_COMMAND_NAMES = set(
-    ("abort", "enterControl", "setValue", "setSimulationMode", "setAuthList")
-)
+# Set of generic commands that need not have an associated do_ method.
+# TODO DM-17157: Remove this constant if and when ``SALSubsystems.xml``
+# explicitly lists all generic topics used by every SAL component.
+OPTIONAL_COMMAND_NAMES = set(("abort", "enterControl", "setValue", "setSimulationMode"))
 
 # Delay before closing the domain participant (seconds).
 # This gives remotes time to read final DDS messages before they disappear.
 SHUTDOWN_DELAY = 1
 
 
+def parse_as_prefix_and_set(items_str: str) -> typing.Tuple[str, typing.Set[str]]:
+    """Parse a string as an optional +/- prefix and a set of items.
+
+    Parameters
+    ----------
+    items_str : `str`
+        Data string formatted as an optional prefix of "+" or "-"
+        followed by a set of comma-separated items (each of which
+        must not contain a comma or space).
+        Whitespace is ignored after the optional prefix and after each comma.
+
+    Returns
+    -------
+    prefix_data : `List` [`str`, `set`]
+        A tuple of two items:
+        * The prefix: "+", "-", or "" (no prefix).
+        * The set of items. Each item is a `str` with no whitespace or comma.
+    """
+    prefix = ""
+    if items_str and items_str[0] in ("+", "-"):
+        prefix = items_str[0]
+        items_str = items_str[1:]
+    data_set = set([name.strip() for name in items_str.split(",")])
+    data_set -= set([""])
+    return prefix, data_set
+
+
 class Controller:
     """A class that receives commands for a SAL component
     and sends telemetry and events from that component.
@@ -53,10 +83,10 @@ class Controller:
     ----------
     name : `str`
         Name of SAL component.
-    index : `int` or `None` (optional)
+    index : `int` or `None`, optional
         SAL component index, or 0 or None if the component is not indexed.
         A value is required if the component is indexed.
-    do_callbacks : `bool` (optional)
+    do_callbacks : `bool`, optional
         Set ``do_<name>`` methods as callbacks for commands?
         If True then there must be exactly one ``do_<name>`` method
         for each command.
@@ -143,34 +173,36 @@ class Controller:
     * logMessage event
     """
 
-    def __init__(self, name, index=None, *, do_callbacks=False):
+    def __init__(
+        self,
+        name: str,
+        index: typing.Optional[int] = None,
+        *,
+        do_callbacks: bool = False,
+    ) -> None:
+        self.isopen = False
+
+        self.start_called = False
+        # Task that is set done when the controller is closed
+        self.done_task: asyncio.Future = asyncio.Future()
+        self._do_callbacks = do_callbacks
+
         domain = Domain()
         try:
             self.salinfo = SalInfo(domain=domain, name=name, index=index)
+            new_identity = self.salinfo.name_index
+            self.salinfo.identity = new_identity
+            domain.default_identity = new_identity
             self.log = self.salinfo.log
-            self.start_called = False
-            # Task that is set done when the controller is closed
-            self.done_task = asyncio.Future()
-            self.isopen = True
-            command_names = self.salinfo.command_names
+
             if do_callbacks:
-                self._assert_do_methods_present(command_names)
-            for cmd_name in command_names:
+                # This must be called after the cmd_ attributes
+                # have been added.
+                self._assert_do_methods_present()
+
+            for cmd_name in self.salinfo.command_names:
                 cmd = ControllerCommand(self.salinfo, cmd_name)
                 setattr(self, cmd.attr_name, cmd)
-                if do_callbacks:
-                    cmd_attr_name = f"do_{cmd_name}"
-                    func = getattr(self, cmd_attr_name, None)
-                    if func:
-                        cmd.callback = getattr(self, cmd_attr_name)
-                    elif cmd_name not in OPTIONAL_COMMAND_NAMES:
-                        raise RuntimeError(f"Can't find method {cmd_attr_name}")
-                    else:
-
-                        def reject_command(data):
-                            raise base.ExpectedError("Not supported by this CSC")
-
-                        cmd.callback = reject_command
 
             for evt_name in self.salinfo.event_names:
                 evt = ControllerEvent(self.salinfo, evt_name)
@@ -180,28 +212,40 @@ class Controller:
                 tel = ControllerTelemetry(self.salinfo, tel_name)
                 setattr(self, tel.attr_name, tel)
 
-            for required_name in ("logMessage", "logLevel"):
+            for required_name in ("logMessage", "logLevel", "authList"):
                 if not hasattr(self, f"evt_{required_name}"):
                     raise RuntimeError(f"{self!r} has no {required_name} event")
 
             self._sal_log_handler = SalLogHandler(controller=self)
             self.log.addHandler(self._sal_log_handler)
-            self.start_task = asyncio.ensure_future(self.start())
-            """This task is set done when the CSC is fully started.
 
-            If `start` fails then the task has an exception set
-            and the CSC is not usable.
-            """
+            # This task is set done when the CSC is fully started.
+            # If `start` fails then the task has an exception set
+            # and the CSC is not usable.
+            self.start_task = asyncio.create_task(self._protected_start())
 
         except Exception:
-            asyncio.ensure_future(domain.close())
+            # Note: Domain.basic_close closes all its SalInfo instances.
+            domain.basic_close()
             raise
+        self.isopen = True
 
-    async def start(self):
-        """Finish construction."""
+    async def _protected_start(self) -> None:
+        """Call `start` and handle exceptions."""
         if self.start_called:
             raise RuntimeError("Start already called")
         self.start_called = True
+        try:
+            await self.start()
+        except asyncio.CancelledError:
+            self.log.warning("start canceled")
+        except Exception as e:
+            self.log.exception("start failed")
+            await self.close(exception=e, cancel_start=False)
+            raise
+
+    async def start(self) -> None:
+        """Finish construction."""
 
         # Allow each remote constructor to begin running its start method.
         await asyncio.sleep(0)
@@ -216,13 +260,32 @@ class Controller:
             start_tasks.append(salinfo.start_task)
         await asyncio.gather(*start_tasks)
         await self.salinfo.start()
+
+        # Assign command callbacks; give up if this fails, since the CSC
+        # will be at least partly broken.
+        if self._do_callbacks:
+            try:
+                self._assign_cmd_callbacks()
+            except Exception:
+                self.log.exception(
+                    "Failed in start on _assign_cmd_callbacks; quitting."
+                )
+                await self.close()
+                return
+
         self.put_log_level()
+        self.evt_authList.set_put(  # type: ignore
+            authorizedUsers=", ".join(sorted(self.salinfo.authorized_users)),
+            nonAuthorizedCSCs=", ".join(sorted(self.salinfo.non_authorized_cscs)),
+        )
 
     @property
-    def domain(self):
+    def domain(self) -> Domain:
         return self.salinfo.domain
 
-    async def close(self, exception=None):
+    async def close(
+        self, exception: typing.Optional[Exception] = None, cancel_start: bool = True
+    ) -> None:
         """Shut down, clean up resources and set done_task done.
 
         May be called multiple times. The first call closes the Controller;
@@ -233,11 +296,14 @@ class Controller:
 
         Parameters
         ----------
-        exception : `Exception` (optional)
+        exception : `Exception`, optional
             The exception that caused stopping, if any, in which case
             the ``self.done_task`` exception is set to this value.
             Specify `None` for a normal exit, in which case
             the ``self.done_task`` result is set to `None`.
+        cancel_start : `bool`, optional
+            Cancel the start task? Leave this true unless calling
+            this from the start task.
 
         Notes
         -----
@@ -245,27 +311,42 @@ class Controller:
         all background tasks, pauses briefly to allow final SAL messages
         to be sent, then closes the dds domain.
         """
+        if not hasattr(self, "start_task"):
+            # Not fully constructed; nothing to do.
+            return
+
+        if cancel_start and not self.start_task.done():
+            self.start_task.cancel()
+
         if not self.isopen:
-            # Closed or closing. Wait for done_task to be finished,
+            # Closed or closing (we know this instance is fully constructed
+            # because we checked that start_task is exists).
+            # Wait for done_task to be finished,
             # ignoring any exception. If you want to know about the exception
             # you can examine done_task yourself.
             try:
+                self.log.info("Already closing; waiting for done_task")
                 await self.done_task
             except Exception:
                 pass
             return
+
         self.isopen = False
         try:
             await self.close_tasks()
         except Exception:
-            self.log.exception("Controller.close_tasks failed")
+            self.log.exception("Controller.close_tasks failed; close continues")
         try:
             # Give remotes time to read final DDS messages before closing
             # the domain participant.
-            await asyncio.sleep(SHUTDOWN_DELAY)
             self.log.removeHandler(self._sal_log_handler)
-            self._sal_log_handler = None
+            await asyncio.sleep(SHUTDOWN_DELAY)
             await self.domain.close()
+        except asyncio.CancelledError:
+            self.domain.basic_close()
+        except Exception:
+            self.domain.basic_close()
+            self.log.exception("Controller.close failed near the end; close continues")
         finally:
             if not self.done_task.done():
                 if exception:
@@ -273,7 +354,7 @@ class Controller:
                 else:
                     self.done_task.set_result(None)
 
-    async def close_tasks(self):
+    async def close_tasks(self) -> None:
         """Shut down pending tasks. Called by `close`.
 
         Perform all cleanup other than disabling logging to SAL
@@ -281,7 +362,61 @@ class Controller:
         """
         pass
 
-    def do_setLogLevel(self, data):
+    def do_setAuthList(self, data: type_hints.BaseDdsDataType) -> None:
+        """Update the authorization list.
+
+        Parameters
+        ----------
+        data : ``cmd_setAuthList.DataType``
+            Authorization lists.
+
+        Notes
+        -----
+        Add items if the data string starts with "+", ignoring duplicates
+        (both with respect to the existing items and within the data string).
+        Remove items if the data string starts with "-", ignoring missing
+        items (items specified for removal that do not exist).
+        Ignore whitespace after each comma and after the +/- prefix.
+        """
+        users_prefix, users_set = parse_as_prefix_and_set(
+            data.authorizedUsers,  # type: ignore
+        )
+        # Remove "me" from users list.
+        users_set.discard(self.salinfo.domain.user_host)
+
+        cscs_prefix, cscs_set = parse_as_prefix_and_set(
+            data.nonAuthorizedCSCs,  # type: ignore
+        )
+        # Strip :0 suffix, if present, for consistency.
+        cscs_set = {csc[:-2] if csc.endswith(":0") else csc for csc in cscs_set}
+
+        if users_prefix == "+":
+            # + prefix: add users.
+            self.salinfo.authorized_users |= users_set
+        elif users_prefix == "-":
+            # - prefix: remove users.
+            self.salinfo.authorized_users -= users_set
+        else:
+            # No prefix: replace users.
+            self.salinfo.authorized_users = users_set
+
+        if cscs_prefix == "+":
+            # + prefix: add CSCs.
+            self.salinfo.non_authorized_cscs |= cscs_set
+        elif cscs_prefix == "-":
+            # - prefix: remove CSCs.
+            self.salinfo.non_authorized_cscs -= cscs_set
+        else:
+            # No prefix: replace CSCs.
+            self.salinfo.non_authorized_cscs = cscs_set
+
+        self.evt_authList.set_put(  # type: ignore
+            authorizedUsers=", ".join(sorted(self.salinfo.authorized_users)),
+            nonAuthorizedCSCs=", ".join(sorted(self.salinfo.non_authorized_cscs)),
+            force_output=True,
+        )
+
+    def do_setLogLevel(self, data: type_hints.BaseDdsDataType) -> None:
         """Set logging level.
 
         Parameters
@@ -289,24 +424,16 @@ class Controller:
         data : ``cmd_setLogLevel.DataType``
             Logging level.
         """
-        self.log.setLevel(data.level)
+        self.log.setLevel(data.level)  # type: ignore
         self.put_log_level()
 
-    def put_log_level(self):
-        """Output the logLevel event.
-        """
-        self.evt_logLevel.set_put(level=self.log.getEffectiveLevel(), force_output=True)
+    def put_log_level(self) -> None:
+        """Output the logLevel event."""
+        self.evt_logLevel.set_put(level=self.log.getEffectiveLevel(), force_output=True)  # type: ignore
 
-    def _assert_do_methods_present(self, command_names):
-        """Assert that all needed do_<name> methods are present,
-        and no extra such methods are present.
-
-        Parameters
-        ----------
-        command_names : `list` of `str`
-            List of command names, e.g. as provided by
-            `salinfo.command_names`
-        """
+    def _assert_do_methods_present(self) -> None:
+        """Assert that all needed do_<name> methods are present."""
+        command_names = self.salinfo.command_names
         do_names = [name for name in dir(self) if name.startswith("do_")]
         supported_command_names = [name[3:] for name in do_names]
         if set(command_names) != set(supported_command_names):
@@ -332,23 +459,35 @@ class Controller:
             err_msg = " and ".join(err_msgs)
             raise TypeError(f"This class {err_msg}")
 
-    async def __aenter__(self):
+    def _assign_cmd_callbacks(self) -> None:
+        """Assign each do_ method as a callback to the appropriate command.
+
+        Must not be called until the command attributes have been added.
+        """
+        command_names = self.salinfo.command_names
+        for cmd_name in command_names:
+            cmd = getattr(self, f"cmd_{cmd_name}")
+            do_method_name = f"do_{cmd_name}"
+            func = getattr(self, do_method_name, None)
+            if func is not None:
+                cmd.callback = func
+            elif cmd_name not in OPTIONAL_COMMAND_NAMES:
+                raise RuntimeError(f"Can't find method {do_method_name}")
+            else:
+
+                def reject_command(data: typing.Any) -> None:
+                    raise base.ExpectedError("Not supported by this CSC")
+
+                cmd.callback = reject_command
+
+    async def __aenter__(self) -> Controller:
         await self.start_task
         return self
 
-    async def __aexit__(self, type, value, traceback):
+    async def __aexit__(
+        self,
+        type: typing.Optional[typing.Type[BaseException]],
+        value: typing.Optional[BaseException],
+        traceback: typing.Optional[types.TracebackType],
+    ) -> None:
         await self.close()
-
-    def __del__(self):
-        """Last-ditch effort to clean up critical resources.
-
-        Users should call `close` instead, because it does more
-        and because ``__del__`` is not reliably called.
-        """
-        handler = getattr(self, "_sal_log_handler", None)
-        log = getattr(self, "log", None)
-        if None not in (handler, log):
-            log.removeHandler(handler)
-        domain = getattr(self, "domain", None)
-        if domain is not None:
-            domain.close_dds()
