@@ -1,6 +1,8 @@
+from __future__ import annotations
+
 # This file is part of ts_salobj.
 #
-# Developed for the LSST Telescope and Site Systems.
+# Developed for the Rubin Observatory Telescope and Site System.
 # This product includes software developed by the LSST Project
 # (https://www.lsst.org).
 # See the COPYRIGHT file at the top-level directory of this distribution
@@ -19,21 +21,27 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
-__all__ = ["FieldMetadata", "TopicMetadata", "SalInfo", "MAX_RESULT_LEN"]
+__all__ = ["SalInfo", "MAX_RESULT_LEN"]
 
 import asyncio
 import atexit
 import concurrent
+import enum
 import logging
 import os
 import time
+import types
+import typing
 import warnings
 
 import dds
-import ddsutil
 
-from . import base
+from lsst.ts import utils
+from . import dds_utils
 from . import idl_metadata
+from . import sal_enums
+from . import topics
+from . import type_hints
 from .domain import Domain
 
 # Maximum length of the ``result`` field in an ``ackcmd`` topic.
@@ -57,77 +65,6 @@ MAX_HISTORY_READ = 10000
 DEFAULT_LSST_DDS_HISTORYSYNC = 60
 
 
-class FieldMetadata:
-    """Information about a field.
-
-    Parameters
-    ----------
-    name : `str`
-        Field name.
-    description : `str` or `None`
-        Description; `None` if not specified.
-    units : `str` or `None`
-        Units; `None` if not specified.
-    type_name : `str`
-        Data type name from the IDL file, e.g.
-        "string<8>", "float", "double".
-        This may not match the Python data type of the field.
-        For instance dds maps most integer types to `int`,
-        and both "float" and "double" to `double`.
-    array_length : `int`
-        Number of elements if an array; None if not an array.
-    str_length : `int`
-        Maximum allowed string length; None if unspecified (no limit)
-        or not a string.
-    """
-
-    def __init__(self, name, description, units, type_name, array_length, str_length):
-        self.name = name
-        self.description = description
-        self.units = units
-        self.type_name = type_name
-        self.array_length = array_length
-        self.str_length = str_length
-
-    def __repr__(self):
-        return (
-            f"FieldMetadata(name={repr(self.name)}, "
-            f"description={repr(self.description)}, "
-            f"units={repr(self.units)}, "
-            f"type_name={repr(self.type_name)},"
-            f"array_length={self.array_length}"
-            f"str_length={self.str_length})"
-        )
-
-    def __str__(self):
-        return f"description={repr(self.description)}, units={repr(self.units)}"
-
-
-class TopicMetadata:
-    """Metadata about a topic.
-
-    Parameters
-    ----------
-    sal_name : `str`
-        SAL topic name, e.g. `logevent_summaryState`.
-    description : `str` or `None`
-        Topic description, or `None` if unknown.
-
-    Attributes
-    ----------
-    field_info : `dict` [`str`, `FieldMetadata`]
-        Dict of field name: field metadata.
-    """
-
-    def __init__(self, sal_name, description):
-        self.sal_name = sal_name
-        self.description = description
-        self.field_info = dict()
-
-    def __repr__(self):
-        return f"TopicMetadata(sal_name={repr(self.sal_name)}, description={self.description})"
-
-
 class SalInfo:
     r"""DDS information for one SAL component and its DDS partition
 
@@ -147,7 +84,8 @@ class SalInfo:
     RuntimeError
         If the IDL file cannot be found for the specified ``name``.
     TypeError
-        If ``domain`` is not a `Domain`.
+        If ``domain`` is not an instance of `Domain`
+        or if ``index`` is not an `int`, `enum.IntEnum`, or `None`.
     ValueError
         If ``index`` is nonzero and the component is not indexed.
 
@@ -237,20 +175,27 @@ class SalInfo:
     events, telemetry, and the ``ackcmd`` topic.
     """
 
-    def __init__(self, domain, name, index=0):
+    def __init__(
+        self, domain: Domain, name: str, index: typing.Optional[int] = 0
+    ) -> None:
         if not isinstance(domain, Domain):
             raise TypeError(f"domain {domain!r} must be an lsst.ts.salobj.Domain")
-        self.isopen = True
+        if index is not None:
+            if not (isinstance(index, int) or isinstance(index, enum.IntEnum)):
+                raise TypeError(
+                    f"index {index!r} must be an integer, enum.IntEnum, or None"
+                )
+        self.isopen = False
         self.domain = domain
         self.name = name
-        self.index = 0 if index is None else int(index)
+        self.index = 0 if index is None else index
         self.identity = domain.default_identity
         self.start_called = False
 
         # Dict of SAL topic name: wait_for_historical_data succeeded
         # for each topic for which wait_for_historical_data was called.
         # This is primarily intended for unit tests.
-        self.wait_history_isok = dict()
+        self.wait_history_isok: typing.Dict[str, bool] = dict()
 
         partition_prefix = os.environ.get("LSST_DDS_PARTITION_PREFIX")
         if partition_prefix is None:
@@ -273,15 +218,15 @@ class SalInfo:
             )
         self.partition_prefix = partition_prefix
 
-        self.start_task = asyncio.Future()
-        self.done_task = asyncio.Future()
+        self.start_task: asyncio.Future = asyncio.Future()
+        self.done_task: asyncio.Future = asyncio.Future()
 
         self.log = logging.getLogger(self.name)
         if self.log.getEffectiveLevel() > MAX_LOG_LEVEL:
             self.log.setLevel(MAX_LOG_LEVEL)
 
-        self.authorized_users = set()
-        self.non_authorized_cscs = set()
+        self.authorized_users: typing.Set[str] = set()
+        self.non_authorized_cscs: typing.Set[str] = set()
 
         # Publishers and subscribers.
         # Create at need to avoid unnecessary instances.
@@ -293,31 +238,35 @@ class SalInfo:
         self._data_subscriber = None
 
         # dict of private_seqNum: salobj.topics.CommandInfo
-        self._running_cmds = dict()
-        # dict of dds.ReadCondition: salobj ReadTopic
-        self._reader_dict = dict()
-        # list of salobj WriteTopic
-        self._writer_list = list()
+        self._running_cmds: typing.Dict[int, topics.CommandInfo] = dict()
+        # dict of dds.ReadCondition: salobj topics.ReadTopic
+        # This is needed because read conditions don't store the associated
+        # data reader. When a wait on a dds.WaitSet returns a read condition
+        # we use this dict to figure out which topic to read.
+        self._reader_dict: typing.Dict[dds.ReadCondition, topics.ReadTopic] = dict()
+        # list of salobj topics.WriteTopic
+        self._writer_list: typing.List[topics.WriteTopic] = list()
         # the first RemoteCommand created should set this to
         # an lsst.ts.salobj.topics.AckCmdReader
         # and set its callback to self._ackcmd_callback
-        self._ackcmd_reader = None
+        self._ackcmd_reader: typing.Optional[topics.AckCmdReader] = None
         # the first ControllerCommand created should set this to
         # an lsst.ts.salobj.topics.AckCmdWriter
-        self._ackcmd_writer = None
+        self._ackcmd_writer: typing.Optional[topics.AckCmdWriter] = None
         # wait_timeout is a failsafe for shutdown; normally all you have to do
         # is call `close` to trigger the guard condition and stop the wait
         self._wait_timeout = dds.DDSDuration(sec=10)
         self._guardcond = dds.GuardCondition()
         self._waitset = dds.WaitSet()
         self._waitset.attach(self._guardcond)
-        self._read_loop_task = base.make_done_future()
+        self._read_loop_task = utils.make_done_future()
 
         idl_path = domain.idl_dir / f"sal_revCoded_{self.name}.idl"
         if not idl_path.is_file():
             raise RuntimeError(
                 f"Cannot find IDL file {idl_path} for name={self.name!r}"
             )
+        self.parsed_idl = dds_utils.parse_idl_file(idl_path)
         self.metadata = idl_metadata.parse_idl(name=self.name, idl_path=idl_path)
         self.parse_metadata()  # Adds self.indexed, self.revnames, etc.
         if self.index != 0 and not self.indexed:
@@ -328,16 +277,19 @@ class SalInfo:
             ackcmd_revname = self.revnames.get("ackcmd")
             if ackcmd_revname is None:
                 raise RuntimeError(f"Could not find {self.name} topic 'ackcmd'")
-            self._ackcmd_type = ddsutil.get_dds_classes_from_idl(
-                idl_path, ackcmd_revname
+            self._ackcmd_type: type_hints.AckCmdDataType = (
+                dds_utils.make_dds_topic_class(
+                    parsed_idl=self.parsed_idl, revname=ackcmd_revname
+                )
             )
 
         domain.add_salinfo(self)
 
         # Make sure the background thread terminates.
         atexit.register(self.basic_close)
+        self.isopen = True
 
-    def _ackcmd_callback(self, data):
+    def _ackcmd_callback(self, data: type_hints.AckCmdDataType) -> None:
         if not self._running_cmds:
             return
         # Note: ReadTopic's reader filters out ackcmd samples
@@ -354,7 +306,7 @@ class SalInfo:
             del self._running_cmds[data.private_seqNum]
 
     @property
-    def AckCmdType(self):
+    def AckCmdType(self) -> typing.Type[type_hints.AckCmdDataType]:
         """The class of command acknowledgement.
 
         It includes these fields, as well as the usual other
@@ -378,20 +330,20 @@ class SalInfo:
         """
         if len(self.command_names) == 0:
             raise RuntimeError("This component has no commands, so no ackcmd topic")
-        return self._ackcmd_type.topic_data_class
+        return self._ackcmd_type.topic_data_class  # type: ignore
 
     @property
-    def cmd_partition_name(self):
+    def cmd_partition_name(self) -> str:
         """Partition name for command topics."""
         return f"{self.partition_prefix}.{self.name}.cmd"
 
     @property
-    def data_partition_name(self):
+    def data_partition_name(self) -> str:
         """Partition name for non-command topics."""
         return f"{self.partition_prefix}.{self.name}.data"
 
     @property
-    def cmd_publisher(self):
+    def cmd_publisher(self) -> dds.Publisher:
         """Publisher for command topics, but not ackcmd.
 
         This has a different partition name than a data_publisher.
@@ -402,7 +354,7 @@ class SalInfo:
         return self._cmd_publisher
 
     @property
-    def cmd_subscriber(self):
+    def cmd_subscriber(self) -> dds.Subscriber:
         """Subscriber for command topics, but not ackcmd.
 
         This has a different partition name than a data_subscriber.
@@ -415,7 +367,7 @@ class SalInfo:
         return self._cmd_subscriber
 
     @property
-    def data_publisher(self):
+    def data_publisher(self) -> dds.Publisher:
         """Publisher for ackcmd, events and telemetry topics.
 
         This has a different partition name than a cmd_publisher.
@@ -428,7 +380,7 @@ class SalInfo:
         return self._data_publisher
 
     @property
-    def data_subscriber(self):
+    def data_subscriber(self) -> dds.Subscriber:
         """Subscriber for ackcmd, events and telemetry topics.
 
         This has a different partition name than a cmd_subscriber.
@@ -441,7 +393,7 @@ class SalInfo:
         return self._data_subscriber
 
     @property
-    def name_index(self):
+    def name_index(self) -> str:
         """Get name[:index].
 
         The suffix is only present if the component is indexed.
@@ -452,16 +404,15 @@ class SalInfo:
             return self.name
 
     @property
-    def started(self):
-        """Return True if successfully started, False otherwise.
-        """
+    def started(self) -> bool:
+        """Return True if successfully started, False otherwise."""
         return (
             self.start_task.done()
             and not self.start_task.cancelled()
             and self.start_task.exception() is None
         )
 
-    def assert_started(self):
+    def assert_started(self) -> None:
         """Raise RuntimeError if not successfully started.
 
         Notes
@@ -473,8 +424,14 @@ class SalInfo:
             raise RuntimeError("Not started")
 
     def make_ackcmd(
-        self, private_seqNum, ack, error=0, result="", timeout=0, truncate_result=False
-    ):
+        self,
+        private_seqNum: int,
+        ack: sal_enums.SalRetCode,
+        error: int = 0,
+        result: str = "",
+        timeout: float = 0,
+        truncate_result: bool = False,
+    ) -> type_hints.AckCmdDataType:
         """Make an AckCmdType object from keyword arguments.
 
         Parameters
@@ -525,7 +482,9 @@ class SalInfo:
             timeout=timeout,
         )
 
-    def makeAckCmd(self, *args, **kwargs):
+    def makeAckCmd(
+        self, *args: typing.Any, **kwargs: typing.Any
+    ) -> type_hints.AckCmdDataType:
         """Deprecated version of make_ackcmd."""
         # TODO DM-26518: remove this method
         warnings.warn(
@@ -533,10 +492,7 @@ class SalInfo:
         )
         return self.make_ackcmd(*args, **kwargs)
 
-    def __repr__(self):
-        return f"SalInfo({self.name}, {self.index})"
-
-    def parse_metadata(self):
+    def parse_metadata(self) -> None:
         """Parse the IDL metadata to generate some attributes.
 
         Set the following attributes (see the class doc string for details):
@@ -574,7 +530,7 @@ class SalInfo:
         self.sal_topic_names = tuple(sorted(self.metadata.topic_info.keys()))
         self.revnames = revnames
 
-    def basic_close(self):
+    def basic_close(self) -> None:
         """A synchronous and less thorough version of `close`.
 
         Intended for exit handlers and constructor error handlers.
@@ -583,12 +539,20 @@ class SalInfo:
             return
         self.isopen = False
         self._guardcond.trigger()
+        self._read_loop_task.cancel()
+        try:
+            self._detach_read_conditions()
+        except Exception as e:
+            print(
+                f"{self}.basic_close: failed to detach one or more conditions "
+                f"from the wait set; continuing: {e!r}"
+            )
         for reader in self._reader_dict.values():
             reader.basic_close()
         for writer in self._writer_list:
             writer.basic_close()
 
-    async def close(self):
+    async def close(self) -> None:
         """Shut down and clean up resources.
 
         May be called multiple times. The first call closes the SalInfo;
@@ -603,6 +567,13 @@ class SalInfo:
             # Give the read loop time to exit.
             await asyncio.sleep(0.01)
             self._read_loop_task.cancel()
+            try:
+                self._detach_read_conditions()
+            except Exception as e:
+                print(
+                    f"{self}.close: failed to detach one or more conditions "
+                    f"from the wait set; continuing: {e!r}"
+                )
             for reader in self._reader_dict.values():
                 await reader.close()
             for writer in self._writer_list:
@@ -620,7 +591,7 @@ class SalInfo:
             if not self.done_task.done():
                 self.done_task.set_result(None)
 
-    def add_reader(self, topic):
+    def add_reader(self, topic: topics.ReadTopic) -> None:
         """Add a ReadTopic, so it can be read by the read loop and closed
         by `close`.
 
@@ -641,7 +612,7 @@ class SalInfo:
         self._reader_dict[topic._read_condition] = topic
         self._waitset.attach(topic._read_condition)
 
-    def add_writer(self, topic):
+    def add_writer(self, topic: topics.WriteTopic) -> None:
         """Add a WriteTopic, so it can be closed by `close`.
 
         Parameters
@@ -651,7 +622,7 @@ class SalInfo:
         """
         self._writer_list.append(topic)
 
-    async def start(self):
+    async def start(self) -> None:
         """Start the read loop.
 
         Call this after all topics have been added.
@@ -732,7 +703,18 @@ class SalInfo:
             self.start_task.set_exception(e)
             raise
 
-    async def _read_loop(self, loop):
+    def _detach_read_conditions(self) -> None:
+        """Try to detach all read conditions from the wait set.
+
+        ADLink suggests doing this at shutdown to work around a bug
+        in their software that produces spurious error messages
+        in the ospl log.
+        """
+        self._waitset.detach(self._guardcond)
+        for read_cond in self._reader_dict:
+            self._waitset.detach(read_cond)
+
+    async def _read_loop(self, loop: asyncio.AbstractEventLoop) -> None:
         """Read and process data.
 
         Parameters
@@ -751,7 +733,7 @@ class SalInfo:
         finally:
             self.domain.num_read_loops -= 1
 
-    def _read_loop_thread(self, loop):
+    def _read_loop_thread(self, loop: asyncio.AbstractEventLoop) -> None:
         """Read and process DDS data in a background thread.
 
         Parameters
@@ -782,18 +764,18 @@ class SalInfo:
                 if sd_list:
                     reader._queue_data(sd_list, loop=loop)
 
-    def _sample_to_data(self, sd, si):
+    def _sample_to_data(self, sd: dds._Sample, si: dds.SampleInfo) -> dds._Sample:
         """Process one sample data, sample info pair.
 
         Set sd.private_rcvStamp based on si.reception_timestamp
         and return the updated sd.
         """
         rcv_utc = si.reception_timestamp * 1e-9
-        rcv_tai = base.tai_from_utc_unix(rcv_utc)
+        rcv_tai = utils.tai_from_utc_unix(rcv_utc)
         sd.private_rcvStamp = rcv_tai
         return sd
 
-    def _wait_history(self):
+    def _wait_history(self) -> bool:
         """Wait for historical data to be available for all topics.
 
         Blocks, so intended to be run in a background thread.
@@ -803,11 +785,11 @@ class SalInfo:
         iosk : `bool`
             True if we got historical data or none was wanted
         """
-        time_limit = os.environ.get("LSST_DDS_HISTORYSYNC")
-        if time_limit is None:
-            time_limit = DEFAULT_LSST_DDS_HISTORYSYNC
+        time_limit_str = os.environ.get("LSST_DDS_HISTORYSYNC")
+        if time_limit_str is None:
+            time_limit: float = DEFAULT_LSST_DDS_HISTORYSYNC
         else:
-            time_limit = float(time_limit)
+            time_limit = float(time_limit_str)
         if time_limit < 0:
             self.log.info(
                 f"Time limit {time_limit} < 0; not waiting for historical data"
@@ -832,10 +814,18 @@ class SalInfo:
             wait_timeout = dds.DDSDuration(sec=rem_time)
         return num_ok > 0 or num_checked == 0
 
-    async def __aenter__(self):
+    async def __aenter__(self) -> SalInfo:
         if self.start_called:
             await self.start_task
         return self
 
-    async def __aexit__(self, type, value, traceback):
+    async def __aexit__(
+        self,
+        type: typing.Optional[typing.Type[BaseException]],
+        value: typing.Optional[BaseException],
+        traceback: typing.Optional[types.TracebackType],
+    ) -> None:
         await self.close()
+
+    def __repr__(self) -> str:
+        return f"SalInfo({self.name}, {self.index})"
