@@ -1,6 +1,6 @@
 # This file is part of ts_salobj.
 #
-# Developed for the LSST Telescope and Site Systems.
+# Developed for the Rubin Observatory Telescope and Site System.
 # This product includes software developed by the LSST Project
 # (https://www.lsst.org).
 # See the COPYRIGHT file at the top-level directory of this distribution
@@ -22,14 +22,17 @@
 __all__ = ["TestCsc"]
 
 import asyncio
-import pathlib
 import string
+import types
+import typing
 
 import numpy as np
 
+from . import type_hints
+from . import __version__
 from .base_csc import State
 from .configurable_csc import ConfigurableCsc
-from . import __version__
+from .config_schema import CONFIG_SCHEMA
 
 
 class TestCsc(ConfigurableCsc):
@@ -67,6 +70,14 @@ class TestCsc(ConfigurableCsc):
         or `State.ENABLED`.
     simulation_mode : `int`, optional
         Simulation mode. The only allowed value is 0.
+    schema_path : `str`, `pathlib.Path` or `None`, optional
+        Path to a schema file used to validate configuration files.
+        This is only for testing the deprecated ``schema_path``
+        `ConfigurableCsc` constructor argument;
+        real CSCs should _not_ provide this argument.
+        If not None then `ConfigurableCsc` is called with this,
+        instead of the ``config_schema`` argument.
+
 
     Raises
     ------
@@ -102,48 +113,33 @@ class TestCsc(ConfigurableCsc):
 
     def __init__(
         self,
-        index,
-        config_dir=None,
-        initial_state=State.STANDBY,
-        settings_to_apply="",
-        simulation_mode=0,
+        index: int,
+        config_dir: typing.Optional[type_hints.PathType] = None,
+        initial_state: State = State.STANDBY,
+        settings_to_apply: str = "",
+        simulation_mode: int = 0,
+        schema_path: typing.Optional[type_hints.PathType] = None,
     ):
-        schema_path = (
-            pathlib.Path(__file__).resolve().parents[4] / "schema" / "Test.yaml"
-        )
+        if schema_path is not None:
+            config_kwargs: typing.Dict[str, typing.Any] = dict(schema_path=schema_path)
+        else:
+            config_kwargs = dict(config_schema=CONFIG_SCHEMA)
+        print("config_kwargs=", config_kwargs)
         super().__init__(
-            "Test",
-            schema_path=schema_path,
-            config_dir=config_dir,
+            name="Test",
             index=index,
+            config_dir=config_dir,
             initial_state=initial_state,
             settings_to_apply=settings_to_apply,
             simulation_mode=simulation_mode,
+            **config_kwargs,
         )
-        self.cmd_wait.allow_multiple_callbacks = True
-        self.config = None
+        self.cmd_wait.allow_multiple_callbacks = True  # type: ignore
+        self.config: typing.Optional[types.SimpleNamespace] = None
 
-    def do_setArrays(self, data):
-        """Execute the setArrays command."""
-        self.assert_enabled()
-        self.log.info("executing setArrays")
-        data_dict = self.as_dict(data, self.arrays_fields)
-        self.evt_arrays.set(**data_dict)
-        self.tel_arrays.set(**data_dict)
-        self.evt_arrays.put()
-        self.tel_arrays.put()
-
-    def do_setScalars(self, data):
-        """Execute the setScalars command."""
-        self.assert_enabled()
-        self.log.info("executing setScalars")
-        data_dict = self.as_dict(data, self.scalars_fields)
-        self.evt_scalars.set(**data_dict)
-        self.tel_scalars.set(**data_dict)
-        self.evt_scalars.put()
-        self.tel_scalars.put()
-
-    def as_dict(self, data, fields):
+    def as_dict(
+        self, data: typing.Any, fields: typing.Sequence[str]
+    ) -> typing.Dict[str, typing.Any]:
         """Return the specified fields from a data struct as a dict.
 
         Parameters
@@ -158,7 +154,27 @@ class TestCsc(ConfigurableCsc):
             ret[field] = getattr(data, field)
         return ret
 
-    def do_fault(self, data):
+    def do_setArrays(self, data: type_hints.BaseDdsDataType) -> None:
+        """Execute the setArrays command."""
+        self.assert_enabled()
+        self.log.info("executing setArrays")
+        data_dict = self.as_dict(data, self.arrays_fields)
+        self.evt_arrays.set(**data_dict)  # type: ignore
+        self.tel_arrays.set(**data_dict)  # type: ignore
+        self.evt_arrays.put()  # type: ignore
+        self.tel_arrays.put()  # type: ignore
+
+    def do_setScalars(self, data: type_hints.BaseDdsDataType) -> None:
+        """Execute the setScalars command."""
+        self.assert_enabled()
+        self.log.info("executing setScalars")
+        data_dict = self.as_dict(data, self.scalars_fields)
+        self.evt_scalars.set(**data_dict)  # type: ignore
+        self.tel_scalars.set(**data_dict)  # type: ignore
+        self.evt_scalars.put()  # type: ignore
+        self.tel_scalars.put()  # type: ignore
+
+    def do_fault(self, data: type_hints.BaseDdsDataType) -> None:
         """Execute the fault command.
 
         Change the summary state to State.FAULT
@@ -166,7 +182,7 @@ class TestCsc(ConfigurableCsc):
         self.log.warning("executing the fault command")
         self.fault(code=1, report="executing the fault command")
 
-    async def do_wait(self, data):
+    async def do_wait(self, data: type_hints.BaseDdsDataType) -> None:
         """Execute the wait command by waiting for the specified duration.
 
         If duration is negative then wait for abs(duration) but do not
@@ -174,22 +190,21 @@ class TestCsc(ConfigurableCsc):
         testing command timeout.
         """
         self.assert_enabled()
-        if data.duration >= 0:
-            self.cmd_wait.ack_in_progress(data, timeout=data.duration)
-        await asyncio.sleep(abs(data.duration))
+        duration: float = data.duration  # type: ignore
+        if duration >= 0:
+            self.cmd_wait.ack_in_progress(data, timeout=duration)  # type: ignore
+        await asyncio.sleep(abs(duration))
 
     @property
-    def field_type(self):
+    def field_type(self) -> typing.Dict[str, typing.Any]:
         """Get a dict of field_name: element type."""
         return dict(
             boolean0=bool,
             byte0=np.uint8,
-            char0=str,
             short0=np.int16,
             int0=np.int32,
             long0=np.int32,
             longLong0=np.int64,
-            octet0=np.uint8,
             unsignedShort0=np.uint16,
             unsignedInt0=np.uint32,
             unsignedLong0=np.uint32,
@@ -199,7 +214,7 @@ class TestCsc(ConfigurableCsc):
         )
 
     @property
-    def arrays_fields(self):
+    def arrays_fields(self) -> typing.Sequence[str]:
         """Get a tuple of the fields in an arrays struct."""
         return (
             "boolean0",
@@ -208,7 +223,6 @@ class TestCsc(ConfigurableCsc):
             "int0",
             "long0",
             "longLong0",
-            "octet0",
             "unsignedShort0",
             "unsignedInt0",
             "unsignedLong0",
@@ -217,17 +231,15 @@ class TestCsc(ConfigurableCsc):
         )
 
     @property
-    def scalars_fields(self):
+    def scalars_fields(self) -> typing.Sequence[str]:
         """Get a tuple of the fields in a scalars struct."""
         return (
             "boolean0",
             "byte0",
-            "char0",
             "short0",
             "int0",
             "long0",
             "longLong0",
-            "octet0",
             "unsignedShort0",
             "unsignedInt0",
             "unsignedLong0",
@@ -237,7 +249,7 @@ class TestCsc(ConfigurableCsc):
         )
 
     @property
-    def int_fields(self):
+    def int_fields(self) -> typing.Sequence[str]:
         """Get a tuple of the integer fields in a struct."""
         return (
             "byte0",
@@ -245,13 +257,12 @@ class TestCsc(ConfigurableCsc):
             "int0",
             "long0",
             "longLong0",
-            "octet0",
             "unsignedShort0",
             "unsignedInt0",
             "unsignedLong0",
         )
 
-    def assert_arrays_equal(self, arrays1, arrays2):
+    def assert_arrays_equal(self, arrays1: typing.Any, arrays2: typing.Any) -> None:
         """Assert that two arrays data structs are equal.
 
         The types need not match; each struct can be command, event
@@ -262,12 +273,15 @@ class TestCsc(ConfigurableCsc):
         for field in reversed(self.arrays_fields):
             field_arr1 = getattr(arrays1, field)
             field_arr2 = getattr(arrays2, field)
-            if not np.array_equal(field_arr1, field_arr2):
+            is_float = field in ("float0", "double0")
+            if not np.array_equal(  # type: ignore
+                field_arr1, field_arr2, equal_nan=is_float
+            ):
                 raise AssertionError(
                     f"arrays1.{field} = {field_arr1} != {field_arr2} = arrays2.{field}"
                 )
 
-    def assert_scalars_equal(self, scalars1, scalars2):
+    def assert_scalars_equal(self, scalars1: typing.Any, scalars2: typing.Any) -> None:
         """Assert that two scalars data structs are equal.
 
         The types need not match; each struct can be command, event
@@ -278,30 +292,33 @@ class TestCsc(ConfigurableCsc):
         for field in reversed(self.scalars_fields):
             field_val1 = getattr(scalars1, field)
             field_val2 = getattr(scalars2, field)
-            if field_val1 != field_val2:
+            is_float = field in ("float0", "double0")
+            if not np.array_equal(  # type: ignore
+                field_val1, field_val2, equal_nan=is_float
+            ):
                 raise AssertionError(
                     f"scalars1.{field} = {field_val1} != {field_val2} = scalars2.{field}"
                 )
 
-    def make_random_cmd_arrays(self):
-        return self._make_random_arrays(dtype=self.cmd_setArrays.DataType)
+    def make_random_cmd_arrays(self) -> typing.Any:
+        return self._make_random_arrays(dtype=self.cmd_setArrays.DataType)  # type: ignore
 
-    def make_random_evt_arrays(self):
-        return self._make_random_arrays(dtype=self.evt_arrays.DataType)
+    def make_random_evt_arrays(self) -> typing.Any:
+        return self._make_random_arrays(dtype=self.evt_arrays.DataType)  # type: ignore
 
-    def make_random_tel_arrays(self):
-        return self._make_random_arrays(dtype=self.tel_arrays.DataType)
+    def make_random_tel_arrays(self) -> typing.Any:
+        return self._make_random_arrays(dtype=self.tel_arrays.DataType)  # type: ignore
 
-    def make_random_cmd_scalars(self):
-        return self._make_random_scalars(dtype=self.cmd_setScalars.DataType)
+    def make_random_cmd_scalars(self) -> typing.Any:
+        return self._make_random_scalars(dtype=self.cmd_setScalars.DataType)  # type: ignore
 
-    def make_random_evt_scalars(self):
-        return self._make_random_scalars(dtype=self.evt_scalars.DataType)
+    def make_random_evt_scalars(self) -> typing.Any:
+        return self._make_random_scalars(dtype=self.evt_scalars.DataType)  # type: ignore
 
-    def make_random_tel_scalars(self):
-        return self._make_random_scalars(dtype=self.tel_scalars.DataType)
+    def make_random_tel_scalars(self) -> typing.Any:
+        return self._make_random_scalars(dtype=self.tel_scalars.DataType)  # type: ignore
 
-    def _make_random_arrays(self, dtype):
+    def _make_random_arrays(self, dtype: typing.Any) -> typing.Any:
         """Make random arrays data using numpy.random."""
         data = dtype()
         nelts = len(data.boolean0)
@@ -323,12 +340,11 @@ class TestCsc(ConfigurableCsc):
             setattr(data, field, field_data)
         return data
 
-    def _make_random_scalars(self, dtype):
+    def _make_random_scalars(self, dtype: typing.Any) -> typing.Any:
         """Make random data for cmd_setScalars using numpy.random."""
         data = dtype()
         data.boolean0 = np.random.choice([False, True])
         printable_chars = [c for c in string.printable]
-        data.char0 = "".join(np.random.choice(printable_chars, size=(20,)))
         data.string0 = "".join(np.random.choice(printable_chars, size=(20,)))
         for field in self.int_fields:
             field_type = self.field_type[field]
@@ -345,8 +361,8 @@ class TestCsc(ConfigurableCsc):
         return data
 
     @staticmethod
-    def get_config_pkg():
+    def get_config_pkg() -> str:
         return "ts_config_ocs"
 
-    async def configure(self, config):
+    async def configure(self, config: types.SimpleNamespace) -> None:
         self.config = config
