@@ -1,6 +1,8 @@
+from __future__ import annotations
+
 # This file is part of ts_salobj.
 #
-# Developed for the LSST Telescope and Site Systems.
+# Developed for the Rubin Observatory Telescope and Site System.
 # This product includes software developed by the LSST Project
 # (https://www.lsst.org).
 # See the COPYRIGHT file at the top-level directory of this distribution
@@ -23,17 +25,23 @@ __all__ = ["AckCmdWriter", "ControllerCommand"]
 
 import asyncio
 import inspect
+import typing
+import warnings
 
 from .. import sal_enums
 from .. import base
+from .. import type_hints
 from . import read_topic
 from . import write_topic
 
+if typing.TYPE_CHECKING:
+    from ..sal_info import SalInfo
+
 
 class AckCmdWriter(write_topic.WriteTopic):
     """Command Acknowledgement writer."""
 
-    def __init__(self, salinfo):
+    def __init__(self, salinfo: SalInfo) -> None:
         super().__init__(
             salinfo=salinfo, name="ackcmd", sal_prefix="", min_seq_num=None
         )
@@ -48,6 +56,8 @@ class ControllerCommand(read_topic.ReadTopic):
         SAL component information
     name : `str`
         Command name
+    queue_len : `int`, optional
+        Number of elements that can be queued for `get_oldest`.
 
     Notes
     -----
@@ -62,13 +72,13 @@ class ControllerCommand(read_topic.ReadTopic):
     After the initial acknowledgement with ``ack=SalRetCode.CMD_ACK``,
     automatic ackowledgement for callback functions works as follows:
 
-    * If the callback function is a coroutine then acknowledge with
-      ``ack=SalRetCode.CMD_INPROGRESS`` just before running the callback.
     * If the callback function returns `None` then send a final
       acknowledgement with ``ack=SalRetCode.CMD_COMPLETE``.
     * If the callback function returns an acknowledgement
-      (instance of `SalInfo.AckType`) instead of `None`, then send that
-      as the final acknowledgement.
+      (instance of `SalInfo.AckCmdType`) instead of `None`, then send that
+      as the final acknowledgement. This is very unusual, but might
+      be useful if the callback wants to exit early and leave some
+      background task or thread processing the rest of the command.
     * If the callback function raises `asyncio.TimeoutError` then send a
       final acknowledgement with ``ack=SalRetCode.CMD_TIMEOUT``.
     * If the callback function raises `asyncio.CancelledError` then send
@@ -80,50 +90,85 @@ class ControllerCommand(read_topic.ReadTopic):
       then do the same as `ExpectedError` and also log a traceback.
     """
 
-    def __init__(self, salinfo, name, max_history=0, queue_len=100):
+    def __init__(
+        self, salinfo: SalInfo, name: str, queue_len: int = read_topic.DEFAULT_QUEUE_LEN
+    ) -> None:
         super().__init__(
             salinfo=salinfo,
             name=name,
             sal_prefix="command_",
-            max_history=max_history,
+            max_history=0,
             queue_len=queue_len,
         )
+        # Set false to ignore authorization.
+        # The only command should ignore authorization is
+        # the command that handles user requests for authorization.
+        # TODO DM-26605: Change this to True
+        self.authorize = False
         self.cmdtype = salinfo.sal_topic_names.index(self.sal_name)
         if salinfo._ackcmd_writer is None:
             self.salinfo._ackcmd_writer = AckCmdWriter(salinfo=salinfo)
 
-    def ack(self, data, ackcmd):
+    def ack(
+        self, data: type_hints.BaseDdsDataType, ackcmd: type_hints.AckCmdDataType
+    ) -> None:
         """Acknowledge a command by writing a new state.
 
         Parameters
         ----------
         data : `DataType`
-            Command data.
+            Data for the command being acknowledged.
         ackcmd : `salobj.AckCmdType`
-            Command acknowledgement.
+            Command acknowledgement data.
         """
-        self.salinfo._ackcmd_writer.set(
+        # mypy thinks salinfo._ackcmd_writer can be None, but it can't.
+        # Testing is expensive, so hide the warnings.
+        self.salinfo._ackcmd_writer.set(  # type: ignore
             private_seqNum=data.private_seqNum,
-            host=data.private_host,
             origin=data.private_origin,
+            identity=data.private_identity,
             cmdtype=self.cmdtype,
             ack=ackcmd.ack,
             error=ackcmd.error,
             result=ackcmd.result,
+            timeout=ackcmd.timeout,
         )
-        self.salinfo._ackcmd_writer.put()
+        self.salinfo._ackcmd_writer.put()  # type: ignore
 
-    def ackInProgress(self, data, result=""):
+    def ackInProgress(self, data: type_hints.BaseDdsDataType, result: str = "") -> None:
+        """Deprecated version of ack_in_progress."""
+        # TODO DM-26518: remove this method
+        warnings.warn(
+            "ackInProgress is deprecated; use ack_in_progress and specify timeout instead",
+            DeprecationWarning,
+        )
+        self.ack_in_progress(data=data, result=result, timeout=0)
+
+    def ack_in_progress(
+        self, data: type_hints.BaseDdsDataType, timeout: float, result: str = ""
+    ) -> None:
         """Ackowledge this command as "in progress".
+
+        Parameters
+        ----------
+        data : `DataType`
+            Data for the command being acknowledged.
+        timeout : `float`
+            Estimated command duration (sec).
+        result : `str`, optional
+            Reason for acknowledgement. Typically left "".
         """
-        ackcmd = self.salinfo.makeAckCmd(
+        ackcmd = self.salinfo.make_ackcmd(
             private_seqNum=data.private_seqNum,
             ack=sal_enums.SalRetCode.CMD_INPROGRESS,
             result=result,
+            timeout=timeout,
         )
         self.ack(data, ackcmd)
 
-    async def next(self, *, timeout=None):
+    async def next(  # type: ignore[override]  # noqa
+        self, *, timeout: typing.Optional[float] = None
+    ) -> type_hints.BaseDdsDataType:
         """Wait for data, returning old data if found.
 
         Unlike `RemoteEvent.next` and `RemoteTelemetry.next`,
@@ -132,7 +177,7 @@ class ControllerCommand(read_topic.ReadTopic):
 
         Parameters
         ----------
-        timeout : `float` (optional)
+        timeout : `float`, optional
             Time limit, in seconds. If None then no time limit.
 
         Returns
@@ -152,18 +197,17 @@ class ControllerCommand(read_topic.ReadTopic):
         """
         return await super().next(flush=False, timeout=timeout)
 
-    def _queue_one_item(self, data):
-        """Convert the value to an ``ackcmd`` and queue it.
-        """
+    def _queue_one_item(self, data: type_hints.BaseDdsDataType) -> None:
+        """Convert the value to an ``ackcmd`` and queue it."""
         if data.private_seqNum <= 0:
             raise ValueError(f"private_seqNum={data.private_seqNum} must be positive")
-        ack = self.salinfo.makeAckCmd(
+        ack = self.salinfo.make_ackcmd(
             private_seqNum=data.private_seqNum, ack=sal_enums.SalRetCode.CMD_ACK
         )
         self.ack(data, ack)
         super()._queue_one_item(data)
 
-    async def _run_callback(self, data):
+    async def _run_callback(self, data: type_hints.BaseDdsDataType) -> None:
         """Run the callback function, acknowledge the command,
         and start another wait.
 
@@ -172,22 +216,51 @@ class ControllerCommand(read_topic.ReadTopic):
         data : `DataType`
             Command data.
         """
+        if self.authorize:
+            try:
+                identity = data.private_identity
+                auth_error = None
+                if "@" in identity:
+                    # A user
+                    if (
+                        identity != self.salinfo.domain.user_host
+                        and identity not in self.salinfo.authorized_users
+                    ):
+                        auth_error = (
+                            f"User identity {identity!r} "
+                            f"not self ({self.salinfo.domain.user_host}) "
+                            f"and not in {self.salinfo.authorized_users}"
+                        )
+                elif identity in self.salinfo.non_authorized_cscs:
+                    auth_error = f"CSC identity {identity!r} in {self.salinfo.non_authorized_cscs}"
+                if auth_error is not None:
+                    ack = self.salinfo.make_ackcmd(
+                        private_seqNum=data.private_seqNum,
+                        ack=sal_enums.SalRetCode.CMD_NOPERM,
+                        error=1,
+                        result=f"Not authorized: {auth_error}",
+                        truncate_result=True,
+                    )
+                    self.ack(data, ack)
+                    return
+            except Exception:
+                self.log.exception("Error checking identity")
+
         try:
-            result = self._callback(data)
+            result = self._callback(data)  # type: ignore
             if inspect.isawaitable(result):
-                self.ackInProgress(data)
-                ack = await result
+                ack = await result  # type: ignore
             else:
-                ack = result
+                ack = result  # type: ignore
             if ack is None:
-                ack = self.salinfo.makeAckCmd(
+                ack = self.salinfo.make_ackcmd(
                     private_seqNum=data.private_seqNum,
                     ack=sal_enums.SalRetCode.CMD_COMPLETE,
                     result="Done",
                 )
             self.ack(data, ack)
         except asyncio.CancelledError:
-            ack = self.salinfo.makeAckCmd(
+            ack = self.salinfo.make_ackcmd(
                 private_seqNum=data.private_seqNum,
                 ack=sal_enums.SalRetCode.CMD_ABORTED,
                 error=1,
@@ -196,7 +269,7 @@ class ControllerCommand(read_topic.ReadTopic):
             )
             self.ack(data, ack)
         except asyncio.TimeoutError:
-            ack = self.salinfo.makeAckCmd(
+            ack = self.salinfo.make_ackcmd(
                 private_seqNum=data.private_seqNum,
                 ack=sal_enums.SalRetCode.CMD_TIMEOUT,
                 error=1,
@@ -205,7 +278,7 @@ class ControllerCommand(read_topic.ReadTopic):
             )
             self.ack(data, ack)
         except Exception as e:
-            ack = self.salinfo.makeAckCmd(
+            ack = self.salinfo.make_ackcmd(
                 private_seqNum=data.private_seqNum,
                 ack=sal_enums.SalRetCode.CMD_FAILED,
                 error=1,
