@@ -1,6 +1,8 @@
+from __future__ import annotations
+
 # This file is part of ts_salobj.
 #
-# Developed for the LSST Telescope and Site Systems.
+# Developed for the Rubin Observatory Telescope and Site System.
 # This product includes software developed by the LSST Project
 # (https://www.lsst.org).
 # See the COPYRIGHT file at the top-level directory of this distribution
@@ -19,18 +21,38 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
-__all__ = ["QueueCapacityChecker", "ReadTopic"]
+__all__ = ["QueueCapacityChecker", "ReadTopic", "DEFAULT_QUEUE_LEN", "MIN_QUEUE_LEN"]
 
 import asyncio
 import bisect
 import collections
 import inspect
+import logging
+import typing
+import warnings
 
 import dds
 
-from .base_topic import BaseTopic
-from ..domain import DDS_READ_QUEUE_LEN
+from lsst.ts import utils
 from .. import base
+from .. import type_hints
+from .base_topic import BaseTopic
+
+if typing.TYPE_CHECKING:
+    from ..sal_info import SalInfo
+
+# Default value for the ``queue_len`` constructor argument.
+DEFAULT_QUEUE_LEN = 100
+
+# Minimum value for the ``queue_len`` constructor argument.
+MIN_QUEUE_LEN = 10
+
+
+_BasicReturnType = typing.Optional[type_hints.AckCmdDataType]
+CallbackType = typing.Callable[
+    [type_hints.BaseDdsDataType],
+    typing.Union[_BasicReturnType, typing.Awaitable[_BasicReturnType]],
+]
 
 
 class QueueCapacityChecker:
@@ -91,9 +113,11 @@ class QueueCapacityChecker:
     produces an error because data is likely to have been lost.
     """
 
-    def __init__(self, descr, log, queue_len):
-        if queue_len < 10:
-            raise ValueError(f"queue_len {queue_len} must be >= 10")
+    def __init__(self, descr: str, log: logging.Logger, queue_len: int) -> None:
+        if queue_len < MIN_QUEUE_LEN:
+            raise ValueError(
+                f"queue_len {queue_len} must be >= MIN_QUEUE_LEN={MIN_QUEUE_LEN}"
+            )
         self.descr = descr
         self.log = log
         self.queue_len = queue_len
@@ -108,10 +132,10 @@ class QueueCapacityChecker:
         self._reset_thresholds = tuple(
             warn_thresh // 2 for warn_thresh in self.warn_thresholds
         )
-        self.warn_threshold = self.warn_thresholds[0]
-        self.reset_threshold = None
+        self.warn_threshold: typing.Optional[int] = self.warn_thresholds[0]
+        self.reset_threshold: typing.Optional[int] = None
 
-    def check_nitems(self, nitems):
+    def check_nitems(self, nitems: int) -> bool:
         """Check the number of items in the queue and log a message
         if appropriate.
 
@@ -167,11 +191,11 @@ class ReadTopic(BaseTopic):
     max_history : `int`
         Maximum number of historical items to read:
 
-        * 0 is required for commands and the ackcmd reader
-        * 1 is recommended for events and telemetry
-    queue_len : `int` (optional)
-        The maximum number of items that can be read and not dealt with
-        by a callback function or `next` before older data will be dropped.
+        * 0 is required for commands, events, and the ackcmd topic
+        * 1 is recommended for telemetry
+    queue_len : `int`, optional
+        The maximum number of messages that can be read and not dealt with
+        by a callback function or `next` before older messages will be dropped.
     filter_ackcmd : `bool`, optional
         Filter out cmdack topics so we only see responses to commands
         that we sent? This is normally what you want, but it is not wanted
@@ -185,9 +209,14 @@ class ReadTopic(BaseTopic):
     ValueError
         If max_history > 0 and the topic is volatile (command or ackcmd).
     ValueError
-        If queue_len <= 0.
+        If queue_len < MIN_QUEUE_LEN.
     ValueError
         If max_history > queue_len.
+    UserWarning
+        If max_history > DDS history queue depth or DDS durability service
+        history depth for this topic.
+        This is a warning rather than an exception, so that the DDS quality
+        of service can be changed without breaking existing code.
 
     Attributes
     ----------
@@ -202,37 +231,51 @@ class ReadTopic(BaseTopic):
     -----
     **Queues**
 
-    There are actually two queues: an internal queue whose length
-    is set by ``queue_len`` and a dds queue whose length is set by
-    low level configuration. Data can be lost in two ways:
+    There are two queues: a Python queue whose length is set by ``queue_len``
+    and a dds queue whose length is set by the DDS Quality of Service file.
+    (The Python queue is needed because of limitations in the API for
+    the OpenSplice DDS queue, including no access to the most recent message,
+    no ability to ask how many messages are on the queue, and no asyncio
+    support). In the doc strings for the methods, below, any reference
+    to the queue refers to the Python queue.
 
-    - If this class cannot read data from the dds queue fast enough, then
-      older data will be dropped from the dds queue. You will get a warning
-      log message if the reader starts to fall behind.
-    - As data is read it is put on the internal queue. if a callback function
-      or `next` does not process data quickly enough then older data
-      is dropped from the internal queue. If you have a callback function
-      you will get several warning log messages as this internal queue
-      fills up. You get no warning otherwise because this class has no way
-      of knowing whether or not you intend to read all data using `next`.
+    Data can be lost from either queue:
+
+    - If this class cannot read messages from the DDS queue fast enough,
+      then older messages will be dropped from the DDS queue. You will get
+      several warning log messages as the DDS queue fills.
+    - As messages are read from the DDS queue they are put on the Python queue.
+      If a callback function or `next` does not process data quickly enough
+      then older messages are dropped from the Python queue.
+      If you have a callback function then you will get several
+      warning log messages as the Python queue fills up;
+      you get no warning otherwise because `ReadTopic` has no way of knowing
+      whether or not you intend to read all messages.
 
     **Reading**
 
-    Reading is performed by the `.SalInfo` which has single read loop that
-    reads all topics. This is more efficient than having each `ReadTopic` read
-    its own data.
+    Reading is performed by the contained `SalInfo`, which has single
+    read loop that reads messages for all topics. This is more efficient
+    than having each `ReadTopic` read its own messages.
+
+    **Modifying Messages**
+
+    All functions that return messages return them from some form of internal
+    cache. This presents a risk: if any reader modifies a message, then it
+    will be modified for all readers of that message. To safely modify a
+    returned message, make your own copy with ``copy.copy(data)``.
     """
 
     def __init__(
         self,
         *,
-        salinfo,
-        name,
-        sal_prefix,
-        max_history,
-        queue_len=100,
-        filter_ackcmd=True,
-    ):
+        salinfo: SalInfo,
+        name: str,
+        sal_prefix: str,
+        max_history: int,
+        queue_len: int = DEFAULT_QUEUE_LEN,
+        filter_ackcmd: bool = True,
+    ) -> None:
         super().__init__(salinfo=salinfo, name=name, sal_prefix=sal_prefix)
         self.isopen = True
         self._allow_multiple_callbacks = False
@@ -240,32 +283,60 @@ class ReadTopic(BaseTopic):
             raise ValueError(f"max_history={max_history} must be >= 0")
         if max_history > 0 and self.volatile:
             raise ValueError(f"max_history={max_history} must be 0 for volatile topics")
-        if queue_len <= 0:
-            raise ValueError(f"queue_len={queue_len} must be positive")
+        if queue_len <= MIN_QUEUE_LEN:
+            raise ValueError(
+                f"queue_len={queue_len} must be >= MIN_QUEUE_LEN={MIN_QUEUE_LEN}"
+            )
         if max_history > queue_len:
             raise ValueError(
                 f"max_history={max_history} must be <= queue_len={queue_len}"
             )
+        if (
+            max_history > self.qos_set.reader_qos.history.depth
+            or max_history > self.qos_set.topic_qos.durability_service.history_depth
+        ):
+            warnings.warn(
+                f"max_history={max_history} > history depth={self.qos_set.reader_qos.history.depth} "
+                f"and/or {self.qos_set.topic_qos.durability_service.history_depth}; "
+                "you will get less historical data than you asked for.",
+                UserWarning,
+            )
         self._max_history = int(max_history)
-        self._data_queue = collections.deque(maxlen=queue_len)
-        self._current_data = None
-        # task that `next` waits on
-        self._next_task = base.make_done_future()
-        self._callback = None
-        self._callback_tasks = set()
-        self._callback_loop_task = base.make_done_future()
+        self._data_queue: typing.Deque[type_hints.BaseDdsDataType] = collections.deque(
+            maxlen=queue_len
+        )
+        self._current_data: typing.Optional[type_hints.BaseDdsDataType] = None
+        # Task that `next` waits on.
+        # Its result is set to the oldest message on the queue.
+        # We do this instead of having `next` itself pop the oldest message
+        # because it allows multiple callers of `next` to all get the same
+        # message, and it avoids a potential race condition with `flush`.
+        self._next_task = utils.make_done_future()
+        self._callback: typing.Optional[CallbackType] = None
+        self._callback_tasks: typing.Set[asyncio.Task] = set()
+        self._callback_loop_task = utils.make_done_future()
         self.dds_queue_length_checker = QueueCapacityChecker(
-            descr=f"{name} DDS read queue", log=self.log, queue_len=DDS_READ_QUEUE_LEN
+            descr=f"{name} DDS read queue",
+            log=self.log,
+            queue_len=self.qos_set.reader_qos.history.depth,
         )
         self.python_queue_length_checker = QueueCapacityChecker(
             descr=f"{name} python read queue", log=self.log, queue_len=queue_len
         )
-        qos = (
-            salinfo.domain.volatile_reader_qos
-            if self.volatile
-            else salinfo.domain.reader_qos
+        # Command topics use a different a partition name than
+        # all other topics, including ackcmd, and the partition name
+        # is part of the publisher and subscriber.
+        # This split allows us to create just one subscriber and one publisher
+        # for each Controller or Remote:
+        # `Controller` only needs a cmd_subscriber and data_publisher,
+        # `Remote` only needs a cmd_publisher and data_subscriber.
+        if sal_prefix == "command_":
+            subscriber = salinfo.cmd_subscriber
+        else:
+            subscriber = salinfo.data_subscriber
+        self._reader = subscriber.create_datareader(
+            self._topic, self.qos_set.reader_qos
         )
-        self._reader = salinfo.subscriber.create_datareader(self._topic, qos)
         # TODO DM-26411: replace ANY_INSTANCE_STATE with ALIVE_INSTANCE_STATE
         # once the OpenSplice issue 00020647 is fixed.
         read_mask = [
@@ -278,7 +349,7 @@ class ReadTopic(BaseTopic):
         if name == "ackcmd" and filter_ackcmd:
             queries += [
                 f"origin = {salinfo.domain.origin}",
-                f"host = {salinfo.domain.host}",
+                f"identity = '{salinfo.identity}'",
             ]
         if queries:
             full_query = " AND ".join(queries)
@@ -290,7 +361,7 @@ class ReadTopic(BaseTopic):
         salinfo.add_reader(self)
 
     @property
-    def allow_multiple_callbacks(self):
+    def allow_multiple_callbacks(self) -> bool:
         """Can callbacks can run simultaneously?
 
         Notes
@@ -304,15 +375,18 @@ class ReadTopic(BaseTopic):
         return self._allow_multiple_callbacks
 
     @allow_multiple_callbacks.setter
-    def allow_multiple_callbacks(self, allow):
+    def allow_multiple_callbacks(self, allow: bool) -> None:
         self._allow_multiple_callbacks = bool(allow)
 
     @property
-    def callback(self):
+    def callback(
+        self,
+    ) -> typing.Optional[CallbackType]:
         """Callback function, or None if there is not one.
 
-        The callback function is called when new data is received;
-        it receives one argument: the data.
+        The callback function is called when a new message is received;
+        it receives one argument: the message (an object of type
+        `topics.BaseTopic.DataType`).
 
         Raises
         ------
@@ -330,13 +404,13 @@ class ReadTopic(BaseTopic):
 
         `get_oldest` and `next` are prohibited if there is a callback function.
         Technically they could both work, but `get_oldest` would always return
-        `None` and `next` would miss data if it arrived while waiting
-        for something else. It seemed safer to just raise an exception.
+        `None` and `next` would miss messages if they arrived while waiting
+        for something else. It seems safer to raise an exception.
         """
         return self._callback
 
     @callback.setter
-    def callback(self, func):
+    def callback(self, func: typing.Optional[CallbackType]) -> None:
         self._cancel_callbacks()
 
         if func is None:
@@ -349,16 +423,16 @@ class ReadTopic(BaseTopic):
             raise TypeError(f"func={func} not callable")
         self._data_queue.clear()
         self._callback = func
-        self._callback_loop_task = asyncio.ensure_future(self._callback_loop())
+        self._callback_loop_task = asyncio.create_task(self._callback_loop())
 
     @property
-    def has_callback(self):
+    def has_callback(self) -> bool:
         """Return True if there is a callback function."""
         return self._callback is not None
 
     @property
-    def has_data(self):
-        """Has any data been seen for this topic?
+    def has_data(self) -> bool:
+        """Has any data ever been seen for this topic?
 
         Raises
         ------
@@ -369,36 +443,59 @@ class ReadTopic(BaseTopic):
         return self._current_data is not None
 
     @property
-    def max_history(self):
+    def nqueued(self) -> int:
+        """Return the number of messages in the Python queue."""
+        return len(self._data_queue)
+
+    @property
+    def max_history(self) -> int:
         return self._max_history
 
-    async def close(self):
+    def basic_close(self) -> None:
+        """A synchronous and possibly less thorough version of `close`.
+
+        Intended for exit handlers and constructor error handlers.
+        """
+        if not self.isopen:
+            return
+        self.isopen = False
+        self._callback = None
+        try:
+            # These raise RuntimeError if the asyncio loop is not running.
+            self._cancel_callbacks()
+            self._next_task.cancel()
+        except RuntimeError:
+            pass
+        self._reader.close()
+        self._data_queue.clear()
+
+    async def close(self) -> None:
         """Shut down and release resources.
 
         Intended to be called by SalInfo.close(),
         since that tracks all topics.
         """
-        if not self.isopen:
-            return
-        self.isopen = False
-        self._cancel_callbacks()
-        self._callback = None
-        self._next_task.cancel()
-        self._reader.close()
-        self._data_queue.clear()
+        self.basic_close()
 
-    async def aget(self, timeout=None):
-        """Get the current value, if any, else wait for the next value.
+    async def aget(
+        self, timeout: typing.Optional[float] = None
+    ) -> type_hints.BaseDdsDataType:
+        """Get the most recent message, or wait for data if no data has
+        ever been seen (`has_data` False).
+
+        This method does not change which message will be returned by
+        any other method (except for the fact that new data
+        will arrive while waiting).
 
         Parameters
         ----------
-        timeout : `float` (optional)
+        timeout : `float`, optional
             Time limit, in seconds. If None then no time limit.
 
         Returns
         -------
         data : `DataType`
-            The current or next value.
+            The current or next message.
 
         Raises
         ------
@@ -408,18 +505,25 @@ class ReadTopic(BaseTopic):
 
         Notes
         -----
-        Do not modify the returned data. To make a copy that you can modify
-        use ``copy.copy(value)``.
+        Do not modify the returned data. To make a copy that you can
+        safely modify, use ``copy.copy(data)``.
         """
         self.salinfo.assert_started()
         if self.has_callback:
             raise RuntimeError("Not allowed because there is a callback function")
-        if self._current_data is not None:
-            return self._current_data
-        return await self._next(timeout=timeout)
+        if self._current_data is None:
+            if self._next_task.done():
+                self._next_task = asyncio.Future()
+            await asyncio.wait_for(self._next_task, timeout=timeout)
+        assert self._current_data is not None  # make mypy happy
+        return self._current_data
 
-    def flush(self):
-        """Flush the queue of unread data.
+    def flush(self) -> None:
+        """Flush the queue used by `get_oldest` and `next`.
+
+        This makes `get_oldest` return `None` and `next` wait,
+        until a new message arrives.
+        It does not change which message will be returned by `aget` or `get`.
 
         Raises
         ------
@@ -430,15 +534,27 @@ class ReadTopic(BaseTopic):
             raise RuntimeError("Not allowed because there is a callback function")
         self._data_queue.clear()
 
-    def get(self, flush=True):
-        """Get the most recently seen value, or `None` if no data ever seen.
+    def get(
+        self, flush: typing.Optional[bool] = None
+    ) -> typing.Optional[type_hints.BaseDdsDataType]:
+        """Get the most recent message, or `None` if no data has ever been seen
+        (`has_data` False).
+
+        This method does not change which message will be returned by `aget`.
+        If ``flush=False`` this method also does not modify which message
+        will be returned by `get_oldest` and `next`.
 
         Parameters
         ----------
-        flush : `bool` (optional)
-            Flush the queue? Defaults to `True` for backwards compatibility.
-            This only affects the next value returned by `next`
-            and is ignored if there is a callback function.
+        flush : `bool`, optional
+            Flush the queue? Flushing the queue is deprecated and so
+            is specifying this argument.
+            False (the default) leaves the cache alone, which has no effect
+            on the messages returned by any read method.
+            True affects which messages will be returned by `get_oldest`
+            and `next`. True has no effect if there is a callback function.
+            Note: `None` is treated as `False`, but please do not specify
+            it; it only supported for now to handle deprecation warnings..
 
         Returns
         -------
@@ -451,18 +567,37 @@ class ReadTopic(BaseTopic):
             If the ``salinfo`` has not started reading.
         """
         self.salinfo.assert_started()
+        if flush is None:
+            flush = False
+        else:
+            if flush:
+                warnings.warn(
+                    "flush=True is deprecated for ReadTopic.get",
+                    DeprecationWarning,
+                )
+            else:
+                warnings.warn(
+                    "Specifying a value for the flush argument is deprecated "
+                    "for ReadTopic.get; use the default of False.",
+                    DeprecationWarning,
+                )
+
         if flush and not self.has_callback:
             self.flush()
         return self._current_data
 
-    def get_oldest(self):
-        """Pop and return the oldest value from the queue, or `None`
-        if the queue is empty.
+    def get_oldest(self) -> typing.Optional[type_hints.BaseDdsDataType]:
+        """Pop and return the oldest message from the queue, or `None` if the
+        queue is empty.
+
+        This is a variant of `next` that does not wait for a new message.
+        This method affects which message will be returned by `next`,
+        but not which message will be returned by `aget` or `get`.
 
         Returns
         -------
         data : ``self.DataType`` or `None`
-            The oldest value found on the queue, if any, else `None`.
+            The oldest message found on the queue, if any, else `None`.
 
         Raises
         ------
@@ -482,22 +617,30 @@ class ReadTopic(BaseTopic):
             return self._data_queue.popleft()
         return None
 
-    async def next(self, *, flush, timeout=None):
-        """Wait for a value, possibly returning the oldest queued value.
+    async def next(
+        self, *, flush: bool, timeout: typing.Optional[float] = None
+    ) -> type_hints.BaseDdsDataType:
+        """Pop and return the oldest message from the queue, waiting for data
+        if the queue is empty.
+
+        This method affects the data returned by `get_oldest`,
+        but not the data returned by `aget` or `get`.
 
         Parameters
         ----------
         flush : `bool`
-            If True then flush the queue before starting a read.
-            If False then pop and return the oldest value from the queue,
-            if any, else wait for new data.
-        timeout : `float` (optional)
+            If `True` then flush the queue before starting a read.
+            This guarantees that the method will wait for a new message.
+            If `False` and there is data on the queue, then pop and return
+            the oldest message from the queue, without waiting;
+            if queue is empty then wait for a new message.
+        timeout : `float`, optional
             Time limit, in seconds. If None then no time limit.
 
         Returns
         -------
         data : `DataType`
-            The data.
+            The message data.
 
         Raises
         ------
@@ -507,8 +650,8 @@ class ReadTopic(BaseTopic):
 
         Notes
         -----
-        Do not modify the returned data. To make a copy that you can modify
-        use ``copy.copy(value)``.
+        Do not modify the returned data. To make a copy that you can
+        safely modify, use ``copy.copy(data)``.
         """
         self.salinfo.assert_started()
         if self.has_callback:
@@ -517,15 +660,12 @@ class ReadTopic(BaseTopic):
             self.flush()
         return await self._next(timeout=timeout)
 
-    async def _next(self, *, timeout=None):
+    async def _next(
+        self, *, timeout: typing.Optional[float] = None
+    ) -> type_hints.BaseDdsDataType:
         """Implement next.
 
         Unlike `next`, this can be called while using a callback function.
-
-        Raises
-        ------
-        RuntimeError
-            If code is already waiting for the next value.
         """
         self.python_queue_length_checker.check_nitems(len(self._data_queue))
         if self._data_queue:
@@ -534,7 +674,7 @@ class ReadTopic(BaseTopic):
             self._next_task = asyncio.Future()
         return await asyncio.wait_for(self._next_task, timeout=timeout)
 
-    async def _callback_loop(self):
+    async def _callback_loop(self) -> None:
         while True:
             if not self.has_callback:
                 return
@@ -545,33 +685,46 @@ class ReadTopic(BaseTopic):
                 self._callback_tasks = {
                     task for task in self._callback_tasks if not task.done()
                 }
-                self._callback_tasks.add(asyncio.ensure_future(result))
+                self._callback_tasks.add(asyncio.create_task(result))
             else:
                 await result
 
-    def _cancel_callbacks(self):
-        """Cancel the callback loop and all existing callback tasks.
-        """
+    def _cancel_callbacks(self) -> None:
+        """Cancel the callback loop and all existing callback tasks."""
         self._callback_loop_task.cancel()
         while self._callback_tasks:
             task = self._callback_tasks.pop()
             task.cancel()
 
-    async def _run_callback(self, data):
+    async def _run_callback(self, data: type_hints.BaseDdsDataType) -> None:
         try:
-            result = self._callback(data)
+            # mypy gets upset because self._callback may be None
+            # but it's too expensive to check that
+            result = self._callback(data)  # type: ignore
             if inspect.isawaitable(result):
-                await result
+                await result  # type: ignore
         except asyncio.CancelledError:
             raise
         except Exception as e:
             if not isinstance(e, base.ExpectedError):
                 self.log.exception(f"Callback {self.callback} failed with data={data}")
 
-    def _queue_data(self, data_list):
-        """Queue multiple one or more values.
+    def _queue_data(
+        self,
+        data_list: typing.Sequence[type_hints.BaseDdsDataType],
+        loop: typing.Optional[asyncio.AbstractEventLoop],
+    ) -> None:
+        """Queue multiple one or more messages.
 
-        This is a no-op if ``data_list`` is empty.
+        Parameters
+        ----------
+        data_list : `list` [dds_messages]
+            DDS messages to be queueued.
+        loop : `asyncio.AbstractEventLoop` or `None`
+            Foreground asyncio loop.
+            Specify the loop if and only if running from a background thread.
+            If running from the main asyncio loop specify ``loop = None``
+            (this is done to read historical data while starting).
 
         Also update ``self._current_data`` and fire `self._next_task`
         (if pending).
@@ -581,19 +734,26 @@ class ReadTopic(BaseTopic):
         for data in data_list:
             self._queue_one_item(data)
         self._current_data = data
-        if not self._data_queue:
-            self.log.warning(
-                f"{self.name}: queue empty after queuing {len(data_list)} items; "
-                "not setting _next_task done."
-            )
-        elif not self._next_task.done():
-            oldest_value = self._data_queue.popleft()
-            self._next_task.set_result(oldest_value)
+        if loop is not None and loop.is_running():
+            # Reading messages in a background thread.
+            loop.call_soon_threadsafe(self._report_next)
+        else:
+            # Reading messages in the main thread.
+            self._report_next()
 
-    def _queue_one_item(self, data):
-        """Add a single value to the internal queue.
+    def _queue_one_item(self, data: type_hints.BaseDdsDataType) -> None:
+        """Add a single message to the Python queue.
 
-        Subclasses may override to massage the value before queuing.
+        Subclasses may override this to modify the message before queuing.
         `ControllerCommand` does this.
         """
         self._data_queue.append(data)
+
+    def _report_next(self) -> None:
+        """Set self._next_task to the oldest message on the queue.
+
+        A no-op if self._next_task is done or the queue is empty.
+        """
+        if not self._next_task.done() and self._data_queue:
+            oldest_message = self._data_queue.popleft()
+            self._next_task.set_result(oldest_message)
