@@ -1,6 +1,8 @@
+from __future__ import annotations
+
 # This file is part of ts_salobj.
 #
-# Developed for the LSST Telescope and Site Systems.
+# Developed for the Rubin Observatory Telescope and Site System.
 # This product includes software developed by the LSST Project
 # (https://www.lsst.org).
 # See the COPYRIGHT file at the top-level directory of this distribution
@@ -19,17 +21,24 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
-__all__ = ["AckCmdReader", "RemoteCommand"]
+__all__ = ["AckCmdReader", "CommandInfo", "RemoteCommand"]
 
 import asyncio
 import collections
 import logging
 import random
+import typing
+import time
 
 from .. import base
 from .. import sal_enums
+from .. import type_hints
 from . import read_topic
 from . import write_topic
+from . import remote_command
+
+if typing.TYPE_CHECKING:
+    from ..sal_info import SalInfo
 
 DEFAULT_TIMEOUT = 60 * 60  # default timeout, in seconds
 
@@ -50,7 +59,9 @@ class AckCmdReader(read_topic.ReadTopic):
     SAL component.
     """
 
-    def __init__(self, salinfo, queue_len=100):
+    def __init__(
+        self, salinfo: SalInfo, queue_len: int = read_topic.DEFAULT_QUEUE_LEN
+    ) -> None:
         super().__init__(
             salinfo=salinfo,
             name="ackcmd",
@@ -60,7 +71,7 @@ class AckCmdReader(read_topic.ReadTopic):
         )
 
 
-class _CommandInfo:
+class CommandInfo:
     """Struct to hold information about a command.
 
     Parameters
@@ -74,13 +85,18 @@ class _CommandInfo:
         If false then wait for the next ack instead.
     """
 
-    def __init__(self, remote_command, seq_num, wait_done):
+    def __init__(
+        self,
+        remote_command: remote_command.RemoteCommand,
+        seq_num: int,
+        wait_done: bool,
+    ) -> None:
         self.remote_command = remote_command
         self.seq_num = int(seq_num)
         self.wait_done = bool(wait_done)
 
-        self._wait_task = asyncio.Future()
-        self._next_ack_task = asyncio.Future()
+        self._wait_task: asyncio.Future = asyncio.Future()
+        self._next_ack_event = asyncio.Event()
 
         self.done_ack_codes = frozenset(
             (
@@ -113,23 +129,23 @@ class _CommandInfo:
 
         # we should see at most 3 acks, but leave room for one more,
         # just in case
-        self._ack_queue = collections.deque(maxlen=4)
-        self._last_ackcmd = None
+        self._ack_queue: typing.Deque[type_hints.AckCmdDataType] = collections.deque(
+            maxlen=4
+        )
+        self._last_ackcmd: typing.Optional[type_hints.AckCmdDataType] = None
 
-    def abort(self, result=""):
+    def abort(self, result: str = "") -> None:
         """Report command as aborted. Ignored if already done."""
         if not self._wait_task.done():
             self._wait_task.cancel()
-        if not self._next_ack_task.done():
-            self._next_ack_task.cancel()
 
-    def add_ackcmd(self, ackcmd):
+    def add_ackcmd(self, ackcmd: type_hints.AckCmdDataType) -> bool:
         """Add a command acknowledgement to the queue.
 
         Parameters
         ----------
         ackcmd : `SalInfo.AckCmdType`
-            Command acknowledgement data.
+            Command acknowledgement message.
 
         Returns
         -------
@@ -139,11 +155,12 @@ class _CommandInfo:
         # print(f"add_ackcmd; ackcmd.ack={ackcmd.ack}")
         isdone = ackcmd.ack in self.done_ack_codes
         self._ack_queue.append(ackcmd)
-        if not self._next_ack_task.done():
-            self._next_ack_task.set_result(None)
+        self._next_ack_event.set()
         return isdone
 
-    async def next_ackcmd(self, timeout):
+    async def next_ackcmd(
+        self, timeout: float = DEFAULT_TIMEOUT
+    ) -> type_hints.AckCmdDataType:
         """Get next command acknowledgement of interest.
 
         If ``wait_done`` true then return the final command acknowledgement,
@@ -153,9 +170,15 @@ class _CommandInfo:
 
         Parameters
         ----------
-        timeout : `float` (optional)
-            Timeout in seconds. If the next command acknowledgement
-            does not arrive in time then raise `AckTimeoutError`.
+        timeout : `float`, optional
+            Time limit, in seconds. If None then use ``DEFAULT_TIMEOUT``.
+            This time limit is for the command to finish, if ``wait_done``
+            is true, else it is for the next command acknowledgement.
+            If the command is acknowledged with ``CMD_INPROGRESS`` then the
+            timeout is extended by the timeout value in the acknowledgement.
+            Thus a slow command will not need a long timeout, so long as
+            the command issues a ``CMD_INPROGRESS`` acknowledgement
+            with a reasonable ``timeout`` value.
 
         Returns
         -------
@@ -169,20 +192,17 @@ class _CommandInfo:
         AckError
             If the command fails.
         """
-        if timeout is None:  # for backwards compatibility
-            timeout = DEFAULT_TIMEOUT
         try:
-            self._wait_task = asyncio.ensure_future(
-                asyncio.wait_for(self._basic_next_ack(), timeout=timeout)
+            self._wait_task = asyncio.create_task(
+                self._basic_next_ackcmd(timeout=timeout)
             )
             ackcmd = await self._wait_task
-            # print(f"next_ackcmd got {ackcmd.ack} from _basic_next_ack")
             if ackcmd.ack in self.failed_ack_codes:
                 raise base.AckError(msg="Command failed", ackcmd=ackcmd)
             return ackcmd
         except asyncio.TimeoutError:
             if self._last_ackcmd is None:
-                last_ackcmd = self.remote_command.salinfo.makeAckCmd(
+                last_ackcmd = self.remote_command.salinfo.make_ackcmd(
                     private_seqNum=self.seq_num,
                     ack=sal_enums.SalRetCode.CMD_NOACK,
                     result="No command acknowledgement seen",
@@ -196,42 +216,42 @@ class _CommandInfo:
                 msg="Timed out waiting for command acknowledgement", ackcmd=last_ackcmd
             )
 
-    async def _basic_next_ack(self):
-        """Get the next command acknowledgment of interest.
+    async def _basic_next_ackcmd(self, timeout: float) -> type_hints.AckCmdDataType:
+        """Basic implementation of next_ackcmd."""
+        t0 = time.monotonic()
+        elapsed_time: float = 0
+        while True:
+            ackcmd = await asyncio.wait_for(
+                self._get_next_ackcmd(), timeout=timeout - elapsed_time
+            )
+            if not self.wait_done or ackcmd.ack in self.done_ack_codes:
+                return ackcmd
+            if ackcmd.ack == sal_enums.SalRetCode.CMD_INPROGRESS:
+                timeout += ackcmd.timeout
+            elapsed_time = time.monotonic() - t0
 
-        Implements `next_ackcmd` but does not interpret the result
-        beyond ignoring acks that are not of interest.
+    async def _get_next_ackcmd(self) -> type_hints.AckCmdDataType:
+        """Get the next cmdack sample.
 
         Returns
         -------
         ackcmd : `SalInfo.AckCmdType`
-            Next command acknowledgement of interest.
+            Next ackcmd sample.
         """
         # note: set self._last_ackcmd here instead of in add_ackcmd
         # because it reduces or eliminates a race condition where a new
         # command acknowledgement comes in as next_ackcmd is timing out.
         while True:
-            while self._ack_queue:
+            if self._ack_queue:
                 ackcmd = self._ack_queue.popleft()
                 self._last_ackcmd = ackcmd
-                if not self.wait_done or ackcmd.ack in self.done_ack_codes:
-                    return ackcmd
-            self._next_ack_task = asyncio.Future()
-            await self._next_ack_task
-            ackcmd = self._ack_queue.popleft()
-            self._last_ackcmd = ackcmd
-            if not self.wait_done or ackcmd.ack in self.done_ack_codes:
                 return ackcmd
+            await self._next_ack_event.wait()
+            self._next_ack_event.clear()
 
-    def __del__(self):
-        for task_name in ("_wait_task", "_next_ack_task"):
-            task = getattr(self, task_name, None)
-            if task is not None and not task.done():
-                task.cancel()
-
-    def __repr__(self):
+    def __repr__(self) -> str:
         return (
-            f"_CommandInfo(remote_command={self.remote_command}, seq_num={self.seq_num}, "
+            f"CommandInfo(remote_command={self.remote_command}, seq_num={self.seq_num}, "
             f"wait_done={self.wait_done})"
         )
 
@@ -247,7 +267,7 @@ class RemoteCommand(write_topic.WriteTopic):
         Command name
     """
 
-    def __init__(self, salinfo, name):
+    def __init__(self, salinfo: SalInfo, name: str) -> None:
         num_commands = len(salinfo.command_names)
         seq_num_increment = write_topic.MAX_SEQ_NUM // num_commands
         name_ind = salinfo.command_names.index(name)
@@ -266,27 +286,37 @@ class RemoteCommand(write_topic.WriteTopic):
         # dict of seq_num: CommandInfo
         if salinfo._ackcmd_reader is None:
             salinfo._ackcmd_reader = AckCmdReader(salinfo)
-            salinfo._ackcmd_reader.callback = salinfo._ackcmd_callback
+            salinfo._ackcmd_reader.callback = salinfo._ackcmd_callback  # type: ignore
 
-    async def next_ackcmd(self, ackcmd, timeout=DEFAULT_TIMEOUT, wait_done=True):
-        """Wait for the next acknowledement for the command
+    async def next_ackcmd(
+        self,
+        ackcmd: type_hints.AckCmdDataType,
+        timeout: float = DEFAULT_TIMEOUT,
+        wait_done: bool = True,
+    ) -> type_hints.AckCmdDataType:
+        """Wait for the next acknowledgement for the command.
 
         Parameters
         ----------
         ackcmd : `SalInfo.AckCmdType`
-            The command acknowledgement returned by
-            the previous wait (e.g. from `start`).
-        timeout : `float` (optional)
-            Time limit, in seconds. If None then no time limit.
-            This time limit is for the entire command if wait_done
-            is true, else it is for the first acknowledgement
-            after the initial "SAL__CMD_ACK".
-        wait_done : `bool` (optional)
+            Command acknowledgement returned by the previous call to
+            `set_start`, `start` or `next_ackcmd`.
+        timeout : `float`, optional
+            Time limit, in seconds. If None then use ``DEFAULT_TIMEOUT``.
+            This time limit is for the entire command if ``wait_done``
+            is true, else it is for the next command acknowledgement.
+            If ``wait_done`` is true and the command is acknowledged with
+            ``CMD_INPROGRESS`` then the timeout is extended by
+            the timeout value in the acknowledgement.
+            Thus a slow command will not need a long timeout, so long as
+            the command issues a ``CMD_INPROGRESS`` acknowledgement
+            with a reasonable ``timeout`` value.
+        wait_done : `bool`, optional
             If True then wait for final command acknowledgement.
-            If False then wait until the next command acknowledgement;
-            if that acknowledgement is not final (the ack code is not in
-            ``done_ack_codes``), then you will almost certainly want to
-            await `next_ackcmd` again.
+            If False then wait only for the next command acknowledgement
+            If that acknowledgement is not final
+            (the ack code is not in ``self.done_ack_codes``),
+            then you will almost certainly want to await `next_ackcmd` again.
 
         Returns
         -------
@@ -309,7 +339,7 @@ class RemoteCommand(write_topic.WriteTopic):
         cmd_info.wait_done = wait_done
         return await cmd_info.next_ackcmd(timeout=timeout)
 
-    def set(self, **kwargs):
+    def set(self, **kwargs: typing.Any) -> bool:
         """Create a new ``self.data`` and set one or more fields.
 
         Parameters
@@ -326,7 +356,8 @@ class RemoteCommand(write_topic.WriteTopic):
         Returns
         -------
         did_change : `bool`
-            True if data was changed or if this was the first call to `set`.
+            True if ``self.data`` was changed, or if this was the first call
+            to `set`.
 
         Raises
         ------
@@ -340,25 +371,35 @@ class RemoteCommand(write_topic.WriteTopic):
         If one or more fields cannot be set, the data may be partially updated.
         """
         self.data = self.DataType()
-        super().set(**kwargs)
+        return super().set(**kwargs)
 
-    async def set_start(self, timeout=DEFAULT_TIMEOUT, wait_done=True, **kwargs):
+    async def set_start(
+        self,
+        timeout: float = DEFAULT_TIMEOUT,
+        wait_done: bool = True,
+        **kwargs: typing.Any,
+    ) -> type_hints.AckCmdDataType:
         """Create a new ``self.data``, set zero or more fields,
         and start the command.
 
         Parameters
         ----------
-        timeout : `float` (optional)
-            Time limit, in seconds. If None then no time limit.
-            This time limit is for the entire command if wait_done
-            is true, else it is for the first acknowledgement
-            after the initial "SAL__CMD_ACK".
-        wait_done : `bool` (optional)
+        timeout : `float`, optional
+            Time limit, in seconds. If None then use ``DEFAULT_TIMEOUT``.
+            This time limit is for the entire command if ``wait_done``
+            is true, else it is for the first command acknowledgement.
+            If ``wait_done`` is true and the command is acknowledged with
+            ``CMD_INPROGRESS`` then the timeout is extended by
+            the timeout value in the acknowledgement.
+            Thus a slow command will not need a long timeout, so long as
+            the command issues a ``CMD_INPROGRESS`` acknowledgement
+            with a reasonable ``timeout`` value.
+        wait_done : `bool`, optional
             If True then wait for final command acknowledgement.
-            If False then wait for the first acknowledgement after the
-            initial "SAL__CMD_ACK"; if that acknowledgement is not final
-            (the ack code is not in done_ack_codes), then you will almost
-            certainly want to await `next_ackcmd`.
+            If False then wait only for the first command acknowledgement
+            (typically ``CMD_ACK``). If that acknowledgement is not final
+            (the ack code is not in ``self.done_ack_codes``),
+            then you will almost certainly want to await `next_ackcmd`.
         **kwargs : `dict` [`str`, ``any``]
             The remaining keyword arguments are
             field name = new value for that field.
@@ -381,24 +422,34 @@ class RemoteCommand(write_topic.WriteTopic):
         self.set(**kwargs)
         return await self.start(timeout=timeout, wait_done=wait_done)
 
-    async def start(self, data=None, timeout=DEFAULT_TIMEOUT, wait_done=True):
+    async def start(
+        self,
+        data: type_hints.BaseDdsDataType = None,
+        timeout: float = DEFAULT_TIMEOUT,
+        wait_done: bool = True,
+    ) -> type_hints.AckCmdDataType:
         """Start a command.
 
         Parameters
         ----------
-        data : ``self.DataType`` (optional)
-            Command data. If None then send the current data.
-        timeout : `float` (optional)
-            Time limit, in seconds. If None then no time limit.
-            This time limit is for the entire command if wait_done
-            is true, else it is for the first acknowledgement
-            after the initial "SAL__CMD_ACK".
-        wait_done : `bool` (optional)
+        data : ``self.DataType``, optional
+            Command message. If `None` then send the current ``self.data``.
+        timeout : `float`, optional
+            Time limit, in seconds. If None then use ``DEFAULT_TIMEOUT``.
+            This time limit is for the entire command if ``wait_done``
+            is true, else it is for the first acknowledgement.
+            If ``wait_done`` is true and the command is acknowledged with
+            ``CMD_INPROGRESS`` then the timeout is extended by
+            the timeout value in the acknowledgement.
+            Thus a slow command will not need a long timeout, so long as
+            the command issues a ``CMD_INPROGRESS`` acknowledgement
+            with a reasonable ``timeout`` value.
+        wait_done : `bool`, optional
             If True then wait for final command acknowledgement.
-            If False then wait for the first acknowledgement after the
-            initial "SAL__CMD_ACK"; if that acknowledgement is not final
-            (the ack code is not in done_ack_codes), then you will almost
-            certainly want to await `next_ackcmd`.
+            If False then wait only for the first command acknowledgement
+            (typically ``CMD_ACK``). If that acknowledgement is not final
+            (the ack code is not in ``self.done_ack_codes``),
+            then you will almost certainly want to await `next_ackcmd`.
 
         Returns
         -------
@@ -425,7 +476,7 @@ class RemoteCommand(write_topic.WriteTopic):
             raise RuntimeError(
                 f"{self.name} bug: a command with seq_num={seq_num} is already running"
             )
-        cmd_info = _CommandInfo(
+        cmd_info = CommandInfo(
             remote_command=self, seq_num=seq_num, wait_done=wait_done
         )
         self.salinfo._running_cmds[seq_num] = cmd_info
