@@ -1,6 +1,8 @@
+from __future__ import annotations
+
 # This file is part of ts_salobj.
 #
-# Developed for the LSST Telescope and Site Systems.
+# Developed for the Rubin Observatory Telescope and Site System.
 # This product includes software developed by the LSST Project
 # (https://www.lsst.org).
 # See the COPYRIGHT file at the top-level directory of this distribution
@@ -21,8 +23,14 @@
 
 __all__ = ["RemoteTelemetry"]
 
+import typing
+import warnings
+
 from . import read_topic
 
+if typing.TYPE_CHECKING:
+    from ..sal_info import SalInfo
+
 
 class RemoteTelemetry(read_topic.ReadTopic):
     """Read a specific telemetry topic.
@@ -33,26 +41,34 @@ class RemoteTelemetry(read_topic.ReadTopic):
         SAL component information
     name : `str`
         Telemetry topic name
-    max_history : `int` (optional)
-        Maximum number of historical items to read:
-
-        * 0 if none; strongly recommended for `RemoteCommand` & `AckCmdReader`
-        * 1 is recommended for events and telemetry
-    queue_len : `int`
+    max_history : `int`, optional
+        Deprecated because historical telemetry data is no longer available.
+        Must be 0 (or None, but please don't do that) if specified.
+    queue_len : `int`, optional
         Number of elements that can be queued for `get_oldest`.
-
-    Notes
-    -----
-    All functions that return data return from an internal cache.
-    This cached value is replaced by all read operations, so as long as
-    no other code modifies the data, the returned data will not change.
     """
 
-    def __init__(self, salinfo, name, max_history=1, queue_len=100):
+    def __init__(
+        self,
+        salinfo: SalInfo,
+        name: str,
+        max_history: typing.Optional[int] = None,
+        queue_len: int = read_topic.DEFAULT_QUEUE_LEN,
+    ) -> None:
+        # TODO DM-26474: remove the max_history argument and this code block.
+        if max_history is not None:
+            if max_history == 0:
+                warnings.warn("max_history is deprecated", DeprecationWarning)
+            else:
+                raise ValueError(
+                    f"max_history={max_history} is deprecated "
+                    "and must be 0 (or None, but please don't do that) if specified"
+                )
+
         super().__init__(
             salinfo=salinfo,
             name=name,
             sal_prefix="",
-            max_history=max_history,
+            max_history=0,
             queue_len=queue_len,
         )
