@@ -1,7 +1,10 @@
 #!/usr/bin/env python
+
+# type: ignore
+
 # This file is part of ts_salobj.
 #
-# Developed for the LSST Telescope and Site Systems.
+# Developed for the Rubin Observatory Telescope and Site System.
 # This product includes software developed by the LSST Project
 # (https://www.lsst.org).
 # See the COPYRIGHT file at the top-level directory of this distribution
@@ -51,8 +54,7 @@ class MinimalSalobjController(salobj.Controller):
 
     @classmethod
     def make_from_cmd_line(cls):
-        """Make an instance from the command line.
-        """
+        """Make an instance from the command line."""
         parser = argparse.ArgumentParser("Run a minimal Salobj Test controller")
         parser.add_argument("index", type=int, help="Script SAL Component index")
         parser.add_argument("initial_log_level", type=int, help="Initial log level")
@@ -63,8 +65,7 @@ class MinimalSalobjController(salobj.Controller):
 
     @classmethod
     async def amain(cls):
-        """Make and run a controller.
-        """
+        """Make and run a controller."""
         controller = cls.make_from_cmd_line()
         await controller.done_task
 
@@ -72,11 +73,9 @@ class MinimalSalobjController(salobj.Controller):
         """Finish construction."""
         await self.salinfo.start()
         print(
-            f"SalobjController: outputting initial logLevel {self.evt_logLevel.data.level} "
-            "and the same value in telemetry scalars.int0"
+            f"SalobjController: writing initial logLevel.level={self.evt_logLevel.data.level} event"
         )
         self.evt_logLevel.put()
-        self.tel_scalars.put()
 
     def do_setLogLevel(self, data):
         print(
@@ -85,8 +84,8 @@ class MinimalSalobjController(salobj.Controller):
         )
 
         print(
-            f"SalobjController: writing logLevel={data.level} "
-            "and the same value in telemetry scalars.int0"
+            f"SalobjController: writing logLevel={data.level} event "
+            "and the same value in scalars.int0 telemetry"
         )
         self.evt_logLevel.set_put(level=data.level)
         self.tel_scalars.set_put(int0=data.level)
