@@ -1,7 +1,10 @@
 #!/usr/bin/env python
+
+# type: ignore
+
 # This file is part of ts_salobj.
 #
-# Developed for the LSST Telescope and Site Systems.
+# Developed for the Rubin Observatory Telescope and Site System.
 # This product includes software developed by the LSST Project
 # (https://www.lsst.org).
 # See the COPYRIGHT file at the top-level directory of this distribution
@@ -43,8 +46,7 @@ class MinimalSALPYController:
 
     @classmethod
     def make_from_cmd_line(cls):
-        """Make an instance from the command line.
-        """
+        """Make an instance from the command line."""
         parser = argparse.ArgumentParser("Run a minimal SALPY Test controller")
         parser.add_argument("index", type=int, help="Script SAL Component index")
         parser.add_argument("initial_log_level", type=int, help="Initial log level")
@@ -55,8 +57,7 @@ class MinimalSALPYController:
 
     @classmethod
     async def amain(cls):
-        """Make and run a controller.
-        """
+        """Make and run a controller."""
         controller = cls.make_from_cmd_line()
         await controller.run()
 
@@ -65,13 +66,10 @@ class MinimalSALPYController:
         logLevel_data = SALPY_Test.Test_logevent_logLevelC()
         scalars_data = SALPY_Test.Test_scalarsC()
         print(
-            f"SALPYController: outputting initial logLevel {self.initial_log_level} "
-            "and the same value in telemetry scalars.int0"
+            f"SALPYController: writing initial logLevel.level={self.initial_log_level} event"
         )
         logLevel_data.level = self.initial_log_level
         self.manager.logEvent_logLevel(logLevel_data, 1)
-        scalars_data.int0 = self.initial_log_level
-        self.manager.putSample_scalars(scalars_data)
 
         while True:
             while True:
@@ -91,8 +89,8 @@ class MinimalSALPYController:
             await asyncio.sleep(0.001)
 
             print(
-                f"SALPYController: writing logLevel={logLevel_data.level} "
-                "and the same value in telemetry scalars.int0"
+                f"SALPYController: writing logLevel={logLevel_data.level} event "
+                "and the same value in scalars.int0 telemetry"
             )
             logLevel_data.level = setLogLevel_data.level
             scalars_data.int0 = setLogLevel_data.level
