@@ -1,7 +1,10 @@
 #!/usr/bin/env python
+
+# type: ignore
+
 # This file is part of ts_salobj.
 #
-# Developed for the LSST Telescope and Site Systems.
+# Developed for the Rubin Observatory Telescope and Site System.
 # This product includes software developed by the LSST Project
 # (https://www.lsst.org).
 # See the COPYRIGHT file at the top-level directory of this distribution
@@ -22,6 +25,7 @@
 
 import asyncio
 
+from lsst.ts import utils
 from lsst.ts import salobj
 
 
@@ -44,10 +48,12 @@ class TopicWriter(salobj.BaseCsc):
         SAL index.
     """
 
+    valid_simulation_modes = [0]
+
     def __init__(self, index):
         print(f"TopicWriter: starting with index={index}")
         super().__init__(name="Test", index=index)
-        self.write_task = salobj.make_done_future()
+        self.write_task = utils.make_done_future()
         self.is_log_level = False
 
     async def do_fault(self, data):
