@@ -1,6 +1,6 @@
 # This file is part of ts_salobj.
 #
-# Developed for the LSST Telescope and Site Systems.
+# Developed for the Rubin Observatory Telescope and Site System.
 # This product includes software developed by the LSST Project
 # (https://www.lsst.org).
 # See the COPYRIGHT file at the top-level directory of this distribution
@@ -23,12 +23,14 @@ import asyncio
 import logging
 import os
 import subprocess
+import types
+import typing
 import unittest
 import warnings
 
-import asynctest
 import yaml
 
+from lsst.ts import utils
 from lsst.ts import salobj
 from lsst.ts.idl.enums.Script import ScriptState
 
@@ -45,45 +47,50 @@ class NonConfigurableScript(salobj.BaseScript):
     In other words get_schema returns None.
     """
 
-    def __init__(self, index):
+    def __init__(self, index: int) -> None:
         super().__init__(index=index, descr="Non-configurable script")
-        self.config = None
+        self.config: typing.Optional[types.SimpleNamespace] = None
         self.run_called = False
         self.set_metadata_called = False
 
     @classmethod
-    def get_schema(cls):
+    def get_schema(cls) -> None:
         return None
 
-    async def configure(self, config):
+    async def configure(self, config: typing.Optional[types.SimpleNamespace]) -> None:
         self.config = config
 
-    async def run(self):
+    async def run(self) -> None:
         self.run_called = True
 
-    def set_metadata(self, metadata):
+    def set_metadata(self, metadata: salobj.BaseDdsDataType) -> None:
         self.set_metadata_called = True
 
 
-class BaseScriptTestCase(asynctest.TestCase):
+class BaseScriptTestCase(unittest.IsolatedAsyncioTestCase):
     """Test `BaseScript` using simple subclasses `TestScript` and
     `NonConfigurableScript`.
     """
 
-    def setUp(self):
+    def setUp(self) -> None:
         salobj.set_random_lsst_dds_partition_prefix()
         self.datadir = os.path.abspath(os.path.join(os.path.dirname(__file__), "data"))
         self.index = next(index_gen)
 
     async def configure_and_check(
-        self, script, log_level=0, pause_checkpoint="", stop_checkpoint="", **kwargs
-    ):
+        self,
+        script: salobj.TestScript,
+        log_level: int = 0,
+        pause_checkpoint: str = "",
+        stop_checkpoint: str = "",
+        **kwargs: typing.Any,
+    ) -> None:
         """Configure a script by calling ``do_configure`` and set group ID
         and check the result.
 
         Parameters
         ----------
-        script : `ts.salobj.TestScript`
+        script : `TestScript`
             A test script
         log_level : `int`, optional
             Log level as a `logging` level,
@@ -148,20 +155,19 @@ class BaseScriptTestCase(asynctest.TestCase):
         self.assertEqual(script.group_id, group_id)
         self.assertEqual(script.evt_state.data.groupId, group_id)
 
-    def test_get_schema(self):
+    def test_get_schema(self) -> None:
         schema = salobj.TestScript.get_schema()
         self.assertTrue(isinstance(schema, dict))
         for name in ("$schema", "$id", "title", "description", "type", "properties"):
             self.assertIn(name, schema)
         self.assertFalse(schema["additionalProperties"])
 
-    def test_non_configurable_script_get_schema(self):
+    def test_non_configurable_script_get_schema(self) -> None:
         schema = NonConfigurableScript.get_schema()
         self.assertIsNone(schema)
 
-    async def test_non_configurable_script_empty_config(self):
-        """Test configuring the script with no data. This should work.
-        """
+    async def test_non_configurable_script_empty_config(self) -> None:
+        """Test configuring the script with no data. This should work."""
         async with NonConfigurableScript(index=self.index) as script:
             data = script.cmd_configure.DataType()
             await script.do_configure(data)
@@ -169,7 +175,7 @@ class BaseScriptTestCase(asynctest.TestCase):
             self.assertTrue(script.set_metadata_called)
             self.assertFalse(script.run_called)
 
-    async def test_non_configurable_script_invalid_config(self):
+    async def test_non_configurable_script_invalid_config(self) -> None:
         async with NonConfigurableScript(index=self.index) as script:
             data = script.cmd_configure.DataType()
             data.config = "invalid: should be empty"
@@ -177,7 +183,7 @@ class BaseScriptTestCase(asynctest.TestCase):
                 await script.do_configure(data)
             self.assertIsNone(script.config)
 
-    async def test_script_environ(self):
+    async def test_script_environ(self) -> None:
         """Test that creating a script does not modify os.environ.
 
         I would like to also test that a script has master priority 0
@@ -186,8 +192,9 @@ class BaseScriptTestCase(asynctest.TestCase):
         but that information is not available.
         """
         for master_priority in ("21", None):
-            env_kwargs = {salobj.MASTER_PRIORITY_ENV_VAR: master_priority}
-            with salobj.modify_environ(**env_kwargs):
+            with utils.modify_environ(
+                **{salobj.MASTER_PRIORITY_ENV_VAR: master_priority}
+            ):
                 initial_environ = os.environ.copy()
                 async with NonConfigurableScript(index=self.index):
                     self.assertEqual(os.environ, initial_environ)
@@ -195,7 +202,7 @@ class BaseScriptTestCase(asynctest.TestCase):
                 # closing a script should not modify the environment.
                 self.assertEqual(os.environ, initial_environ)
 
-    async def test_setCheckpoints(self):
+    async def test_setCheckpoints(self) -> None:
         async with salobj.TestScript(index=self.index) as script:
 
             # try valid values
@@ -226,7 +233,7 @@ class BaseScriptTestCase(asynctest.TestCase):
                 self.assertEqual(script.checkpoints.pause, initial_pause)
                 self.assertEqual(script.checkpoints.stop, initial_stop)
 
-    async def test_set_state_and_attributes(self):
+    async def test_set_state_and_attributes(self) -> None:
         async with salobj.TestScript(index=self.index) as script:
             # check keep_old_reason argument of set_state
             reason = "initial reason"
@@ -271,7 +278,7 @@ class BaseScriptTestCase(asynctest.TestCase):
 
             self.assertFalse(script.done_task.done())
 
-    async def test_next_supplemented_group_id(self):
+    async def test_next_supplemented_group_id(self) -> None:
         async with salobj.TestScript(index=self.index) as script:
             await self.configure_and_check(script)
             group_id = script.group_id
@@ -300,7 +307,7 @@ class BaseScriptTestCase(asynctest.TestCase):
             with self.assertRaises(RuntimeError):
                 script.next_supplemented_group_id()
 
-    async def test_pause(self):
+    async def test_pause(self) -> None:
         async with salobj.TestScript(index=self.index) as script:
             # Cannot run in UNCONFIGURED state.
             run_data = script.cmd_run.DataType()
@@ -383,7 +390,7 @@ class BaseScriptTestCase(asynctest.TestCase):
             print(f"test_pause duration={duration:0.2f}")
             self.assertLess(abs(duration - desired_duration), 0.2)
 
-    async def test_stop_at_checkpoint(self):
+    async def test_stop_at_checkpoint(self) -> None:
         async with salobj.TestScript(index=self.index) as script:
             wait_time = 0.1
             await self.configure_and_check(script, wait_time=wait_time)
@@ -410,7 +417,7 @@ class BaseScriptTestCase(asynctest.TestCase):
             print(f"test_stop_at_checkpoint duration={duration:0.2f}")
             self.assertLess(abs(duration - desired_duration), 0.2)
 
-    async def test_stop_while_paused(self):
+    async def test_stop_while_paused(self) -> None:
         async with salobj.TestScript(index=self.index) as script:
             wait_time = 5
             await self.configure_and_check(script, wait_time=wait_time)
@@ -444,7 +451,7 @@ class BaseScriptTestCase(asynctest.TestCase):
             self.assertGreater(duration, 0.0)
             self.assertLess(abs(duration - desired_duration), 0.2)
 
-    async def test_stop_while_running(self):
+    async def test_stop_while_running(self) -> None:
         async with salobj.TestScript(index=self.index) as script:
             wait_time = 5
             pause_time = 0.5
@@ -471,7 +478,7 @@ class BaseScriptTestCase(asynctest.TestCase):
             print(f"test_stop_while_running duration={duration:0.2f}")
             self.assertLess(abs(duration - desired_duration), 0.2)
 
-    async def check_fail(self, fail_run):
+    async def check_fail(self, fail_run: bool) -> None:
         """Check failure in run or cleanup.
 
         Parameters
@@ -509,17 +516,17 @@ class BaseScriptTestCase(asynctest.TestCase):
             print(f"test_fail duration={duration:0.3f} with fail_run={fail_run}")
             self.assertLess(abs(duration - desired_duration), 0.2)
 
-    async def test_fail_run(self):
+    async def test_fail_run(self) -> None:
         await self.check_fail(fail_run=True)
 
-    async def test_fail_cleanup(self):
+    async def test_fail_cleanup(self) -> None:
         await self.check_fail(fail_run=False)
 
-    async def test_zero_index(self):
+    async def test_zero_index(self) -> None:
         with self.assertRaises(ValueError):
             salobj.TestScript(index=0)
 
-    async def test_remote(self):
+    async def test_remote(self) -> None:
         """Test a script with remotes.
 
         Check that the remote_names attribute of the description event
@@ -528,7 +535,9 @@ class BaseScriptTestCase(asynctest.TestCase):
         """
 
         class ScriptWithRemotes(salobj.TestScript):
-            def __init__(self, index, remote_indices):
+            def __init__(
+                self, index: int, remote_indices: typing.Iterable[int]
+            ) -> None:
                 super().__init__(index, descr="Script with remotes")
                 remotes = []
                 # use remotes that read history here, to check that
@@ -547,9 +556,8 @@ class BaseScriptTestCase(asynctest.TestCase):
             for remote in script.remotes:
                 self.assertTrue(remote.start_task.done())
 
-    async def test_script_process(self):
-        """Test running a script as a subprocess.
-        """
+    async def test_script_process(self) -> None:
+        """Test running a script as a subprocess."""
         script_path = os.path.join(self.datadir, "script1")
 
         for fail in (None, "fail_run", "fail_cleanup"):
@@ -557,11 +565,14 @@ class BaseScriptTestCase(asynctest.TestCase):
                 async with salobj.Domain() as domain:
                     index = next(index_gen)
                     remote = salobj.Remote(
-                        domain=domain, name="Script", index=index, evt_max_history=0,
+                        domain=domain,
+                        name="Script",
+                        index=index,
+                        evt_max_history=0,
                     )
                     await asyncio.wait_for(remote.start_task, timeout=STD_TIMEOUT)
 
-                    def logcallback(data):
+                    def logcallback(data: salobj.BaseDdsDataType) -> None:
                         print(f"message={data.message}")
 
                     remote.evt_logMessage.callback = logcallback
@@ -624,9 +635,8 @@ class BaseScriptTestCase(asynctest.TestCase):
                                 "Killed a process that was not properly terminated"
                             )
 
-    async def test_script_schema_process(self):
-        """Test running a script with --schema as a subprocess.
-        """
+    async def test_script_schema_process(self) -> None:
+        """Test running a script with --schema as a subprocess."""
         script_path = os.path.join(self.datadir, "script1")
         index = 1  # index is ignored
         process = await asyncio.create_subprocess_exec(
