@@ -1,6 +1,6 @@
 # This file is part of ts_salobj.
 #
-# Developed for the LSST Telescope and Site Systems.
+# Developed for the Rubin Observatory Telescope and Site System.
 # This product includes software developed by the LSST Project
 # (https://www.lsst.org).
 # See the COPYRIGHT file at the top-level directory of this distribution
@@ -19,48 +19,30 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
+import asyncio
 import datetime
 import getpass
 import os
 import random
 import socket
-import time
 import unittest
 
-import asynctest
-
 import astropy.time
 from astropy.coordinates import Angle
 import astropy.units as u
 import numpy as np
+import pytest
 
+from lsst.ts import utils
 from lsst.ts import salobj
-import lsst.ts.salobj.base
 
 
-def alternate_tai_from_utc_unix(utc_unix):
-    """Compute TAI in unix seconds given UTC in unix seconds.
-
-    Parameters
-    ----------
-    utc_unix : `float`
-
-    This is not the implementation in base.py for two reasons:
-
-    * It is too slow; it slows tests/test_speed.py by a factor of 8.
-    * It blocks while downloading a leap second table.
-    """
-    ap_time = astropy.time.Time(utc_unix, scale="utc", format="unix")
-    return ap_time.tai.mjd * salobj.SECONDS_PER_DAY - salobj.MJD_MINUS_UNIX_SECONDS
-
-
-class BasicsTestCase(asynctest.TestCase):
-    def setUp(self):
+class BasicsTestCase(unittest.IsolatedAsyncioTestCase):
+    def setUp(self) -> None:
         salobj.set_random_lsst_dds_partition_prefix()
 
-    async def test_assert_raises_ack_error(self):
-        """Test the assertRaisesAckError function.
-        """
+    async def test_assert_raises_ack_error(self) -> None:
+        """Test the assertRaisesAckError function."""
         async with salobj.Domain() as domain:
             salinfo = salobj.SalInfo(domain, "Test", index=1)
             private_seqNum = 5
@@ -131,7 +113,7 @@ class BasicsTestCase(asynctest.TestCase):
                     ),
                 )
 
-    async def test_ack_error_repr(self):
+    async def test_ack_error_repr(self) -> None:
         """Test AckError.__str__ and AckError.__repr__"""
         async with salobj.Domain() as domain:
             salinfo = salobj.SalInfo(domain, "Test", index=1)
@@ -154,30 +136,29 @@ class BasicsTestCase(asynctest.TestCase):
             for item in ("AckError", msg, private_seqNum, ack, error, result):
                 self.assertIn(str(item), repr_err)
 
-    def test_astropy_time_from_tai_unix(self):
+    def test_astropy_time_from_tai_unix(self) -> None:
         # Check the function at a leap second transition,
         # since that is likely to cause problems
         unix_time0 = datetime.datetime.fromisoformat("2017-01-01").timestamp()
         for dt in (-1, -0.5, -0.1, 0, 0.1, 1):
             with self.subTest(dt=dt):
                 utc_unix = unix_time0 + dt
-                tai_unix = salobj.tai_from_utc(utc_unix)
-                astropy_time1 = salobj.astropy_time_from_tai_unix(tai_unix)
-                self.assertIsInstance(astropy_time1, astropy.time.Time)
-                self.assertEqual(astropy_time1.scale, "tai")
-                tai_unix_round_trip1 = salobj.tai_from_utc(astropy_time1)
-                self.assertAlmostEqual(tai_unix, tai_unix_round_trip1, delta=1e-6)
+                tai_unix = utils.tai_from_utc(utc_unix)
+                with self.assertWarns(DeprecationWarning):
+                    astropy_time1 = salobj.astropy_time_from_tai_unix(tai_unix)
+                astropy_time2 = utils.astropy_time_from_tai_unix(tai_unix)
+                assert astropy_time1 == astropy_time2
 
-    async def test_get_opensplice_version(self):
+    async def test_get_opensplice_version(self) -> None:
         ospl_version = salobj.get_opensplice_version()
         self.assertRegex(ospl_version, r"^\d+\.\d+\.\d+")
 
-    async def test_get_user_host(self):
+    async def test_get_user_host(self) -> None:
         expected_user_host = getpass.getuser() + "@" + socket.getfqdn()
         user_host = salobj.get_user_host()
         self.assertEqual(expected_user_host, user_host)
 
-    async def test_long_ack_result(self):
+    async def test_long_ack_result(self) -> None:
         async with salobj.Domain() as domain:
             salinfo = salobj.SalInfo(domain, "Test", index=1)
             ack = salobj.SalRetCode.CMD_FAILED
@@ -209,17 +190,19 @@ class BasicsTestCase(asynctest.TestCase):
             self.assertEqual(ackcmd.ack, ack)
             self.assertEqual(ackcmd.error, error)
 
-    def test_set_random_lsst_dds_domain(self):
+    def test_set_random_lsst_dds_domain(self) -> None:
         """Test that set_random_lsst_dds_domain is a deprecated
         alias for set_random_lsst_dds_partition_prefix.
         """
         old_prefix = os.environ["LSST_DDS_PARTITION_PREFIX"]
-        with self.assertWarns(DeprecationWarning):
+        with self.assertWarnsRegex(
+            DeprecationWarning, "Use set_random_lsst_dds_partition_prefix"
+        ):
             salobj.set_random_lsst_dds_domain()
         new_prefix = os.environ["LSST_DDS_PARTITION_PREFIX"]
         self.assertNotEqual(old_prefix, new_prefix)
 
-    def test_set_random_lsst_dds_partition_prefix(self):
+    def test_set_random_lsst_dds_partition_prefix(self) -> None:
         random.seed(42)
         NumToTest = 1000
         names = set()
@@ -228,10 +211,11 @@ class BasicsTestCase(asynctest.TestCase):
             name = os.environ.get("LSST_DDS_PARTITION_PREFIX")
             self.assertTrue(name)
             names.add(name)
+            self.assertNotIn(".", name)  # type: ignore
         # any duplicate names will reduce the size of names
         self.assertEqual(len(names), NumToTest)
 
-    def test_modify_environ(self):
+    def test_modify_environ(self) -> None:
         rng = np.random.default_rng(seed=45)
         original_environ = os.environ.copy()
         n_to_delete = 3
@@ -248,7 +232,7 @@ class BasicsTestCase(asynctest.TestCase):
             new_key0: "bar",
             new_key1: None,
         }
-        with salobj.modify_environ(**kwargs):
+        with self.assertWarns(DeprecationWarning), salobj.modify_environ(**kwargs):
             for name, value in kwargs.items():
                 if value is None:
                     self.assertNotIn(name, os.environ)
@@ -265,12 +249,14 @@ class BasicsTestCase(asynctest.TestCase):
         for bad_value in (3, 1.23, True, False):
             with self.assertRaises(RuntimeError):
                 bad_kwargs = kwargs.copy()
-                bad_kwargs[new_key1] = bad_value
-                with salobj.modify_environ(**bad_kwargs):
+                bad_kwargs[new_key1] = bad_value  # type: ignore
+                with self.assertWarns(DeprecationWarning), salobj.modify_environ(
+                    **bad_kwargs
+                ):
                     pass
             self.assertEqual(os.environ, original_environ)
 
-    async def test_domain_attr(self):
+    async def test_domain_attr(self) -> None:
         async with salobj.Domain() as domain:
             self.assertEqual(domain.origin, os.getpid())
 
@@ -285,7 +271,7 @@ class BasicsTestCase(asynctest.TestCase):
             self.assertFalse(domain.event_qos_set.volatile)
             self.assertTrue(domain.telemetry_qos_set.volatile)
 
-    def test_index_generator(self):
+    def test_index_generator(self) -> None:
         with self.assertRaises(ValueError):
             salobj.index_generator(1, 1)  # imin >= imax
         with self.assertRaises(ValueError):
@@ -310,7 +296,7 @@ class BasicsTestCase(asynctest.TestCase):
         values = [next(gen) for i in range(len(expected_values))]
         self.assertEqual(values, expected_values)
 
-    def test_name_to_name_index(self):
+    def test_name_to_name_index(self) -> None:
         for name, expected_result in (
             ("Script", ("Script", 0)),
             ("Script:0", ("Script", 0)),
@@ -332,7 +318,8 @@ class BasicsTestCase(asynctest.TestCase):
                 with self.assertRaises(ValueError):
                     salobj.name_to_name_index(bad_name)
 
-    def check_tai_from_utc(self, utc_ap):
+    # TODO DM-31660: Remove this test of deprecated code
+    def check_tai_from_utc(self, utc_ap: astropy.time.Time) -> None:
         """Check tai_from_utc at a specific UTC date.
 
         Parameters
@@ -340,55 +327,46 @@ class BasicsTestCase(asynctest.TestCase):
         utc_ap : `astropy.time.Time`
             UTC date as an astropy time.
         """
-        tai = salobj.tai_from_utc(utc_ap.utc.unix)
+        tai = utils.tai_from_utc_unix(utc_ap.utc.unix)
+        with self.assertWarns(DeprecationWarning):
+            tai0 = salobj.tai_from_utc_unix(utc_ap.utc.unix)
+        assert tai == tai0
 
-        tai_alt = alternate_tai_from_utc_unix(utc_ap.utc.unix)
-        self.assertAlmostEqual(tai, tai_alt, delta=1e-6)
+        with self.assertWarns(DeprecationWarning):
+            tai1 = salobj.tai_from_utc(utc_ap.utc.unix)
+        self.assertAlmostEqual(tai, tai1, delta=1e-6)
 
-        tai2 = salobj.tai_from_utc(utc_ap.utc.iso, format="iso")
+        with self.assertWarns(DeprecationWarning):
+            tai2 = salobj.tai_from_utc(utc_ap.utc.iso, format="iso")
         self.assertAlmostEqual(tai, tai2, delta=1e-6)
 
-        tai3 = salobj.tai_from_utc(utc_ap.utc.iso, format=None)
+        with self.assertWarns(DeprecationWarning):
+            tai3 = salobj.tai_from_utc(utc_ap.utc.iso, format=None)
         self.assertAlmostEqual(tai, tai3, delta=1e-6)
 
-        tai4 = salobj.tai_from_utc(utc_ap.utc.mjd, format="mjd")
+        with self.assertWarns(DeprecationWarning):
+            tai4 = salobj.tai_from_utc(utc_ap.utc.mjd, format="mjd")
         self.assertAlmostEqual(tai, tai4, delta=1e-6)
 
         tai_mjd = (tai + salobj.MJD_MINUS_UNIX_SECONDS) / salobj.SECONDS_PER_DAY
         tai_mjd_ap = astropy.time.Time(tai_mjd, scale="tai", format="mjd")
-        tai5 = salobj.tai_from_utc(tai_mjd_ap)
+        with self.assertWarns(DeprecationWarning):
+            tai5 = salobj.tai_from_utc(tai_mjd_ap)
         self.assertAlmostEqual(tai, tai5, delta=1e-6)
 
         tai_iso_ap = astropy.time.Time(utc_ap.tai.iso, scale="tai", format="iso")
-        tai6 = salobj.tai_from_utc(tai_iso_ap)
+        with self.assertWarns(DeprecationWarning):
+            tai6 = salobj.tai_from_utc(tai_iso_ap)
         self.assertAlmostEqual(tai, tai6, delta=1e-6)
 
-        tai7 = salobj.tai_from_utc(utc_ap)
+        with self.assertWarns(DeprecationWarning):
+            tai7 = salobj.tai_from_utc(utc_ap)
         self.assertAlmostEqual(tai, tai7, delta=1e-6)
 
-    def test_leap_second_table(self):
-        """Check that the leap second table is set and an update is scheduled.
-        """
-        self.assertIsNotNone(lsst.ts.salobj.base._LEAP_SECOND_TABLE)
-        update_timer = lsst.ts.salobj.base._LEAP_SECOND_TABLE_UPDATE_TIMER
-        self.assertTrue(update_timer.is_alive())
-        self.assertTrue(update_timer.daemon)
-        self.assertIs(
-            update_timer.function, lsst.ts.salobj.base._update_leap_second_table
-        )
-        update_margin = (
-            lsst.ts.salobj.base._LEAP_SECOND_TABLE_UPDATE_MARGIN_DAYS
-            * salobj.SECONDS_PER_DAY
-        )
-        current_duration = (
-            time.time() - lsst.ts.salobj.base._LEAP_SECOND_TABLE[-1][0] - update_margin
-        )
-        self.assertGreater(update_timer.interval, current_duration)
-
-    def test_tai_from_utc(self):
-        """Test tai_from_utc.
-        """
-        # Check tai_from_utc at leap second transition just before 2017-01-01
+    # TODO DM-31660: Remove this test of deprecated code
+    def test_tai_from_utc(self) -> None:
+        """Test tai_from_utc."""
+        # Check tai_from_utc near leap second transition at UTC = 2017-01-01
         # when leap seconds went from 36 to 37.
         utc0_ap = astropy.time.Time("2017-01-01", scale="utc", format="iso")
         for utc_ap in (
@@ -400,144 +378,136 @@ class BasicsTestCase(asynctest.TestCase):
             (utc0_ap + 1 * u.second),
         ):
             with self.subTest(utc_ap=utc_ap):
-                self.check_tai_from_utc(utc_ap=utc_ap)
+                tai1 = utils.tai_from_utc(utc_ap)
+                with self.assertWarns(DeprecationWarning):
+                    tai2 = salobj.tai_from_utc(utc_ap)
+                assert tai1 == tai2
 
-        # Test values near the limits, which are at 1972-01-01T00:00:00
-        # and one day before the current leap second table expires.
-        min_utc_unix = astropy.time.Time("1972-01-01", scale="utc", format="iso").unix
-        max_utc_unix = (
-            lsst.ts.salobj.base._LEAP_SECOND_TABLE[-1][0] - salobj.SECONDS_PER_DAY
-        )
-        min_tai_unix = salobj.tai_from_utc(min_utc_unix)
-        self.assertAlmostEqual(min_tai_unix, min_utc_unix + 10)
-        max_tai_unix = salobj.tai_from_utc(max_utc_unix)
-        # Final value of TAI-UTC in the table.
-        # Note that the last entry in the table has TAI-UTC = None.
-        final_tai_minus_utc = lsst.ts.salobj.base._LEAP_SECOND_TABLE[-2][1]
-        self.assertAlmostEqual(max_tai_unix, max_utc_unix + final_tai_minus_utc)
-        with self.assertRaises(ValueError):
-            salobj.tai_from_utc(min_utc_unix - 0.001)
-        with self.assertRaises(ValueError):
-            salobj.tai_from_utc(max_utc_unix + 0.001)
-
-    def test_current_tai(self):
-        utc0 = time.time()
-        tai0 = salobj.tai_from_utc(utc0)
-        # salobj.base.current_tai_from_utc uses tai_from_utc,
-        # so it should give the same answer.
-        tai1 = salobj.base.current_tai_from_utc()
-        # salobj.current_tai uses the system TAI clock, if available.
-        # This gives the correct answer (if your operating system
-        # is correctly configured) and can differ from the answer given by
-        # tai_from_utc by as much as a second on the day of a leap second.
-        tai2 = salobj.current_tai()
-        print(f"tai1-tai0={tai1-tai0:0.4f}")
-        # The difference should be much less than 0.1
-        # but pytest can introduce unexpected delays.
-        self.assertLess(abs(tai1 - tai0), 0.1)
-        self.assertGreaterEqual(tai1, tai0)
-        # The difference between the value returned by current_tai
-        # and current_tai_from_utc
-        self.assertLess(abs(tai2 - tai0), 1.1)
-        self.assertIs(type(tai0), float)
-        self.assertIs(type(tai1), float)
-        self.assertIs(type(tai2), float)
-
-    def test_angle_diff(self):
-        for angle1, angle2, expected_diff in (
-            (5.15, 0, 5.15),
-            (5.21, 359.20, 6.01),
-            (270, -90, 0),
+    # TODO DM-31660: Remove this test of deprecated code
+    def test_utc_from_tai_unix(self) -> None:
+        # Check utc_from_tai_unix near leap second transition at
+        # UTC = 2017-01-01 when leap seconds went from 36 to 37.
+        utc0 = astropy.time.Time("2017-01-01", scale="utc", format="iso").unix
+        tai_minus_utc_after = 37
+        tai0 = utc0 + tai_minus_utc_after
+        # Don't test right at tai0 because roundoff error could cause failure;
+        # utc_from_tai_unix is discontinuous at a leap second tranition.
+        for tai in (
+            tai0 - 0.5 * utils.SECONDS_PER_DAY,
+            tai0 - 1,
+            tai0 - 0.001,
+            tai0 + 0.001,
+            tai0 + 1,
+            tai0 + 0.5 * utils.SECONDS_PER_DAY,
         ):
-            with self.subTest(
-                angle1=angle1, angle2=angle2, expected_diff=expected_diff
-            ):
-                diff = salobj.angle_diff(angle1, angle2)
-                self.assertAlmostEqual(diff.deg, expected_diff)
-                diff = salobj.angle_diff(angle2, angle1)
-                self.assertAlmostEqual(diff.deg, -expected_diff)
-                diff = salobj.angle_diff(Angle(angle1, u.deg), angle2)
-                self.assertAlmostEqual(diff.deg, expected_diff)
-                diff = salobj.angle_diff(angle1, Angle(angle2, u.deg))
-                self.assertAlmostEqual(diff.deg, expected_diff)
-                diff = salobj.angle_diff(Angle(angle1, u.deg), Angle(angle2, u.deg))
-                self.assertAlmostEqual(diff.deg, expected_diff)
+            with self.assertWarns(DeprecationWarning):
+                utc1 = salobj.utc_from_tai_unix(tai)
+            utc2 = utils.utc_from_tai_unix(tai)
+            assert utc1 == utc2
 
-    def test_angle_wrap_center(self):
-        for base_angle, expected_result in (
-            (-180.001, 179.999),
-            (-180, -180),
-            (0, 0),
-            (179.999, 179.999),
-            (180, -180),
+    # TODO DM-31660: Remove this test of deprecated code
+    def test_current_tai(self) -> None:
+        with self.assertWarns(DeprecationWarning):
+            tai0 = salobj.current_tai()
+        tai1 = utils.current_tai()
+        # Leave plenty of slop because time has jitter on macOS Docker.
+        pytest.approx(tai0, tai1, abs=0.2)
+
+    # TODO DM-31660: Remove this test of deprecated code
+    def test_angle_diff(self) -> None:
+        for angle1, angle2 in (
+            (5.15, 0),
+            (5.21, 359.20),
+            (270, -90),
         ):
+            for swap in (False, True):
+                if swap:
+                    angle1, angle2 = angle2, angle1
+                with self.subTest(angle1=angle1, angle2=angle2):
+                    diff1 = utils.angle_diff(angle1, angle2)
+                    with self.assertWarns(DeprecationWarning):
+                        diff2 = salobj.angle_diff(angle1, angle2)
+                    assert diff1 == diff2
+
+    # TODO DM-31660: Remove this test of deprecated code
+    def test_angle_wrap_center(self) -> None:
+        for base_angle in (-180, -180, 0, 179, 180):
             for nwraps in (-2, -1, 0, 1, 2):
-                with self.subTest(
-                    base_angle=base_angle,
-                    expected_result=expected_result,
-                    nwraps=nwraps,
-                ):
+                with self.subTest(base_angle=base_angle, nwraps=nwraps):
                     angle = base_angle + 360 * nwraps
-                    result = salobj.angle_wrap_center(angle)
-                    self.assertAlmostEqual(result.deg, expected_result)
-                    result = salobj.angle_wrap_center(Angle(angle, u.deg))
-                    self.assertAlmostEqual(result.deg, expected_result)
+                    with self.assertWarns(DeprecationWarning):
+                        wrapped1 = salobj.angle_wrap_center(angle)
+                    wrapped2 = utils.angle_wrap_center(angle)
+                    assert wrapped1 == wrapped2
 
-    def test_angle_wrap_nonnegative(self):
-        for base_angle, expected_result in (
-            (-0.001, 359.999),
-            (0, 0),
-            (180, 180),
-            (359.999, 359.999),
-            (360, 0),
-        ):
+    # TODO DM-31660: Remove this test of deprecated code
+    def test_angle_wrap_nonnegative(self) -> None:
+        for base_angle in (-0, 0, 180, 359, 360):
             for nwraps in (-2, -1, 0, 1, 2):
-                with self.subTest(
-                    base_angle=base_angle,
-                    expected_result=expected_result,
-                    nwraps=nwraps,
-                ):
+                with self.subTest(base_angle=base_angle, nwraps=nwraps):
                     angle = base_angle + 360 * nwraps
-                    result = salobj.angle_wrap_nonnegative(angle)
-                    self.assertAlmostEqual(result.deg, expected_result)
-                    result = salobj.angle_wrap_nonnegative(Angle(angle, u.deg))
-                    self.assertAlmostEqual(result.deg, expected_result)
+                    with self.assertWarns(DeprecationWarning):
+                        wrapped1 = salobj.angle_wrap_nonnegative(angle)
+                    wrapped2 = utils.angle_wrap_nonnegative(angle)
+                    assert wrapped1 == wrapped2
 
-    def test_assertAnglesAlmostEqual(self):
+    # TODO DM-31660: Remove this test of deprecated code
+    def test_make_done_future(self) -> None:
+        with self.assertWarns(DeprecationWarning):
+            done_future = salobj.make_done_future()
+            assert isinstance(done_future, asyncio.Future)
+            assert done_future.done()
+            assert done_future.result() is None
+            assert not done_future.cancelled()
+            assert done_future.exception() is None
+
+    # TODO DM-31660: Remove this test of deprecated code
+    def test_assertAnglesAlmostEqual(self) -> None:
         for angle1, angle2 in ((5.15, 5.14), (-0.20, 359.81), (270, -90.1)):
             epsilon = Angle(1e-15, u.deg)
             with self.subTest(angle1=angle1, angle2=angle2):
-                diff = abs(salobj.angle_diff(angle1, angle2))
+                diff = abs(utils.angle_diff(angle1, angle2))
                 bad_diff = diff - epsilon
                 self.assertGreater(bad_diff.deg, 0)
-                with self.assertRaises(AssertionError):
-                    salobj.assertAnglesAlmostEqual(angle1, angle2, bad_diff)
-                with self.assertRaises(AssertionError):
-                    salobj.assertAnglesAlmostEqual(angle1, angle2, bad_diff.deg)
-                with self.assertRaises(AssertionError):
-                    salobj.assertAnglesAlmostEqual(angle2, angle1, bad_diff)
-                with self.assertRaises(AssertionError):
-                    salobj.assertAnglesAlmostEqual(
-                        Angle(angle1, u.deg), angle2, bad_diff
-                    )
-                with self.assertRaises(AssertionError):
-                    salobj.assertAnglesAlmostEqual(
-                        angle1, Angle(angle2, u.deg), bad_diff
-                    )
-                with self.assertRaises(AssertionError):
-                    salobj.assertAnglesAlmostEqual(
-                        Angle(angle1, u.deg), Angle(angle2, u.deg), bad_diff
-                    )
 
-                good_diff = diff + epsilon
-                salobj.assertAnglesAlmostEqual(angle1, angle2, good_diff)
-                salobj.assertAnglesAlmostEqual(angle1, angle2, good_diff.deg)
-                salobj.assertAnglesAlmostEqual(angle2, angle1, good_diff)
-                salobj.assertAnglesAlmostEqual(Angle(angle1, u.deg), angle2, good_diff)
-                salobj.assertAnglesAlmostEqual(angle1, Angle(angle2, u.deg), good_diff)
-                salobj.assertAnglesAlmostEqual(
-                    Angle(angle1, u.deg), Angle(angle2, u.deg), good_diff
-                )
+                for arg1, arg2 in (
+                    (angle1, angle2),
+                    (angle2, angle1),
+                    (Angle(angle1, u.deg), angle2),
+                    (Angle(angle1, u.deg), Angle(angle2, u.deg)),
+                ):
+                    with self.subTest(arg1=arg1, arg2=arg2):
+                        # Test too-large differences
+                        with self.assertRaises(AssertionError):
+                            utils.assert_angles_almost_equal(
+                                angle1=arg1, angle2=arg2, max_diff=bad_diff
+                            )
+                        with self.assertRaises(AssertionError), self.assertWarns(
+                            DeprecationWarning
+                        ):
+                            salobj.assertAnglesAlmostEqual(
+                                arg1, arg2, max_diff=bad_diff
+                            )
+                        with self.assertRaises(AssertionError), self.assertWarns(
+                            DeprecationWarning
+                        ):
+                            salobj.assertAnglesAlmostEqual(
+                                arg1, arg2, max_diff=bad_diff.deg
+                            )
+
+                        # Test acceptable differences
+                        good_diff = diff + epsilon
+                        utils.assert_angles_almost_equal(
+                            angle1=arg1, angle2=arg2, max_diff=good_diff
+                        )
+                        with self.assertWarns(DeprecationWarning):
+                            salobj.assertAnglesAlmostEqual(
+                                arg1, arg2, max_diff=good_diff
+                            )
+                        with self.assertWarns(DeprecationWarning):
+                            salobj.assertAnglesAlmostEqual(
+                                arg1, arg2, max_diff=good_diff.deg
+                            )
 
 
 if __name__ == "__main__":
