@@ -1,6 +1,6 @@
 # This file is part of ts_salobj.
 #
-# Developed for the LSST Telescope and Site Systems.
+# Developed for the Rubin Observatory Telescope and Site System.
 # This product includes software developed by the LSST Project
 # (https://www.lsst.org).
 # See the COPYRIGHT file at the top-level directory of this distribution
@@ -19,9 +19,9 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
+import typing
 import unittest
 
-import asynctest
 import numpy as np
 
 from lsst.ts import salobj
@@ -42,21 +42,21 @@ class ControllerWithDoMethods(salobj.Controller):
         methods.
     """
 
-    def __init__(self, command_names):
+    def __init__(self, command_names: typing.Iterable[str]) -> None:
         index = next(index_gen)
         for name in command_names:
             setattr(self, f"do_{name}", self.mock_do_method)
         super().__init__("Test", index, do_callbacks=True)
 
-    def mock_do_method(self, *args, **kwargs):
+    def mock_do_method(self, *args: typing.Any, **kwargs: typing.Any) -> None:
         pass
 
 
-class ControllerConstructorTestCase(asynctest.TestCase):
-    def setUp(self):
-        salobj.set_random_lsst_dds_domain()
+class ControllerConstructorTestCase(unittest.IsolatedAsyncioTestCase):
+    def setUp(self) -> None:
+        salobj.set_random_lsst_dds_partition_prefix()
 
-    async def test_do_callbacks_false(self):
+    async def test_do_callbacks_false(self) -> None:
         index = next(index_gen)
         async with salobj.Controller("Test", index, do_callbacks=False) as controller:
             command_names = controller.salinfo.command_names
@@ -65,33 +65,36 @@ class ControllerConstructorTestCase(asynctest.TestCase):
                     cmd = getattr(controller, "cmd_" + name)
                     self.assertFalse(cmd.has_callback)
 
-    async def test_do_callbacks_true(self):
+            self.assertEqual(controller.salinfo.identity, f"Test:{index}")
+
+    async def test_do_callbacks_true(self) -> None:
         index = next(index_gen)
-        async with salobj.Domain() as domain:
-            salinfo = salobj.SalInfo(domain=domain, name="Test", index=index)
+        async with salobj.Domain() as domain, salobj.SalInfo(
+            domain=domain, name="Test", index=index
+        ) as salinfo:
             command_names = salinfo.command_names
 
-            # make sure I can build one
-            async with ControllerWithDoMethods(command_names) as good_controller:
-                for cmd_name in command_names:
-                    with self.subTest(cmd_name=cmd_name):
-                        cmd = getattr(good_controller, "cmd_" + cmd_name)
-                        self.assertTrue(cmd.has_callback)
+        # Build a controller and check that callbacks are asigned.
+        async with ControllerWithDoMethods(command_names) as controller:
+            for cmd_name in command_names:
+                with self.subTest(cmd_name=cmd_name):
+                    cmd = getattr(controller, "cmd_" + cmd_name)
+                    self.assertTrue(cmd.has_callback)
 
-            skip_names = salobj.OPTIONAL_COMMAND_NAMES.copy()
-            # do_setLogLevel is provided by Controller
-            skip_names.add("setLogLevel")
-            for missing_name in command_names:
-                if missing_name in skip_names:
-                    continue
-                with self.subTest(missing_name=missing_name):
-                    bad_names = [name for name in command_names if name != missing_name]
-                    with self.assertRaises(TypeError):
-                        ControllerWithDoMethods(bad_names)
+        skip_names = salobj.OPTIONAL_COMMAND_NAMES.copy()
+        # do_setAuthList and do_setLogLevel are provided by Controller
+        skip_names |= {"setAuthList", "setLogLevel"}
+        for missing_name in command_names:
+            if missing_name in skip_names:
+                continue
+            with self.subTest(missing_name=missing_name):
+                bad_names = [name for name in command_names if name != missing_name]
+                with self.assertRaises(TypeError):
+                    ControllerWithDoMethods(bad_names)
 
-            extra_names = list(command_names) + ["extra_command"]
-            with self.assertRaises(TypeError):
-                ControllerWithDoMethods(extra_names)
+        extra_names = list(command_names) + ["extra_command"]
+        with self.assertRaises(TypeError):
+            ControllerWithDoMethods(extra_names)
 
 
 if __name__ == "__main__":
