@@ -1,6 +1,6 @@
 # This file is part of ts_salobj.
 #
-# Developed for the LSST Telescope and Site Systems.
+# Developed for the Rubin Observatory Telescope and Site System.
 # This product includes software developed by the LSST Project
 # (https://www.lsst.org).
 # See the COPYRIGHT file at the top-level directory of this distribution
@@ -20,15 +20,16 @@
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 import unittest
+import unittest.mock
 
 from lsst.ts import salobj
 
 
 class DdsUtilsTestCase(unittest.TestCase):
-    def setUp(self):
-        salobj.set_random_lsst_dds_domain()
+    def setUp(self) -> None:
+        salobj.set_random_lsst_dds_partition_prefix()
 
-    def test_get_dds_version(self):
+    def test_get_dds_version(self) -> None:
         for dds_file, desired_version in (
             ("dds-6.9.181127OSS-py3.7-linux-x86_64.egg/dds.so", "6.9.181127"),
             ("other-6.9.181127OSS-py3.7/dds.so", "6.9.181127"),
