@@ -1,6 +1,6 @@
 # This file is part of ts_salobj.
 #
-# Developed for the LSST Telescope and Site Systems.
+# Developed for the Rubin Observatory Telescope and Site System.
 # This product includes software developed by the LSST Project
 # (https://www.lsst.org).
 # See the COPYRIGHT file at the top-level directory of this distribution
@@ -19,20 +19,28 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
+import asyncio
+import os
 import unittest
 
-import asynctest
 import numpy as np
 
 from lsst.ts import salobj
 
+# Long enough to perform any reasonable operation
+# including starting a CSC or loading a script (seconds)
+STD_TIMEOUT = 60
+
+HISTORY_TIMEOUT_NAME = "LSST_DDS_HISTORYSYNC"
+INITIAL_HISTORY_TIMEOUT = os.environ.get(HISTORY_TIMEOUT_NAME, None)
+
 np.random.seed(47)
 
 index_gen = salobj.index_generator()
 
 
-class RemoteTestCase(asynctest.TestCase):
-    async def test_constructor_include_exclude(self):
+class RemoteTestCase(unittest.IsolatedAsyncioTestCase):
+    async def test_constructor_include_exclude(self) -> None:
         """Test the include and exclude arguments for salobj.Remote."""
 
         index = next(index_gen)
@@ -175,7 +183,9 @@ class RemoteTestCase(asynctest.TestCase):
                     exclude=exclude,
                 )
 
-    def assert_max_history(self, remote, evt_max_history=1, tel_max_history=1):
+    def assert_max_history(
+        self, remote: salobj.Remote, evt_max_history: int = 1
+    ) -> None:
         for evt in [
             getattr(remote, f"evt_{name}") for name in remote.salinfo.event_names
         ]:
@@ -184,30 +194,18 @@ class RemoteTestCase(asynctest.TestCase):
         for tel in [
             getattr(remote, f"tel_{name}") for name in remote.salinfo.telemetry_names
         ]:
-            self.assertEqual(tel.max_history, tel_max_history)
+            self.assertEqual(tel.max_history, 0)
+            self.assertFalse(evt.volatile)
 
-    async def test_default_max_history(self):
-        """Test default evt_max_history and tel_max_history ctor arguments.
-        """
+    async def test_default_max_history(self) -> None:
+        """Test default evt_max_history ctor argument."""
         index = next(index_gen)
         async with salobj.Domain() as domain:
             remote = salobj.Remote(domain=domain, name="Test", index=index)
             self.assert_max_history(remote)
 
-    async def test_tel_max_history(self):
-        """Test non-default tel_max_history Remote constructor argument.
-        """
-        tel_max_history = 9
-        index = next(index_gen)
-        async with salobj.Domain() as domain:
-            remote = salobj.Remote(
-                domain=domain, name="Test", index=index, tel_max_history=tel_max_history
-            )
-            self.assert_max_history(remote, tel_max_history=tel_max_history)
-
-    async def test_evt_max_history(self):
-        """Test non-default evt_max_history Remote constructor argument.
-        """
+    async def test_evt_max_history(self) -> None:
+        """Test non-default evt_max_history Remote constructor argument."""
         evt_max_history = 0
         index = next(index_gen)
         async with salobj.Domain() as domain:
@@ -216,7 +214,7 @@ class RemoteTestCase(asynctest.TestCase):
             )
             self.assert_max_history(remote, evt_max_history=evt_max_history)
 
-    async def test_start_false(self):
+    async def test_start_false(self) -> None:
         """Test the start argument of Remote."""
 
         index = next(index_gen)
@@ -224,6 +222,39 @@ class RemoteTestCase(asynctest.TestCase):
             remote = salobj.Remote(domain=domain, name="Test", index=index, start=False)
             self.assertFalse(hasattr(remote, "start_task"))
 
+    @unittest.skipIf(
+        INITIAL_HISTORY_TIMEOUT is not None and float(INITIAL_HISTORY_TIMEOUT) < 0,
+        f"${HISTORY_TIMEOUT_NAME}={INITIAL_HISTORY_TIMEOUT} < 0",
+    )
+    async def test_negative_lsst_dds_historysync(self) -> None:
+        """Test that setting LSST_DDS_HISTORYSYNC < 0
+        prevents waiting for historical data.
+
+        This setting applies to SalInfo, not Remote,
+        but it requires some topics in order to be tested,
+        and Remote provides topics.
+        """
+        index = next(index_gen)
+        async with salobj.Domain() as domain:
+            # Make a normal remote that waits for historical data.
+            remote1 = salobj.Remote(domain=domain, name="Test", index=index)
+            await asyncio.wait_for(remote1.start_task, timeout=STD_TIMEOUT)
+            self.assertGreater(len(remote1.salinfo.wait_history_isok), 0)
+
+            # Make a remote that does not wait for historical data
+            # by defining the history timeout env variable < 0.
+            os.environ[HISTORY_TIMEOUT_NAME] = "-1"
+            try:
+                remote2 = salobj.Remote(domain=domain, name="Test", index=index)
+                await asyncio.wait_for(remote2.start_task, timeout=STD_TIMEOUT)
+            finally:
+                if INITIAL_HISTORY_TIMEOUT is None:
+                    del os.environ[HISTORY_TIMEOUT_NAME]
+                else:
+                    os.environ[HISTORY_TIMEOUT_NAME] = INITIAL_HISTORY_TIMEOUT
+
+            self.assertEqual(len(remote2.salinfo.wait_history_isok), 0)
+
 
 if __name__ == "__main__":
     unittest.main()
