@@ -1,6 +1,6 @@
 # This file is part of ts_salobj.
 #
-# Developed for the LSST Telescope and Site Systems.
+# Developed for the Rubin Observatory Telescope and Site System.
 # This product includes software developed by the LSST Project
 # (https://www.lsst.org).
 # See the COPYRIGHT file at the top-level directory of this distribution
@@ -20,23 +20,26 @@
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 import asyncio
+import enum
 import logging
+import os
 import unittest
 
-import asynctest
-
+from lsst.ts import utils
 from lsst.ts import salobj
 
 # Long enough to perform any reasonable operation
 # including starting a CSC or loading a script (seconds)
 STD_TIMEOUT = 60
 
+index_gen = salobj.index_generator()
 
-class SalInfoTestCase(asynctest.TestCase):
-    def setUp(self):
-        salobj.set_random_lsst_dds_domain()
 
-    async def test_salinfo_constructor(self):
+class SalInfoTestCase(unittest.IsolatedAsyncioTestCase):
+    def setUp(self) -> None:
+        salobj.set_random_lsst_dds_partition_prefix()
+
+    async def test_salinfo_constructor(self) -> None:
         with self.assertRaises(TypeError):
             salobj.SalInfo(domain=None, name="Test")
 
@@ -44,8 +47,14 @@ class SalInfoTestCase(asynctest.TestCase):
             with self.assertRaises(RuntimeError):
                 salobj.SalInfo(domain=domain, name="invalid_component_name")
 
-            salinfo = salobj.SalInfo(domain=domain, name="Test")
+            for invalid_index in (1.1, "one"):
+                with self.assertRaises(TypeError):
+                    salobj.SalInfo(domain=domain, name="Test", index=invalid_index)
+
+            index = next(index_gen)
+            salinfo = salobj.SalInfo(domain=domain, name="Test", index=index)
             self.assertEqual(salinfo.name, "Test")
+            self.assertEqual(salinfo.index, index)
             self.assertFalse(salinfo.start_task.done())
             self.assertFalse(salinfo.done_task.done())
             self.assertFalse(salinfo.started)
@@ -69,11 +78,21 @@ class SalInfoTestCase(asynctest.TestCase):
             self.assertTrue(salinfo.started)
             salinfo.assert_started()
 
-    async def test_salinfo_attributes(self):
-        async with salobj.Domain() as domain:
-            salinfo = salobj.SalInfo(domain=domain, name="Test")
+            # Test enum index
+            class SalIndex(enum.IntEnum):
+                ONE = 1
+                TWO = 2
 
-            self.assertEqual(salinfo.name_index, f"Test:{salinfo.index}")
+            salinfo = salobj.SalInfo(domain=domain, name="Script", index=SalIndex.ONE)
+            assert isinstance(salinfo.index, SalIndex)
+            assert salinfo.index == SalIndex.ONE
+
+    async def test_salinfo_attributes(self) -> None:
+        async with salobj.Domain() as domain:
+            index = next(index_gen)
+            salinfo = salobj.SalInfo(domain=domain, name="Test", index=index)
+
+            self.assertEqual(salinfo.name_index, f"Test:{index}")
 
             # expected_commands omits a few commands that TestCsc
             # does not support, but that are in generics.
@@ -122,12 +141,18 @@ class SalInfoTestCase(asynctest.TestCase):
                 sorted(expected_sal_topic_names), list(salinfo.sal_topic_names)
             )
 
+            self.assertEqual(salinfo.identity, domain.user_host)
+
             # Check that the name_index for a non-indexed component
             # has no :index suffix.
             salinfo.indexed = False
             self.assertEqual(salinfo.name_index, "Test")
 
-    async def test_salinfo_metadata(self):
+            self.assertEqual(
+                salinfo.partition_prefix, os.environ["LSST_DDS_PARTITION_PREFIX"]
+            )
+
+    async def test_salinfo_metadata(self) -> None:
         """Test some of the metadata in SalInfo.
 
         The main tests of the IDL parser are elsewhere.
@@ -156,7 +181,46 @@ class SalInfoTestCase(asynctest.TestCase):
                 )
             )
 
-    async def test_log_level(self):
+    async def test_lsst_dds_domain_fallback(self) -> None:
+        # Pick a value for LSST_DDS_DOMAIN that does not match
+        # LSST_DDS_PARTITION_PREFIX.
+        new_lsst_dds_domain = os.environ["LSST_DDS_PARTITION_PREFIX"] + "Extra text"
+
+        # Test that LSST_DDS_DOMAIN is ignored, with a warning,
+        # if both that and LSST_DDS_PARTITION_PREFIX are defined.
+        with utils.modify_environ(LSST_DDS_DOMAIN=new_lsst_dds_domain):
+            async with salobj.Domain() as domain:
+                with self.assertWarnsRegex(
+                    DeprecationWarning,
+                    "LSST_DDS_PARTITION_PREFIX instead of deprecated",
+                ):
+                    salinfo = salobj.SalInfo(domain=domain, name="Test", index=1)
+                self.assertEqual(
+                    salinfo.partition_prefix, os.environ["LSST_DDS_PARTITION_PREFIX"]
+                )
+
+        # Test that LSST_DDS_DOMAIN is used, with a warning,
+        # if LSST_DDS_PARTITION_PREFIX is not defined.
+        with utils.modify_environ(
+            LSST_DDS_PARTITION_PREFIX=None, LSST_DDS_DOMAIN=new_lsst_dds_domain
+        ):
+            async with salobj.Domain() as domain:
+                with self.assertWarnsRegex(
+                    DeprecationWarning,
+                    "LSST_DDS_PARTITION_PREFIX not defined; using deprecated fallback",
+                ):
+                    salinfo = salobj.SalInfo(domain=domain, name="Test", index=1)
+                self.assertEqual(salinfo.partition_prefix, new_lsst_dds_domain)
+
+    async def test_lsst_dds_domain_required(self) -> None:
+        # Delete LSST_DDS_PARTITION_PREFIX and (if defined) LSST_DDS_DOMAIN.
+        # This should raise an error.
+        with utils.modify_environ(LSST_DDS_PARTITION_PREFIX=None, LSST_DDS_DOMAIN=None):
+            async with salobj.Domain() as domain:
+                with self.assertRaises(RuntimeError):
+                    salobj.SalInfo(domain=domain, name="Test", index=1)
+
+    async def test_log_level(self) -> None:
         """Test that log level is decreased (verbosity increased) to INFO."""
         log = logging.getLogger()
         log.setLevel(logging.WARNING)
@@ -179,19 +243,24 @@ class SalInfoTestCase(asynctest.TestCase):
                 for salinfo in salinfos:
                     await salinfo.close()
 
-    async def test_make_ack_cmd(self):
+    async def test_make_ack_cmd(self) -> None:
         async with salobj.Domain() as domain:
             salinfo = salobj.SalInfo(domain=domain, name="Test")
 
             # Use all defaults
             seqNum = 55
             ack = salobj.SalRetCode.CMD_COMPLETE
-            ackcmd = salinfo.makeAckCmd(private_seqNum=seqNum, ack=ack)
+            ackcmd = salinfo.make_ackcmd(private_seqNum=seqNum, ack=ack)
             self.assertEqual(ackcmd.private_seqNum, seqNum)
             self.assertEqual(ackcmd.ack, ack)
             self.assertEqual(ackcmd.error, 0)
             self.assertEqual(ackcmd.result, "")
 
+            warning_regex = "makeAckCmd is deprecated"
+            with self.assertWarnsRegex(DeprecationWarning, warning_regex):
+                ackcmd2 = salinfo.makeAckCmd(private_seqNum=seqNum, ack=ack)
+            self.assertEqual(ackcmd.get_vars(), ackcmd2.get_vars())
+
             # Specify an error code and result
             for truncate_result in (False, True):
                 with self.subTest(truncate_result=truncate_result):
@@ -199,7 +268,7 @@ class SalInfoTestCase(asynctest.TestCase):
                     ack = salobj.SalRetCode.CMD_FAILED
                     error = 127
                     result = "why not?"
-                    ackcmd = salinfo.makeAckCmd(
+                    ackcmd = salinfo.make_ackcmd(
                         private_seqNum=seqNum,
                         ack=ack,
                         error=error,
@@ -211,20 +280,30 @@ class SalInfoTestCase(asynctest.TestCase):
                     self.assertEqual(ackcmd.error, error)
                     self.assertEqual(ackcmd.result, result)
 
+                    with self.assertWarnsRegex(DeprecationWarning, warning_regex):
+                        ackcmd2 = salinfo.makeAckCmd(
+                            private_seqNum=seqNum,
+                            ack=ack,
+                            error=error,
+                            result=result,
+                            truncate_result=truncate_result,
+                        )
+                    self.assertEqual(ackcmd.get_vars(), ackcmd2.get_vars())
+
             # Test behavior with too-long result strings
             seqNum = 27
             ack = salobj.SalRetCode.CMD_FAILED
             error = 127
             result = "a" * (salobj.MAX_RESULT_LEN + 5)
             with self.assertRaises(ValueError):
-                salinfo.makeAckCmd(
+                salinfo.make_ackcmd(
                     private_seqNum=seqNum,
                     ack=ack,
                     error=error,
                     result=result,
                     truncate_result=False,
                 )
-            ackcmd = salinfo.makeAckCmd(
+            ackcmd = salinfo.make_ackcmd(
                 private_seqNum=seqNum,
                 ack=ack,
                 error=error,
@@ -237,19 +316,6 @@ class SalInfoTestCase(asynctest.TestCase):
             self.assertNotEqual(ackcmd.result, result)
             self.assertEqual(ackcmd.result, result[0 : salobj.MAX_RESULT_LEN])
 
-    async def test_no_commands(self):
-        """Test a SAL component with no commands.
-        """
-        async with salobj.Domain() as domain:
-            salinfo = salobj.SalInfo(domain=domain, name="LOVE")
-            self.assertEqual(salinfo.command_names, ())
-            with self.assertRaises(RuntimeError):
-                salinfo.AckCmdType
-            with self.assertRaises(RuntimeError):
-                salinfo.makeAckCmd(
-                    private_seqNum=1, ack=salobj.SalRetCode.CMD_COMPLETE, result="Done"
-                )
-
 
 if __name__ == "__main__":
     unittest.main()
