@@ -1,6 +1,6 @@
 # This file is part of ts_salobj.
 #
-# Developed for the LSST Telescope and Site Systems.
+# Developed for the Rubin Observatory Telescope and Site System.
 # This product includes software developed by the LSST Project
 # (https://www.lsst.org).
 # See the COPYRIGHT file at the top-level directory of this distribution
@@ -25,8 +25,6 @@ import time
 import unittest
 import warnings
 
-import asynctest
-
 from lsst.ts import salobj
 
 try:
@@ -44,40 +42,43 @@ SAL__CMD_COMPLETE = 303
 index_gen = salobj.index_generator()
 
 
-class SALPYTestCase(asynctest.TestCase):
-    def setUp(self):
-        salobj.set_random_lsst_dds_domain()
+class SALPYTestCase(unittest.IsolatedAsyncioTestCase):
+    def setUp(self) -> None:
+        salobj.set_random_lsst_dds_partition_prefix()
         self.datadir = pathlib.Path(__file__).resolve().parent / "data"
         self.index = next(index_gen)
 
     @unittest.skipIf(SALPY_Test is None, "Could not import SALPY_Test")
-    async def test_salpy_remote_salobj_controller(self):
+    async def test_salpy_remote_salobj_controller(self) -> None:
         await self.check_salpy_remote("minimal_salobj_controller.py")
 
     @unittest.skipIf(SALPY_Test is None, "Could not import SALPY_Test")
-    async def test_salpy_remote_salpy_controller(self):
+    async def test_salpy_remote_salpy_controller(self) -> None:
         await self.check_salpy_remote("minimal_salpy_controller.py")
 
-    async def check_salpy_remote(self, exec_name):
+    async def check_salpy_remote(self, exec_name: str) -> None:
+        # Create at least the scalars telemetry reader before creating
+        # the subprocess, to be sure the remote sees telemetry from the
+        # subprocess (telemetry is volatile, so has no historical data).
+        print(f"Remote: create SALPY object and topics with index={self.index}")
+        t0 = time.monotonic()
+        manager = SALPY_Test.SAL_Test(self.index)
+        manager.setDebugLevel(0)
+        manager.salEventSub("Test_logevent_logLevel")
+        manager.salTelemetrySub("Test_scalars")
+        manager.salCommand("Test_command_setLogLevel")
+        dt = time.monotonic() - t0
+        print(f"Remote: took {dt:0.2f} seconds to create topics")
+
         print(f"Remote: start {exec_name} in a subprocess")
         script_path = self.datadir / exec_name
         process = await asyncio.create_subprocess_exec(
             str(script_path), str(self.index), str(INITIAL_LOG_LEVEL)
         )
 
-        manager = None
         try:
-            print(f"Remote: create SALPY remote with index={self.index}")
-            t0 = time.monotonic()
-            manager = SALPY_Test.SAL_Test(self.index)
-            manager.setDebugLevel(0)
-            manager.salEventSub("Test_logevent_logLevel")
-            manager.salTelemetrySub("Test_scalars")
-            manager.salCommand("Test_command_setLogLevel")
-            dt = time.monotonic() - t0
-            print(f"Remote: took {dt:0.2f} seconds to create topics")
 
-            async def get_logLevel():
+            async def get_logLevel() -> salobj.BaseDdsDataType:
                 data = SALPY_Test.Test_logevent_logLevelC()
                 while True:
                     retcode = manager.getEvent_logLevel(data)
@@ -87,7 +88,7 @@ class SALPYTestCase(asynctest.TestCase):
                         raise RuntimeError(f"Unexpected return code {retcode}")
                     await asyncio.sleep(0.01)
 
-            async def get_scalars():
+            async def get_scalars() -> salobj.BaseDdsDataType:
                 data = SALPY_Test.Test_scalarsC()
                 while True:
                     retcode = manager.getNextSample_scalars(data)
@@ -97,7 +98,7 @@ class SALPYTestCase(asynctest.TestCase):
                         raise RuntimeError(f"Unexpected return code {retcode}")
                     await asyncio.sleep(0.01)
 
-            async def send_setLogLevel(level):
+            async def send_setLogLevel(level: int) -> salobj.AckCmdDataType:
                 done_ack_codes = frozenset(
                     (
                         SALPY_Test.SAL__CMD_ABORTED,
@@ -121,6 +122,7 @@ class SALPYTestCase(asynctest.TestCase):
                 while True:
                     response_id = manager.getResponse_setLogLevel(ack_data)
                     if response_id == cmd_id:
+                        self.assertEqual(ack_data.identity, f"Test:{self.index}")
                         if ack_data.ack == SALPY_Test.SAL__CMD_COMPLETE:
                             return ack_data
                         elif ack_data.ack in done_ack_codes:
@@ -129,24 +131,20 @@ class SALPYTestCase(asynctest.TestCase):
                             )
                     await asyncio.sleep(0.01)
 
-            print("Remote: wait for initial logLevel")
+            print("Remote: wait for initial logLevel event")
             data = await asyncio.wait_for(get_logLevel(), timeout=STD_TIMEOUT)
             print(f"Remote: read logLevel.level={data.level}")
             self.assertEqual(data.level, INITIAL_LOG_LEVEL)
-            print("Remote: wait for initial scalars")
-            data = await asyncio.wait_for(get_scalars(), timeout=STD_TIMEOUT)
-            print(f"Remote: read scalars.int0={data.int0}")
-            self.assertEqual(data.int0, INITIAL_LOG_LEVEL)
 
             for level in (10, 52, 0):
-                print(f"Remote: sending setLogLevel(level={level})")
+                print(f"Remote: send setLogLevel(level={level}) command")
                 await asyncio.wait_for(send_setLogLevel(level), timeout=STD_TIMEOUT)
 
-                print("Remote: wait for logLevel")
+                print("Remote: wait for logLevel event")
                 data = await asyncio.wait_for(get_logLevel(), timeout=STD_TIMEOUT)
                 print(f"Remote: read logLevel.level={data.level}")
                 self.assertEqual(data.level, level)
-                print("Remote: wait for scalars")
+                print("Remote: wait for scalars telemetry")
                 data = await asyncio.wait_for(get_scalars(), timeout=STD_TIMEOUT)
                 print(f"Remote: read scalars.int0={data.int0}")
                 self.assertEqual(data.int0, level)
